@@ -5,7 +5,7 @@
 /*
 
     This file is part of the extensible drawing editor Ipe.
-    Copyright (c) 1993-2020 Otfried Cheong
+    Copyright (c) 1993-2023 Otfried Cheong
 
     Ipe is free software; you can redistribute it and/or modify it
     under the terms of the GNU General Public License as published by
@@ -37,10 +37,6 @@
 #include <algorithm>
 #include <memory>
 
-#ifdef IPESTRICT
-#include "ipeosx.h"
-#endif
-
 // --------------------------------------------------------------------
 
 #undef assert
@@ -67,14 +63,14 @@ namespace ipe {
 
   //! Ipelib version.
   /*! \ingroup base */
-  const int IPELIB_VERSION = 70217;
+  const int IPELIB_VERSION = 70228;
 
   //! Oldest readable file format version.
   /*! \ingroup base */
   const int OLDEST_FILE_FORMAT = 70000;
   //! Current file format version.
   /*! \ingroup base */
-  const int FILE_FORMAT = 70216;
+  const int FILE_FORMAT = 70218;
 
   enum class LatexType { Default, Pdftex, Xetex, Luatex };
 
@@ -244,6 +240,8 @@ namespace ipe {
     std::shared_ptr<std::vector<char>> iData;
   };
 
+  extern void ipeDebugBuffer(Buffer data, int maxsize);
+
   // --------------------------------------------------------------------
 
   class Stream {
@@ -372,6 +370,8 @@ namespace ipe {
     static int runLatex(String dir, LatexType engine, String docname) noexcept;
     static double toDouble(String s);
     static int toNumber(String s, int &iValue, double &dValue);
+    static String spiroVersion();
+    static String gslVersion();
   };
 
   // --------------------------------------------------------------------
