@@ -5,7 +5,7 @@
 /*
 
     This file is part of the extensible drawing editor Ipe.
-    Copyright (c) 1993-2019 Otfried Cheong
+    Copyright (c) 1993-2023 Otfried Cheong
 
     Ipe is free software; you can redistribute it and/or modify it
     under the terms of the GNU General Public License as published by
@@ -43,6 +43,7 @@ namespace ipe {
 
   class BitmapFinder;
   class PdfResources;
+  class Latex;
 
   //! Flags for saving Ipe documents (to PDF)
   class SaveFlag {
@@ -60,8 +61,6 @@ namespace ipe {
   enum class FileFormat {
     Xml,  //!< Save as XML
     Pdf,  //!< Save as PDF
-    Eps,  //!< Encapsulated Postscript (loading only)
-    Ipe5,  //!< Ancient Ipe format
     Unknown //!< Unknown file format
   };
 
@@ -73,10 +72,12 @@ namespace ipe {
       String iAuthor;
       String iSubject;
       String iKeywords;
+      String iLanguage;
       String iPreamble;
       LatexType iTexEngine { LatexType::Default };
       bool iFullScreen { false };
       bool iNumberPages { false };
+      bool iSequentialText { false };
       //! Date/time in PDF style "D:20010428191400" format.
       String iCreated;
       String iModified;
@@ -151,10 +152,11 @@ namespace ipe {
 
     //! Error codes returned by RunLatex.
     enum { ErrNone, ErrNoText, ErrNoDir, ErrWritingSource,
-	   ErrOldPdfLatex, ErrRunLatex, ErrLatex, ErrLatexOutput };
+	   ErrRunLatex, ErrLatex, ErrLatexOutput };
     int runLatex(String docname, String &logFile);
     int runLatex(String docname);
-
+    int runLatexAsync(String docname, String &texLog, Latex **pConverter);
+    int completeLatexRun(Latex *converter);
 
   private:
     std::vector<Page *> iPages;
