@@ -5,7 +5,7 @@
 /*
 
     This file is part of the extensible drawing editor Ipe.
-    Copyright (c) 1993-2019 Otfried Cheong
+    Copyright (c) 1993-2023 Otfried Cheong
 
     Ipe is free software; you can redistribute it and/or modify it
     under the terms of the GNU General Public License as published by
@@ -79,13 +79,16 @@ namespace ipe {
     inline TTransformations transformations() const { return iTransformations; }
     void setTransformations(TTransformations trans);
 
-    void setMatrix(const Matrix &matrix);
+    virtual void setMatrix(const Matrix &matrix);
     //! Return transformation matrix.
     inline const Matrix &matrix() const { return iMatrix; }
 
     virtual bool setAttribute(Property prop, Attribute value);
     virtual Attribute getAttribute(Property prop) const noexcept;
 
+    void setCustom(Attribute value);
+    Attribute getCustom() const noexcept;
+
     //! Save the object in XML format.
     virtual void saveAsXml(Stream &stream, String layer) const = 0;
 
@@ -139,6 +142,7 @@ namespace ipe {
 
   protected:
     Matrix iMatrix;
+    Attribute iCustom;
     TPinned iPinned : 8;
     TTransformations iTransformations : 8;
   };
