@@ -5,7 +5,7 @@
 /*
 
     This file is part of the extensible drawing editor Ipe.
-    Copyright (c) 1993-2019 Otfried Cheong
+    Copyright (c) 1993-2023 Otfried Cheong
 
     Ipe is free software; you can redistribute it and/or modify it
     under the terms of the GNU General Public License as published by
@@ -44,6 +44,8 @@ namespace ipe {
 
   class Page {
   public:
+    enum class SnapMode { Never, Visible, Always };
+
     explicit Page();
 
     static Page *basic();
@@ -53,21 +55,19 @@ namespace ipe {
     void saveSelection(Stream &stream) const;
 
     //! Return number of layers.
-    inline size_t countLayers() const noexcept { return iLayers.size(); }
+    inline int countLayers() const noexcept { return iLayers.size(); }
     //! Return name of layer \a index.
     inline String layer(int index) const noexcept {
       return iLayers[index].iName; }
 
     //! Is layer \a i locked?
-    inline bool isLocked(int i) const noexcept {
-      return !!(iLayers[i].iFlags & ELocked); }
-    //! Does layer \a i have active snapping?
-    inline bool hasSnapping(int i) const noexcept {
-      return !(iLayers[i].iFlags & ENoSnapping); }
+    inline bool isLocked(int i) const noexcept { return iLayers[i].locked; }
+    //! Does layer \a i have snapping?
+    inline SnapMode snapping(int i) const noexcept { return iLayers[i].snapMode; }
     bool objSnapsInView(int objNo, int view) const noexcept;
 
     void setLocked(int i, bool flag);
-    void setSnapping(int i, bool flag);
+    void setSnapping(int i, SnapMode mode);
 
     void moveLayer(int index, int newIndex);
     int findLayer(String name) const;
@@ -75,10 +75,13 @@ namespace ipe {
     void addLayer();
     void removeLayer(String name);
     void renameLayer(String oldName, String newName);
+    void setLayerData(int index, String data);
+    //! Return layer data.
+    inline String layerData(int index) const { return iLayers[index].iData; }
 
     //! Return number of views.
-    inline size_t countViews() const { return iViews.size(); }
-    size_t countMarkedViews() const;
+    inline int countViews() const { return iViews.size(); }
+    int countMarkedViews() const;
     //! Return effect of view.
     inline Attribute effect(int index) const { return iViews[index].iEffect; }
     void setEffect(int index, Attribute sym);
@@ -86,6 +89,10 @@ namespace ipe {
     inline String active(int index) const { return iViews[index].iActive; }
     //! Set active layer of view.
     void setActive(int index, String name);
+    //! Return name of view.
+    String viewName(int index) const noexcept { return iViews[index].iName; }
+    //! Set name of view.
+    void setViewName(int index, String name) noexcept { iViews[index].iName = name; }
     //! Return if view is marked.
     bool markedView(int index) const { return iViews[index].iMarked; }
     //! Set if view is marked.
@@ -97,6 +104,9 @@ namespace ipe {
 
     int findView(String viewNumberOrName) const;
 
+    const AttributeMap &viewMap(int index) const { return iViews[index].iAttributeMap; }
+    void setViewMap(int index, const AttributeMap &map);
+
     //! Is \a layer visible in \a view?
     inline bool visible(int view, int layer) const {
       return iLayers[layer].iVisible[view]; }
@@ -104,6 +114,10 @@ namespace ipe {
     inline bool objectVisible(int view, int objno) const {
       return iLayers[layerOf(objno)].iVisible[view]; }
 
+    std::vector<Matrix> layerMatrices(int view) const;
+    void clearLayerMatrices(int view) { iViews[view].iLayerMatrices.clear(); }
+    void setLayerMatrix(int view, int layer, const Matrix &m);
+
     void setVisible(int view, String layer, bool vis);
 
     //! Return title of this page.
@@ -125,7 +139,7 @@ namespace ipe {
     void setNotes(String notes);
 
     //! Return number of objects on the page.
-    inline size_t count() const { return iObjects.size(); }
+    inline int count() const { return iObjects.size(); }
 
     void objectsPerLayer(std::vector<int> &objcounts) const;
 
@@ -167,19 +181,24 @@ namespace ipe {
     void ensurePrimarySelection();
 
   private:
-    enum { ELocked = 0x01, ENoSnapping = 0x02 };
-
     struct SLayer {
     public:
       SLayer(String name);
     public:
       String iName;
-      int iFlags;
+      String iData;
+      bool locked;
+      SnapMode snapMode;
       // Invariant: iVisible.size() == iViews.size()
       std::vector<bool> iVisible;
     };
     typedef std::vector<SLayer> LayerSeq;
 
+    struct SLayerMatrix {
+      String iLayer;
+      Matrix iMatrix;
+    };
+
     struct SView {
     public:
       SView() { iEffect = Attribute::NORMAL(); }
@@ -187,6 +206,9 @@ namespace ipe {
       Attribute iEffect;
       String iActive;
       bool iMarked;
+      String iName;
+      AttributeMap iAttributeMap;
+      std::vector<SLayerMatrix> iLayerMatrices;
     };
     typedef std::vector<SView> ViewSeq;
 
