@@ -5,7 +5,7 @@
 /*
 
     This file is part of the extensible drawing editor Ipe.
-    Copyright (c) 1993-2019 Otfried Cheong
+    Copyright (c) 1993-2023 Otfried Cheong
 
     Ipe is free software; you can redistribute it and/or modify it
     under the terms of the GNU General Public License as published by
@@ -32,6 +32,8 @@
 #ifndef IPEPAINTER_H
 #define IPEPAINTER_H
 
+#include <list>
+
 #include "ipeattributes.h"
 #include "ipestyle.h"
 #include "ipebitmap.h"
@@ -54,6 +56,7 @@ namespace ipe {
       Color iSymFill;
       Fixed iSymPen;
       Fixed iOpacity;
+      Fixed iStrokeOpacity;
       Attribute iTiling;
       Attribute iGradient;
     };
@@ -62,6 +65,9 @@ namespace ipe {
     Painter(const Cascade *style);
     virtual ~Painter();
 
+    void setAttributeMap(const AttributeMap *map);
+    Attribute lookup(Kind kind, Attribute sym) const;
+
     void transform(const Matrix &m);
     void untransform(TTransformations trans);
     void translate(const Vector &v);
@@ -96,6 +102,7 @@ namespace ipe {
     void setSymFill(Attribute color);
     void setSymPen(Attribute wid);
     void setOpacity(Attribute opaq);
+    void setStrokeOpacity(Attribute opaq);
     void setTiling(Attribute til);
     void setGradient(Attribute grad);
 
@@ -126,6 +133,8 @@ namespace ipe {
     inline Fixed symPen() const { return iState.back().iSymPen; }
     //! Return current opacity.
     inline Fixed opacity() const { return iState.back().iOpacity; }
+    //! Return current stroke opacity.
+    inline Fixed strokeOpacity() const { return iState.back().iStrokeOpacity; }
     //! Return current tiling.
     inline Attribute tiling() const { return iState.back().iTiling; }
     //! Return current gradient fill.
@@ -157,6 +166,7 @@ namespace ipe {
     std::list<State> iState;
     std::list<Matrix> iMatrix;
     const Cascade *iCascade;
+    const AttributeMap *iAttributeMap;
     int iInPath;
   };
 
