@@ -4,7 +4,7 @@
 /*
 
     This file is part of the extensible drawing editor Ipe.
-    Copyright (c) 1993-2020 Otfried Cheong
+    Copyright (c) 1993-2023 Otfried Cheong
 
     Ipe is free software; you can redistribute it and/or modify it
     under the terms of the GNU General Public License as published by
@@ -38,7 +38,7 @@
 using namespace ipe;
 using namespace ipelua;
 
-// --------------------------------------------------------------------
+// --------------------------------------------------------------------------------
 
 const char * const AppUiBase::selectorNames[] =
   { "stroke", "fill", "pen", "dashstyle", "textsize", "markshape", "symbolsize",
@@ -147,6 +147,9 @@ void AppUiBase::buildMenus()
   addItem(EFileMenu, "Save", "save");
   addItem(EFileMenu, "Save as", "save_as");
   addItem(EFileMenu);
+  startSubMenu(EFileMenu, "Recent files", ESubmenuRecentFiles);
+  iRecentFileMenu = endSubMenu();
+  addItem(EFileMenu);
   addItem(EFileMenu, "Export as PNG", "export_png");
   addItem(EFileMenu, "Export as EPS", "export_eps");
   addItem(EFileMenu, "Export as SVG", "export_svg");
@@ -178,6 +181,7 @@ void AppUiBase::buildMenus()
   addItem(EEditMenu, "Just behind", "behind");
   addItem(EEditMenu, "Duplicate", "duplicate");
   addItem(EEditMenu, "Select all", "select_all");
+  addItem(EEditMenu, "Deselect all", "deselect_all");
   addItem(EEditMenu);
   addItem(EEditMenu, "Pick properties", "pick_properties");
   addItem(EEditMenu, "Apply properties", "apply_properties");
@@ -397,9 +401,7 @@ void AppUiBase::buildMenus()
   addItem(EHelpMenu, "Show &configuration", "show_configuration");
   addItem(EHelpMenu, "Show &libraries", "show_libraries");
   addItem(EHelpMenu, "&Ipelet information", "about_ipelets");
-#ifdef IPECURL
   addItem(EHelpMenu, "Enable online Latex-compilation", "cloud_latex");
-#endif
 
   lua_getglobal(L, "prefs");
   lua_getfield(L, -1, "developer");
@@ -646,6 +648,15 @@ void AppUiBase::luaBookmarkSelected(int index)
   luacall(L, 2, 0);
 }
 
+void AppUiBase::luaRecentFileSelected(String name)
+{
+  lua_rawgeti(L, LUA_REGISTRYINDEX, iModel);
+  lua_getfield(L, -1, "recent_file");
+  lua_insert(L, -2); // method, model
+  push_string(L, name);
+  luacall(L, 2, 0);
+}
+
 // --------------------------------------------------------------------
 
 static String stripMark(Attribute mark)
@@ -788,4 +799,43 @@ void AppUiBase::setAttributes(const AllAttributes &all, Cascade *sheet)
   setCheckMark("fillrule", Attribute(iAll.iFillRule));
 }
 
+int AppUiBase::ipeIcon(String action)
+{
+  if (!ipeIcons) {
+    String fname = ipeIconDirectory() + "icons.ipe";
+    if (!Platform::fileExists(fname))
+      return -1;
+    ipeIconsDark.reset(Document::loadWithErrorReport(fname.z()));
+    ipeIcons.reset(new Document(*ipeIconsDark));
+    ipeIcons->cascade()->remove(0);
+  }
+  return ipeIcons->findPage(action);
+}
+
+int AppUiBase::readImage(lua_State *L, String fn)
+{
+  // check if it is perhaps a JPEG file
+  FILE *f = Platform::fopen(fn.z(), "rb");
+  bool jpeg = false;
+  if (f != nullptr) {
+    jpeg = (std::fgetc(f) == 0xff && std::fgetc(f) == 0xd8);
+    fclose(f);
+  }
+
+  ipeDebug("Dropping file %s (jpeg: %d)", fn.z(), jpeg);
+
+  Vector dpi;
+  const char *errmsg;
+  Bitmap bm = jpeg ? Bitmap::readJpeg(fn.z(), dpi, errmsg) : Bitmap::readPNG(fn.z(), dpi, errmsg);
+
+  if (bm.isNull())
+    return 0;
+
+  ipe::Rect r(Vector::ZERO, Vector(bm.width(), bm.height()));
+  Image *img = new Image(r, bm);
+  push_object(L, img);
+  return 1;
+}
+
 // --------------------------------------------------------------------
+
