@@ -5,7 +5,7 @@
 /*
 
     This file is part of the extensible drawing editor Ipe.
-    Copyright (c) 1993-2020 Otfried Cheong
+    Copyright (c) 1993-2023 Otfried Cheong
 
     Ipe is free software; you can redistribute it and/or modify it
     under the terms of the GNU General Public License as published by
@@ -62,7 +62,7 @@ typedef NSMenu *MENUHANDLE;
 
 using namespace ipe;
 
-constexpr int COPYRIGHT_YEAR = 2020;
+constexpr int COPYRIGHT_YEAR = 2023;
 
 #define IPEABSOLUTE "<absolute>"
 
@@ -70,6 +70,7 @@ class AppUiBase;
 extern WINID check_winid(lua_State *L, int i);
 extern void push_winid(lua_State *L, WINID win);
 extern String ipeIconDirectory();
+extern Buffer ipeIcon(String action, int size);
 extern AppUiBase *createAppUi(lua_State *L0, int model);
 
 // --------------------------------------------------------------------
@@ -90,6 +91,7 @@ public:
   enum { ESubmenuGridSize = 1000, ESubmenuAngleSize,
 	 ESubmenuTextStyle, ESubmenuLabelStyle,
 	 ESubmenuSelectLayer, ESubmenuMoveLayer,
+	 ESubmenuRecentFiles,
 	 ESubmenuFin };
 
 public:
@@ -103,7 +105,9 @@ public:
 
   void luaShowPathStylePopup(Vector v);
   void luaBookmarkSelected(int index);
+  void luaRecentFileSelected(String name);
   inline void setInkMode(bool ink) { isInkMode = ink; }
+  static int readImage(lua_State *L, String fn);
 
 public:  // What platforms must implement:
   virtual WINID windowId() = 0;
@@ -117,7 +121,8 @@ public:  // What platforms must implement:
   virtual void setWindowCaption(bool mod, const char *s) = 0;
   virtual void setNotes(String notes) = 0;
   virtual void explain(const char *s, int t) = 0;
-  virtual void showWindow(int width, int height) = 0;
+  virtual void showWindow(int width, int height, int x, int y, const Color & pathViewColor) = 0;
+  virtual void setFullScreen(int mode) = 0;
   virtual void action(String name) = 0;
   virtual void setActionsEnabled(bool mode) = 0;
   virtual void setMouseIndicator(const char *s) = 0;
@@ -131,6 +136,8 @@ public:  // What platforms must implement:
   // Only used on Windows to compute shortcuts:
   virtual int actionInfo(lua_State *L) const;
 
+  virtual void setRecentFileMenu(const std::vector<String> & names) = 0;
+
 protected:   // What platforms must implement:
   virtual void addRootMenu(int id, const char *name) = 0;
   // if title == 0, add separator
@@ -170,10 +177,13 @@ protected:
   void showMarksInCombo(const Cascade *sheet);
   void setAttribute(int sel, Attribute a);
 
+  int ipeIcon(String action);
+
 protected:
   lua_State *L;
   int iModel;  // reference to Lua model
 
+  MENUHANDLE iRecentFileMenu;
   MENUHANDLE iSelectLayerMenu;
   MENUHANDLE iMoveToLayerMenu;
   MENUHANDLE iTextStyleMenu;
@@ -198,6 +208,9 @@ protected:
   bool isMiniEdit;
   bool iLeftDockFloats;
   bool isInkMode;
+
+  std::unique_ptr<Document> ipeIcons;
+  std::unique_ptr<Document> ipeIconsDark;
 };
 
 // --------------------------------------------------------------------
