@@ -3,7 +3,7 @@
 /*
 
     This file is part of the extensible drawing editor Ipe.
-    Copyright (c) 1993-2020 Otfried Cheong
+    Copyright (c) 1993-2023 Otfried Cheong
 
     Ipe is free software; you can redistribute it and/or modify it
     under the terms of the GNU General Public License as published by
@@ -33,6 +33,11 @@
 
 #include "ipelua.h"
 
+#include "ipethumbs.h"
+
+#include <cairo.h>
+#include <cairo-quartz.h>
+
 #include <cstdio>
 #include <cstdlib>
 
@@ -130,6 +135,8 @@ static const char * const touchbar_customization_label[] = {
       self.action = @selector(selectAll:);
     else if ([_ipeAction isEqualToString:@"fullscreen"])
       self.action = @selector(toggleFullScreen:);
+    else if ([_ipeAction isEqualToString:@"recent_file"])
+      self.action = @selector(ipeRecentFileAction:);
     // make sure these work when no window exists
     else if ([@[@"show_configuration", @"new_window", @"open",
 		@"manual", @"about_ipelets"]
@@ -151,18 +158,6 @@ static NSImage *colorIcon(Color color, int pixels = 20)
   return colorIcon(red, green, blue, pixels);
 }
 
-static NSImage *loadIcon(String action)
-{
-  String dir = ipeIconDirectory();
-  String fname = dir + action + ".tiff";
-  if (!Platform::fileExists(fname)) {
-    fname = dir + action + ".png";
-    if (!Platform::fileExists(fname))
-      return nil;
-  }
-  return [[NSImage alloc] initWithContentsOfFile:I2N(fname)];
-}
-
 // --------------------------------------------------------------------
 
 #if MAC_OS_X_VERSION_MAX_ALLOWED < 101202
@@ -201,6 +196,8 @@ static NSImage *loadIcon(String action)
   }
   if (item.action == @selector(toggleSnapbarShown:))
     return appui->validateMenuItem(item, @"snapbar");
+  if (item.action == @selector(togglePropertiesShown:))
+    return appui->validateMenuItem(item, @"propertiespanel");
   if (item.tag >= AppUi::ESubmenuGridSize && item.tag < AppUi::ESubmenuFin)
     appui->fillDynamicSubmenu(item);
   return YES;
@@ -286,6 +283,11 @@ static NSImage *loadIcon(String action)
   appui->toggleSnapbarShown();
 }
 
+- (void) togglePropertiesShown:(id) sender
+{
+  appui->togglePropertiesShown();
+}
+
 // --------------------------------------------------------------------
 
 - (void) pathViewAttributeChanged:(String) attr
@@ -328,13 +330,15 @@ static NSImage *loadIcon(String action)
 {
   NSToolbarItem *t =
     [[NSToolbarItem alloc] initWithItemIdentifier:itemIdentifier];
-  t.image = loadIcon(N2I(itemIdentifier));
+  t.image = appui->loadIcon(N2I(itemIdentifier), false, 32);
   if (![itemIdentifier hasPrefix:@"mode_"])
     t.visibilityPriority = NSToolbarItemVisibilityPriorityLow;
   t.action = @selector(ipeToolbarAction:);
   IpeAction *s = appui->findAction(itemIdentifier);
-  if (s)
+  if (s) {
     t.toolTip = s.title;
+    t.label = s.title;
+  }
   return t;
 }
 
@@ -389,8 +393,6 @@ static NSImage *loadIcon(String action)
 }
 
 #if MAC_OS_X_VERSION_MAX_ALLOWED >= 101202
-#pragma clang diagnostic push
-#pragma clang diagnostic ignored "-Wdeprecated-declarations"
 - (NSTouchBar *) makeTouchBar
 {
   NSTouchBar *bar = [[NSTouchBar alloc] init];
@@ -415,7 +417,7 @@ static NSImage *loadIcon(String action)
       auto title = touchbar_titles[tag];
       NSButton *b;
       if (title == nullptr) {
-	auto image = loadIcon(touchbar_action[tag]);
+	auto image = appui->loadIcon(touchbar_action[tag], true);
 	b = [NSButton buttonWithTitle:identifier image:image target:self action:@selector(ipeTouchBar:)];
 	b.buttonType = NSPushOnPushOffButton;
 	b.imagePosition = NSImageOnly;
@@ -432,7 +434,6 @@ static NSImage *loadIcon(String action)
   }
   return nil;
 }
-#pragma clang diagnostic pop
 #endif
 
 - (void) ipeTouchBar:(id) sender
@@ -636,6 +637,11 @@ AppUi::AppUi(lua_State *L0, int model)
   [tb setSizeMode:NSToolbarSizeModeSmall];
   [iWindow setToolbar:tb];
 
+#if MAC_OS_X_VERSION_MAX_ALLOWED >= 110000
+  if (@available(macOS 11, *))
+    [iWindow setToolbarStyle:NSWindowToolbarStyleExpanded];
+#endif
+
   [iWindow setWindowController:iDelegate];
 
   iContent = [[NSView alloc] initWithFrame:contentRect];
@@ -689,11 +695,9 @@ AppUi::AppUi(lua_State *L0, int model)
   layout(iSnapBar, iView, "l=l");
   layout(iPropertiesBox, iContent, "l=l");
   layout(iPropertiesBox, iContent, "t=t");
-  layout(iPropertiesBox, iView, "r=l");
   layout(iView, iContent, "r=r");
   layout(iView, iStatus, "b=t");
   layout(iLayerBox, iContent, "l=l");
-  layout(iLayerBox, iView, "r=l");
   layout(iLayerBox, iPropertiesBox, "t=b");
   layout(iLayerBox, iStatus, "b=t");
   layout(iStatus, iContent, "l=l");
@@ -715,6 +719,12 @@ AppUi::AppUi(lua_State *L0, int model)
   iViewToTop = layout(iView, iContent, "t=t", 0.0, 1.0, NO);
   iViewToSnapBar.active = YES;
 
+  iViewToProperties = layout(iPropertiesBox, iView, "r=l", 0.0, 1.0, NO);
+  iViewToLayers = layout(iLayerBox, iView, "r=l", 0.0, 1.0, NO);
+  iViewToLeft = layout(iView, iContent, "l=l", 0.0, 1.0, NO);
+  iViewToProperties.active = YES;
+  iViewToLayers.active = YES;
+
   [iStatus
     setContentHuggingPriority:NSLayoutPriorityDefaultLow
 	       forOrientation:NSLayoutConstraintOrientationHorizontal];
@@ -1065,6 +1075,11 @@ BOOL AppUi::validateMenuItem(NSMenuItem *item, NSString *name)
       @"Hide Snap Toolbar";
     return YES;
   }
+  if ([name isEqualToString:@"propertiespanel"]) {
+    item.title = [iPropertiesBox isHidden] ? @"Show Properties Panel" :
+      @"Hide Properties Panel";
+    return YES;
+  }
   IpeAction *s = findAction(name);
   if (s) {
     if ([name isEqualToString:@"toggle_notes"])
@@ -1125,9 +1140,24 @@ void AppUi::fillDynamicSubmenu(NSMenuItem *item)
       ++count;
       [sm addItem:mi];
     }
+  } else if (item.tag == ESubmenuRecentFiles) {
+    for (auto & name : iRecentFiles) {
+      NSMenuItem *mi = [[IpeMenuItem alloc]
+			   initWithTitle:I2N(name)
+			       ipeAction:@"recent_file"
+			   keyEquivalent:@""];
+      [sm addItem:mi];
+    }
   }
 }
 
+void AppUi::setRecentFileMenu(const std::vector<String> & names)
+{
+  iRecentFiles.clear();
+  for (const auto & name : names)
+    iRecentFiles.push_back(name);
+}
+
 void AppUi::toggleSnapbarShown()
 {
   iSnapBar.hidden = ![iSnapBar isHidden];
@@ -1141,6 +1171,23 @@ void AppUi::toggleSnapbarShown()
   iCanvas->update();
 }
 
+void AppUi::togglePropertiesShown()
+{
+  auto hidden = ![iPropertiesBox isHidden];
+  iPropertiesBox.hidden = hidden;
+  iLayerBox.hidden = hidden;
+  if (hidden) {
+    iViewToProperties.active = NO;
+    iViewToLayers.active = NO;
+    iViewToLeft.active = YES;
+  } else {
+    iViewToProperties.active = YES;
+    iViewToLayers.active = YES;
+    iViewToLeft.active = NO;
+  }
+  iCanvas->update();
+}
+
 // --------------------------------------------------------------------
 
 void AppUi::setNotes(String notes)
@@ -1285,10 +1332,8 @@ void AppUi::setActionState(const char *name, bool value)
     }
     // update touch bar if it exists
 #if MAC_OS_X_VERSION_MAX_ALLOWED >= 101202
-#pragma clang diagnostic push
-#pragma clang diagnostic ignored "-Wdeprecated-declarations"
-    if ([iDelegate respondsToSelector:@selector(touchBar)]) {
-      id tbar = [iDelegate performSelector:@selector(touchBar)];
+    if (@available(macOS 10.12, *)) {
+      id tbar = [iDelegate touchBar];
       if (tbar != nil) {
 	auto bar = (NSTouchBar *) tbar;
 	auto item = [bar itemForIdentifier:a];
@@ -1298,7 +1343,6 @@ void AppUi::setActionState(const char *name, bool value)
 	}
       }
     }
-#pragma clang diagnostic push
 #endif
   } else if ([a isEqualToString:@"shift_key"]) {
     int mod = s.state ? CanvasBase::EShift : 0;
@@ -1342,18 +1386,23 @@ void AppUi::setWindowCaption(bool mod, const char *s)
   [iWindow setTitle:[[NSString alloc] initWithUTF8String:s]];
 }
 
-void AppUi::showWindow(int width, int height)
+void AppUi::showWindow(int width, int height, int x, int y, const Color & pathViewColor)
 {
   if (width > 0 && height > 0) {
     NSRect e = [[NSScreen mainScreen] frame];
     auto wd = e.size.width - width;
     auto hd = e.size.height - height;
-    NSRect winr = NSMakeRect(0.5 * wd, 0.5 * hd, width, height);
+    NSRect winr = NSMakeRect(x < 0 ? 0.5 * wd : x, y < 0 ? 0.5 * hd : y, width, height);
     [iWindow setFrame:winr display:YES];
   }
   [iWindow makeKeyAndOrderFront:iWindow];
 }
 
+void AppUi::setFullScreen(int mode)
+{
+  // not implemented, as OSX only provides toggleFullScreen
+}
+
 // show for t milliseconds, or permanently if t == 0
 void AppUi::explain(const char *s, int t)
 {
@@ -1419,15 +1468,61 @@ void AppUi::closeWindow()
 
 // --------------------------------------------------------------------
 
+NSImage *AppUi::createIcon(int pno, int size, bool touchBar)
+{
+  NSSize s = NSMakeSize(size, size);
+  int w = 2 * size;  // icons are square
+  int h = 2 * size;
+  return [NSImage imageWithSize:s
+			flipped:YES
+		 drawingHandler:^(NSRect rect) {
+      auto col = [[NSColor textColor] colorUsingColorSpace:[NSColorSpace genericGrayColorSpace]];
+      bool dark = touchBar || (col.whiteComponent > 0.5);
+      const Document *doc = dark ? ipeIconsDark.get() : ipeIcons.get();
+      Thumbnail thumbs(doc, w);
+      thumbs.setTransparent(true);
+      Buffer b = thumbs.render(doc->page(pno), 0);
+      cairo_surface_t *image =
+	cairo_image_surface_create_for_data((uint8_t *) b.data(),
+					    CAIRO_FORMAT_ARGB32,
+					    w, h, 4 * w);
+      CGContextRef ctx = [[NSGraphicsContext currentContext] CGContext];
+      cairo_surface_t *surface =
+	cairo_quartz_surface_create_for_cg_context(ctx, w, h);
+      cairo_t *cr = cairo_create(surface);
+      cairo_set_source_surface(cr, image, 0.0, 0.0);
+      cairo_matrix_t matrix;
+      cairo_matrix_init_scale(&matrix, 2.0, 2.0);
+      cairo_pattern_set_matrix(cairo_get_source(cr), &matrix);
+      cairo_paint(cr);
+      cairo_destroy(cr);
+      cairo_surface_finish(surface);
+      cairo_surface_destroy(surface);
+      cairo_surface_destroy(image);
+      return YES;
+    }];
+}
+
+NSImage *AppUi::loadIcon(String action, bool touchBar, int size)
+{
+  int pno = ipeIcon(action);
+  if (pno >= 0)
+    return createIcon(pno, size, touchBar);
+  else
+    // fallback if no icon has been defined
+    return colorIcon(0.8, 0.5, 0.7, size-2);
+}
+
+// --------------------------------------------------------------------
+
 int AppUi::clipboard(lua_State *L)
 {
   NSPasteboard *pasteBoard = [NSPasteboard generalPasteboard];
   bool allow_bitmap = lua_toboolean(L, 2);
   if (allow_bitmap) {
-    NSArray *arr = [pasteBoard readObjectsForClasses:@[[NSImage class]]
-					     options:@{ } ];
-    if (arr && [arr count] > 0) {
-      CGImageRef bitmap = [arr[0] CGImageForProposedRect:nil context:nil hints:nil];
+    NSBitmapImageRep *rep = (NSBitmapImageRep *)[NSBitmapImageRep imageRepWithPasteboard:pasteBoard];
+    if (rep) {
+      CGImageRef bitmap = [rep CGImage];
       int w = CGImageGetWidth(bitmap);
       int h = CGImageGetHeight(bitmap);
       Buffer data(w * h * sizeof(uint32_t));
