@@ -4,7 +4,7 @@
 /*
 
     This file is part of the extensible drawing editor Ipe.
-    Copyright (c) 1993-2020 Otfried Cheong
+    Copyright (c) 1993-2023 Otfried Cheong
 
     Ipe is free software; you can redistribute it and/or modify it
     under the terms of the GNU General Public License as published by
@@ -34,6 +34,8 @@
 
 #include "ipelua.h"
 
+#include "ipethumbs.h"
+
 #include <cstdio>
 #include <cstdlib>
 
@@ -48,46 +50,58 @@
 #include <QToolTip>
 #include <QDialogButtonBox>
 #include <QClipboard>
+#include <QMimeData>
+#include <QWindow>
+#include <QScreen>
+
+#include <sys/types.h>
+#include <sys/stat.h>
 
 using namespace ipe;
 using namespace ipelua;
 
 // --------------------------------------------------------------------
 
-#define SIZE 16
-
 inline QSize adapt_size(const QSize &size, int factor)
 {
   return QSize(factor * size.width() / 100, factor * size.height() / 100);
 }
 
-static bool add_bitmap(QIcon &icon, String base, const char *ext)
+QIcon AppUi::prefsIcon(String name)
 {
-  if (Platform::fileExists(base + ext)) {
-    icon.addFile(QIpe(base + ext));
-    return true;
+  if (name == "ipe") {
+    String fname = ipeIconDirectory() + "icon_128x128.png";
+    if (Platform::fileExists(fname))
+      return QIcon(QIpe(fname));
   }
-  return false;
-}
 
-QIcon prefsIcon(String name)
-{
-  String dir = ipeIconDirectory();
-  String base = dir + name;
-  if (Platform::fileExists(base + ".png")) {
-    QIcon icon(QIpe(base + ".png"));
-    if (!add_bitmap(icon, base, "@2x.png"))
-      icon.addPixmap(icon.pixmap(QSize(24, 24)).scaledToHeight(48));
-    add_bitmap(icon, base, "_32x32.png");
-    add_bitmap(icon, base, "_32x32@2x.png");
-    add_bitmap(icon, base, "128.png");  // for application icon
-    add_bitmap(icon, base, "512.png");  // for application icon
-    /*
-    auto l = icon.availableSizes();
-    for (int i = 0; i < l.size(); ++i)
-      ipeDebug("Icon %s has size %d x %d", name.z(), l[i].width(), l[i].height());
-    */
-    return icon;
+  String svgdir = Platform::latexDirectory() + "/icons/";
+  if (!Platform::fileExists(svgdir) && mkdir(svgdir.z(), 0700) != 0)
+    return QIcon();
+
+  if (name == "ipe") {
+    String png = svgdir + "icon_128x128.png";
+    if (!Platform::fileExists(png)) {
+      String fname = ipeIconDirectory() + "ipe_logo.ipe";
+      std::unique_ptr<Document> doc(Document::loadWithErrorReport(fname.z()));
+      if (!doc || doc->runLatex(fname) != Document::ErrNone)
+	return QIcon();
+      Thumbnail thumbs(doc.get(), 128);
+      thumbs.setNoCrop(true);
+      thumbs.saveRender(Thumbnail::EPNG, png.z(), doc->page(0), 0, 4.47);
+    }
+    return QIcon(QIpe(png));
+  }
+
+  String svgname = svgdir + name + ".svg";
+  int pno = ipeIcon(name);
+  if (pno >= 0) {
+    bool dark = QGuiApplication::palette().text().color().value() > 128;
+    Document *doc = dark ? ipeIconsDark.get() : ipeIcons.get();
+    Thumbnail thumbs(doc, 22);
+    thumbs.setNoCrop(true);
+    thumbs.saveRender(Thumbnail::ESVG, svgname.z(), doc->page(pno), 0, 1.0);
+    return QIcon(QIpe(svgname));
   } else
     return QIcon();
 }
@@ -95,13 +109,13 @@ QIcon prefsIcon(String name)
 QPixmap AppUi::prefsPixmap(String name)
 {
   QIcon icon = prefsIcon(name);
-  int size = iUiScale * 24 / 100;
+  int size = uiscale(24);
   return icon.pixmap(QSize(size, size));
 }
 
 QIcon AppUi::prefsColorIcon(Color color)
 {
-  int size = iUiScale * SIZE / 100;
+  int size = uiscale(16);
   QPixmap pixmap(size, size);
   pixmap.fill(QIpe(color));
   return QIcon(pixmap);
@@ -236,16 +250,11 @@ static void set_toolbar_size(QToolBar *bar, int factor)
 
 // --------------------------------------------------------------------
 
-#if QT_VERSION < 0x050000
-AppUi::AppUi(lua_State *L0, int model, Qt::WFlags f)
-#else
 AppUi::AppUi(lua_State *L0, int model, Qt::WindowFlags f)
-#endif
-    : QMainWindow(nullptr, f), AppUiBase(L0, model)
+  : QMainWindow(nullptr, f), AppUiBase(L0, model)
 {
-  setWindowIcon(prefsIcon("ipe"));
+  qApp->setWindowIcon(prefsIcon("ipe"));
   QMainWindow::setAttribute(Qt::WA_DeleteOnClose);
-  qApp->setAttribute(Qt::AA_UseHighDpiPixmaps);
   setDockOptions(AnimatedDocks); // do not allow stacking properties and layers
 
   Canvas *canvas = new Canvas(this);
@@ -257,13 +266,12 @@ AppUi::AppUi(lua_State *L0, int model, Qt::WindowFlags f)
   addToolBarBreak();
   iObjectTools = addToolBar("Objects");
 
-  ipeDebug("devicePixelRatio: %g", qApp->devicePixelRatio());
   set_toolbar_size(iEditTools, iToolbarScale);
   set_toolbar_size(iSnapTools, iToolbarScale);
   set_toolbar_size(iObjectTools, iToolbarScale);
 
   iActionMap = new QSignalMapper(this);
-  connect(iActionMap, SIGNAL(mapped(const QString &)),
+  connect(iActionMap, SIGNAL(mappedString(const QString &)),
 	  SLOT(qAction(const QString &)));
 
   iModeActionGroup = new QActionGroup(this);
@@ -286,6 +294,8 @@ AppUi::AppUi(lua_State *L0, int model, Qt::WindowFlags f)
     findAction(action)->setActionGroup(cs);
   }
 
+  connect(iRecentFileMenu, SIGNAL(triggered(QAction *)),
+	  SLOT(recentFileAction(QAction *)));
   connect(iSelectLayerMenu, SIGNAL(triggered(QAction *)),
 	  SLOT(selectLayerAction(QAction *)));
   connect(iMoveToLayerMenu, SIGNAL(triggered(QAction *)),
@@ -303,8 +313,6 @@ AppUi::AppUi(lua_State *L0, int model, Qt::WindowFlags f)
 
   iSelector[EUiGridSize] = new QComboBox();
   iSelector[EUiAngleSize] = new QComboBox();
-  iSelector[EUiGridSize]->setFocusPolicy(Qt::NoFocus);
-  iSelector[EUiAngleSize]->setFocusPolicy(Qt::NoFocus);
   connect(iSelector[EUiGridSize], SIGNAL(activated(int)),
 	  comboMap, SLOT(map()));
   connect(iSelector[EUiAngleSize], SIGNAL(activated(int)),
@@ -382,14 +390,14 @@ AppUi::AppUi(lua_State *L0, int model, Qt::WindowFlags f)
   QWidget *wg = new QFrame();
   QGridLayout *lo = new QGridLayout();
   wg->setLayout(lo);
-  int m = iUiScale * 2 / 100;
+  int m = uiscale(2);
   lo->setSpacing(1);
   lo->setContentsMargins(m, m, m, m); // l t r b
   // wg->setMaximumWidth(150);
   lo->setSizeConstraint(QLayout::SetFixedSize);
   QButtonGroup *bg = new QButtonGroup(wg);
   bg->setExclusive(false);
-  connect(bg, SIGNAL(buttonClicked(int)), this, SLOT(absoluteButton(int)));
+  connect(bg, SIGNAL(idClicked(int)), this, SLOT(absoluteButton(int)));
   iButton[EUiMarkShape] = nullptr;   // no such buttons
   iButton[EUiDashStyle] = nullptr;
   /* enum { EUiStroke, EUiFill, EUiPen, EUiDashStyle,
@@ -421,7 +429,6 @@ AppUi::AppUi(lua_State *L0, int model, Qt::WindowFlags f)
       iButton[i]->setIconSize(adapt_size(iButton[i]->iconSize(), iUiScale));
     }
     iSelector[i] = new QComboBox();
-    iSelector[i]->setFocusPolicy(Qt::NoFocus);
     if (i == EUiOpacity)
       lo->addWidget(iSelector[i], i+1, 0, 1, 2);
     else
@@ -453,9 +460,9 @@ AppUi::AppUi(lua_State *L0, int model, Qt::WindowFlags f)
   iSelector[EUiGridSize]->setToolTip("Grid size");
   iSelector[EUiAngleSize]->setToolTip("Angle for angular snap");
 
-  connect(comboMap, SIGNAL(mapped(int)), this, SLOT(comboSelector(int)));
+  connect(comboMap, SIGNAL(mappedInt(int)), this, SLOT(comboSelector(int)));
 
-  iPathView = new PathView(iUiScale);
+  iPathView = new PathView(uiscale(100));
   connect(iPathView, SIGNAL(activated(String)), SLOT(action(String)));
   connect(iPathView, SIGNAL(showPathStylePopup(Vector)),
 	  SLOT(showPathStylePopup(Vector)));
@@ -550,6 +557,20 @@ AppUi::~AppUi()
 
 // --------------------------------------------------------------------
 
+void AppUi::setRecentFileMenu(const std::vector<String> & names)
+{
+  iRecentFileMenu->clear();
+  for (const auto & name : names) {
+    QAction *a = new QAction(QIpe(name), iRecentFileMenu);
+    iRecentFileMenu->addAction(a);
+  }
+}
+
+void AppUi::recentFileAction(QAction *a)
+{
+  luaRecentFileSelected(IpeQ(a->text()));
+}
+
 static void populateLayerMenu(LayerBox *layerList, QMenu *menu)
 {
   menu->clear();
@@ -673,7 +694,7 @@ void AppUi::abortDrawing()
 
 void AppUi::resetCombos()
 {
-  for (int i = 0; i < EUiOpacity; ++i)
+  for (int i = 0; i < EUiView; ++i)
     iSelector[i]->clear();
 }
 
@@ -764,7 +785,7 @@ void AppUi::setBookmarks(int no, const String *s)
   for (int i = 0; i < no; ++i) {
     QListWidgetItem *item = new QListWidgetItem(QIpe(s[i]));
     if (s[i][0] == ' ')
-      item->setTextColor(Qt::blue);
+      item->setForeground(Qt::blue);
     iBookmarks->addItem(item);
   }
 }
@@ -793,8 +814,7 @@ void AppUi::setToolVisible(int m, bool vis)
 
 void AppUi::setZoom(double zoom)
 {
-  QString s;
-  s.sprintf("(%dppi)", int(72.0 * zoom));
+  QString s = QString("(%1ppi)").arg(int(72.0 * zoom));
   iResolution->setText(s);
   iCanvas->setZoom(zoom);
 }
@@ -890,14 +910,11 @@ void AppUi::comboSelector(int id)
 static const char * const aboutText =
   "<qt><h1>Ipe %d.%d.%d</h1>"
   "<p>Copyright (c) 1993-%d Otfried Cheong</p>"
-  "<p>The extensible drawing editor Ipe creates figures "
-  "in Postscript and PDF format, "
+  "<p>The extensible drawing editor Ipe creates figures in PDF format, "
   "using LaTeX to format the text in the figures.</p>"
   "<p>Ipe is released under the GNU Public License.</p>"
   "<p>See the <a href=\"http://ipe.otfried.org\">Ipe homepage</a>"
   " for further information.</p>"
-  "<p>You can \"like\" Ipe and follow Ipe announcements on "
-  "<a href=\"http://www.facebook.com/drawing.editor.Ipe7\">Facebook</a>.</p>"
   "<p>If you are an Ipe fan and want to show others, have a look at the "
   "<a href=\"https://www.shirtee.com/en/store/ipe\">Ipe T-shirts</a>.</p>"
   "<h3>Platinum and gold sponsors</h3>"
@@ -905,7 +922,9 @@ static const char * const aboutText =
   "<li>Günter Rote</li>"
   "<li>SCALGO</li>"
   "<li>Martin Ziegler</li></ul>"
-  "<p>If you enjoy Ipe, you can become a member of the exclusive community of "
+  "<p>If you enjoy Ipe, feel free to treat the author on a cup of coffee at "
+  "<a href=\"https://ko-fi.com/ipe7author\">Ko-fi</a>.</p>"
+  "<p>You can also become a member of the exclusive community of "
   "<a href=\"http://patreon.com/otfried\">Ipe patrons</a>. "
   "For the price of a cup of coffee per month you can make a meaningful contribution "
   "to the continuing development of Ipe.</p>"
@@ -914,7 +933,7 @@ static const char * const aboutText =
 void AppUi::aboutIpe()
 {
   std::vector<char> buf(strlen(aboutText) + 100);
-  sprintf(&buf[0], aboutText,
+  sprintf(buf.data(), aboutText,
 	  IPELIB_VERSION / 10000,
 	  (IPELIB_VERSION / 100) % 100,
 	  IPELIB_VERSION % 100,
@@ -923,7 +942,7 @@ void AppUi::aboutIpe()
   QMessageBox msgBox(this);
   msgBox.setWindowTitle("About Ipe");
   msgBox.setWindowIcon(prefsIcon("ipe"));
-  msgBox.setInformativeText(&buf[0]);
+  msgBox.setInformativeText(buf.data());
   msgBox.setIconPixmap(prefsPixmap("ipe"));
   msgBox.setStandardButtons(QMessageBox::Ok);
   msgBox.exec();
@@ -1041,13 +1060,32 @@ void AppUi::explain(const char *s, int t)
   statusBar()->showMessage(QString::fromUtf8(s), t);
 }
 
-void AppUi::showWindow(int width, int height)
+void AppUi::showWindow(int width, int height, int x, int y, const Color & pathViewColor)
 {
+  iPathView->setColor(pathViewColor);
   if (width > 0 && height > 0)
     resize(width, height);
+  if (x >= 0 && y >= 0)
+    move(x, y);
   show();
 }
 
+void AppUi::setFullScreen(int mode)
+{
+  Qt::WindowStates state = windowState() & ~(Qt::WindowFullScreen | Qt::WindowMaximized);
+  switch (mode) {
+  case 1:
+    state |= Qt::WindowMaximized;
+    break;
+  case 2:
+    state |= Qt::WindowFullScreen;
+    break;
+  default:
+    break;
+  }
+  setWindowState(state);
+}
+
 int AppUi::setClipboard(lua_State *L)
 {
   QString data = QString::fromUtf8(luaL_checkstring(L, 2));
@@ -1080,8 +1118,16 @@ static int getImage(lua_State *L, QImage &img)
 int AppUi::clipboard(lua_State *L)
 {
   bool allow_bitmap = lua_toboolean(L, 2);
-  QClipboard *cb = QApplication::clipboard();
+  const QClipboard *cb = QApplication::clipboard();
   if (allow_bitmap) {
+    const QMimeData *md = cb->mimeData();
+    if (md->hasUrls()) {
+      auto urls = md->urls();
+      if (urls.size() == 1 && urls[0].isLocalFile()) {
+	if (readImage(L, IpeQ(urls[0].toLocalFile())))
+	  return 1;
+      }
+    }
     QImage img = cb->image();
     if (!img.isNull()) {
       return getImage(L, img);
