@@ -5,7 +5,7 @@
 /*
 
     This file is part of the extensible drawing editor Ipe.
-    Copyright (c) 1993-2020 Otfried Cheong
+    Copyright (c) 1993-2023 Otfried Cheong
 
     Ipe is free software; you can redistribute it and/or modify it
     under the terms of the GNU General Public License as published by
@@ -61,11 +61,7 @@ class AppUi : public QMainWindow, public AppUiBase {
   Q_OBJECT
 
 public:
-#if QT_VERSION < 0x050000
-  AppUi(lua_State *L0, int model, Qt::WFlags f=0);
-#else
-  AppUi(lua_State *L0, int model, Qt::WindowFlags f=nullptr);
-#endif
+  AppUi(lua_State *L0, int model, Qt::WindowFlags f=Qt::Widget);
   ~AppUi();
 
   QAction *findAction(const char *name) const;
@@ -83,7 +79,8 @@ public:
   virtual void setActionState(const char *name, bool value) override;
   virtual void setWindowCaption(bool mod, const char *s) override;
   virtual void explain(const char *s, int t) override;
-  virtual void showWindow(int width, int height) override;
+  virtual void showWindow(int width, int height, int x, int y, const Color & pathViewColor) override;
+  virtual void setFullScreen(int mode) override;
   virtual void setBookmarks(int no, const String *s) override;
   virtual void setToolVisible(int m, bool vis) override;
   virtual int pageSorter(lua_State *L, Document *doc, int pno,
@@ -91,6 +88,8 @@ public:
   virtual int clipboard(lua_State *L) override;
   virtual int setClipboard(lua_State *L) override;
 
+  virtual void setRecentFileMenu(const std::vector<String> & names) override;
+
 public slots:
   void action(String name) override;
   void qAction(const QString &name);
@@ -111,6 +110,7 @@ public slots:
   void comboSelector(int id);
 
   void bookmarkSelected(QListWidgetItem *item);
+  void recentFileAction(QAction *a);
 
   void aboutToShowSelectLayerMenu();
   void aboutToShowMoveToLayerMenu();
@@ -127,10 +127,13 @@ private:
   void addItem(int m, const QString &title, const char *name);
   void addSnap(const char *name);
   void addEdit(const char *name);
+  QIcon prefsIcon(String name);
   QIcon prefsColorIcon(Color color);
   QPixmap prefsPixmap(String name);
   void aboutToShowStyleMenu(Kind kind, MENUHANDLE menu, String current);
 
+  inline int uiscale(int x) { return iUiScale * x / 100; }
+
 private:
   virtual void addRootMenu(int id, const char *name) override;
   virtual void addItem(int id, const char *title, const char *name) override;
