@@ -4,7 +4,7 @@
 /*
 
     This file is part of the extensible drawing editor Ipe.
-    Copyright (c) 1993-2020 Otfried Cheong
+    Copyright (c) 1993-2023 Otfried Cheong
 
     Ipe is free software; you can redistribute it and/or modify it
     under the terms of the GNU General Public License as published by
@@ -35,11 +35,14 @@
 
 #include "ipelua.h"
 
+#include "ipethumbs.h"
+
 #include <cstdio>
 #include <cstdlib>
 
 #include <windowsx.h>
 #include <gdiplus.h>
+#include <shlobj.h>
 
 #define IDI_MYICON 1
 #define ID_STATUS_TIMER 1
@@ -49,13 +52,14 @@
 #define IDC_LAYERS 7300
 #define IDBASE 8000
 #define TEXT_STYLE_BASE 8300
-#define ID_SELECTOR_BASE 8500
-#define ID_ABSOLUTE_BASE 8600
-#define ID_PATHVIEW 8700
-#define ID_MOVETOLAYER_BASE 8800
-#define ID_SELECTINLAYER_BASE 9000
-#define ID_GRIDSIZE_BASE 9100
-#define ID_ANGLESIZE_BASE 9500
+#define RECENT_FILE_BASE 8500
+#define ID_SELECTOR_BASE 9000
+#define ID_ABSOLUTE_BASE 9100
+#define ID_PATHVIEW 9200
+#define ID_MOVETOLAYER_BASE 9300
+#define ID_SELECTINLAYER_BASE 11300
+#define ID_GRIDSIZE_BASE 13300
+#define ID_ANGLESIZE_BASE 13500
 
 using namespace ipe;
 using namespace ipelua;
@@ -78,40 +82,22 @@ const wchar_t AppUi::className[] = L"ipeWindowClass";
 
 // --------------------------------------------------------------------
 
-static HBITMAP loadIcon(String action, int w, int h, int r0, int g0, int b0)
+HBITMAP AppUi::loadIcon(String action, int w, int h, int r0, int g0, int b0)
 {
-  String dir = ipeIconDirectory();
-  String base = dir + action;
-  String fname;
-  int ww = w < 0 ? -w : w;
-  if (ww >= 64 && Platform::fileExists(base + "_32x32@2x.png"))
-    fname = base + "_32x32@2x.png";
-  else if (ww >= 48 && Platform::fileExists(base + "@2x.png"))
-    fname = base + "@2x.png";
-  else if (ww >= 32 && Platform::fileExists(base + "_32x32.png"))
-    fname = base + "_32x32.png";
-  else
-    fname = base + ".png";
-  // ipeDebug("loadIcon %s at %d x %d from %s", action.z(), w, h, fname.z());
-  if (!Platform::fileExists(fname))
+  int ww = (w < 0 ? -w : w);
+  int pno = ipeIcon(action);
+  if (pno < 0)
     return nullptr;
 
-  Gdiplus::Bitmap *bitmap = Gdiplus::Bitmap::FromFile(WString(fname).data(), FALSE);
-  if (bitmap->GetLastStatus() != Gdiplus::Ok) {
-    delete bitmap;
-    return nullptr;
-  }
+  Thumbnail thumbs(ipeIcons.get(), ww);
+  thumbs.setTransparent(true);
+  Buffer bits = thumbs.render(ipeIcons->page(pno), 0);
 
-  int wd = bitmap->GetWidth();
-  int ht = bitmap->GetHeight();
+  if (w < 0) w = ww;
+  if (h < 0) h = ww;
 
-  Gdiplus::BitmapData* bitmapData = new Gdiplus::BitmapData;
-  Gdiplus::Rect rect(0, 0, wd, ht);
-  bitmap->LockBits(&rect, Gdiplus::ImageLockModeRead,
-		   PixelFormat32bppPARGB, bitmapData);
-
-  if (w < 0) w = wd;
-  if (h < 0) h = ht;
+  int wd = ww;
+  int ht = ww;
 
   BITMAPINFO bmi;
   ZeroMemory(&bmi, sizeof(bmi));
@@ -124,9 +110,9 @@ static HBITMAP loadIcon(String action, int w, int h, int r0, int g0, int b0)
   void *pbits = nullptr;
   HBITMAP bm = CreateDIBSection(nullptr, &bmi, DIB_RGB_COLORS, &pbits, nullptr, 0);
 
-  unsigned char *p = (unsigned char *) bitmapData->Scan0;
+  unsigned char *p = (unsigned char *) bits.data();
   unsigned char *q = (unsigned char *) pbits;
-  int stride = bitmapData->Stride;
+  int stride = wd * sizeof(uint32_t);
 
   if (r0 < 0) {
     // make it transparent
@@ -162,13 +148,10 @@ static HBITMAP loadIcon(String action, int w, int h, int r0, int g0, int b0)
       }
     }
   }
-  bitmap->UnlockBits(bitmapData);
-  delete bitmapData;
-  delete bitmap;
   return bm;
 }
 
-static int loadIcon(String action, HIMAGELIST il, int scale)
+int AppUi::loadIcon(String action, HIMAGELIST il, int scale)
 {
   int size = scale * TBICONSIZE / 9600;
   HBITMAP bm = loadIcon(action, size, size, -1, -1, -1);
@@ -179,7 +162,7 @@ static int loadIcon(String action, HIMAGELIST il, int scale)
   return r;
 }
 
-static HBITMAP loadButtonIcon(String action, int scale)
+HBITMAP AppUi::loadButtonIcon(String action, int scale)
 {
   int size = scale * 24 / 9600;
   DWORD rgb = GetSysColor(COLOR_BTNFACE);
@@ -632,11 +615,17 @@ void AppUi::initUi()
 
   hLayers = CreateWindowExW(0, WC_LISTVIEWW, nullptr,
 			    WS_CHILD | WS_VISIBLE |
-			    LVS_LIST | LVS_SINGLESEL,
+			    LVS_REPORT | LVS_NOCOLUMNHEADER | LVS_SINGLESEL,
 			    0, 0, 0, 0, hwnd,
 			    (HMENU) IDC_LAYERS,
 			    hInst, nullptr);
-  (void) ListView_SetExtendedListViewStyle(hLayers, LVS_EX_CHECKBOXES);
+  ListView_SetExtendedListViewStyle(hLayers, LVS_EX_CHECKBOXES | LVS_EX_ONECLICKACTIVATE);
+  LVCOLUMN lvc;
+  lvc.mask = LVCF_FMT | LVCF_SUBITEM | LVCF_WIDTH;
+  lvc.fmt = LVCFMT_LEFT;
+  lvc.iSubItem = 0;
+  lvc.cx = uiscale(140);
+  ListView_InsertColumn(hLayers, 0, &lvc);
 
   hNotesGroup = CreateWindowExW(0, L"button", L"Notes",
 				WS_CHILD | WS_VISIBLE | BS_GROUPBOX,
@@ -1064,6 +1053,9 @@ void AppUi::setLayers(const Page *page, int view)
   page->objectsPerLayer(objCounts);
 
   iSettingLayers = true;
+  int top = ListView_GetTopIndex(hLayers);
+  RECT topRect;
+  ListView_GetItemRect(hLayers, top, &topRect, LVIR_BOUNDS);
   // TODO send WM_SETREDRAW
   (void) ListView_DeleteAllItems(hLayers);
 
@@ -1095,6 +1087,7 @@ void AppUi::setLayers(const Page *page, int view)
     (void) ListView_InsertItem(hLayers, &lvI);
     ListView_SetCheckState(hLayers, i, page->visible(view, i));
   }
+  ListView_Scroll(hLayers, 0, top * (topRect.bottom - topRect.top));
   iSettingLayers = false;
 }
 
@@ -1171,6 +1164,12 @@ bool AppUi::actionState(const char *name)
   return GetMenuState(hMenuBar, idx, MF_CHECKED);
 }
 
+static void clearMenu(HMENU h)
+{
+  for (int i = GetMenuItemCount(h) - 1; i >= 0; --i)
+    DeleteMenu(h, i, MF_BYPOSITION);
+}
+
 // Check/uncheck an action.
 // Used by Lua for snapangle and grid_visible
 // Also to initialize mode_select
@@ -1194,6 +1193,16 @@ void AppUi::setBookmarks(int no, const String *s)
     sendMessage(hBookmarks, LB_ADDSTRING, s[i].z());
 }
 
+void AppUi::setRecentFileMenu(const std::vector<String> & names)
+{
+  iRecentFiles.clear();
+  clearMenu(iRecentFileMenu);
+  for (int i = 0; i < size(names); ++i) {
+    iRecentFiles.push_back(names[i]);
+    AppendMenuW(iRecentFileMenu, MF_STRING, RECENT_FILE_BASE + i, WString(names[i]).data());
+  }
+}
+
 void AppUi::setToolVisible(int m, bool vis)
 {
   if (m == 1) {
@@ -1210,12 +1219,6 @@ void AppUi::setToolVisible(int m, bool vis)
   layoutChildren(false);
 }
 
-static void clearMenu(HMENU h)
-{
-  for (int i = GetMenuItemCount(h) - 1; i >= 0; --i)
-    DeleteMenu(h, i, MF_BYPOSITION);
-}
-
 void AppUi::populateTextStyleMenu()
 {
   AttributeSeq seq;
@@ -1303,23 +1306,24 @@ void AppUi::cmd(int id, int notification)
     luaBookmarkSelected(ListBox_GetCurSel(hBookmarks));
   else if (ID_ABSOLUTE_BASE <= id && id <= ID_ABSOLUTE_BASE + EUiPageMarked)
     luaAbsoluteButton(selectorNames[id - ID_ABSOLUTE_BASE]);
+  else if (RECENT_FILE_BASE <= id && id < RECENT_FILE_BASE + size(iRecentFiles))
+    luaRecentFileSelected(iRecentFiles[id - RECENT_FILE_BASE]);
   else if (ID_SELECTOR_BASE <= id && id <= ID_SELECTOR_BASE + EUiAngleSize) {
-    int sel = id - ID_SELECTOR_BASE;
-    int idx = SendMessage(hSelector[sel], CB_GETCURSEL, 0, 0);
-    luaSelector(String(selectorNames[sel]), iComboContents[sel][idx]);
-  } else if (ID_SELECTINLAYER_BASE <= id &&
-	     id < ID_SELECTINLAYER_BASE + int(iLayerNames.size())) {
+    if (notification == CBN_SELCHANGE) {
+      int sel = id - ID_SELECTOR_BASE;
+      int idx = SendMessage(hSelector[sel], CB_GETCURSEL, 0, 0);
+      luaSelector(String(selectorNames[sel]), iComboContents[sel][idx]);
+    }
+  } else if (ID_SELECTINLAYER_BASE <= id && id < ID_SELECTINLAYER_BASE + size(iLayerNames)) {
     action(String("selectinlayer-") + iLayerNames[id - ID_SELECTINLAYER_BASE]);
-  } else if (ID_MOVETOLAYER_BASE <= id &&
-	     id < ID_MOVETOLAYER_BASE + int(iLayerNames.size())) {
+  } else if (ID_MOVETOLAYER_BASE <= id && id < ID_MOVETOLAYER_BASE + size(iLayerNames)) {
     action(String("movetolayer-") + iLayerNames[id - ID_MOVETOLAYER_BASE]);
-  } else if (ID_GRIDSIZE_BASE <= id &&
-	     id < ID_GRIDSIZE_BASE + int(iComboContents[EUiGridSize].size())) {
-    luaSelector("gridsize", iComboContents[EUiGridSize][id - ID_GRIDSIZE_BASE]);
-  } else if (ID_ANGLESIZE_BASE <= id && id < ID_ANGLESIZE_BASE +
-	     int(iComboContents[EUiAngleSize].size())) {
-    luaSelector("anglesize",
-		iComboContents[EUiAngleSize][id - ID_ANGLESIZE_BASE]);
+  } else if (ID_GRIDSIZE_BASE <= id && id < ID_GRIDSIZE_BASE + size(iComboContents[EUiGridSize])) {
+    if (notification == CBN_SELCHANGE)
+      luaSelector("gridsize", iComboContents[EUiGridSize][id - ID_GRIDSIZE_BASE]);
+  } else if (ID_ANGLESIZE_BASE <= id && id < ID_ANGLESIZE_BASE + size(iComboContents[EUiAngleSize])) {
+    if (notification == CBN_SELCHANGE)
+      luaSelector("anglesize", iComboContents[EUiAngleSize][id - ID_ANGLESIZE_BASE]);
   } else if (id == ID_PATHVIEW) {
     luaShowPathStylePopup(Vector(iPathView->popupPos().x,
 				 iPathView->popupPos().y));
@@ -1334,13 +1338,10 @@ void AppUi::cmd(int id, int notification)
 static const char * const aboutText =
   "Ipe %d.%d.%d\n\n"
   "Copyright (c) 1993-%d Otfried Cheong\n\n"
-  "The extensible drawing editor Ipe creates figures "
-  "in Postscript and PDF format, "
+  "The extensible drawing editor Ipe creates figures in PDF format, "
   "using LaTeX to format the text in the figures.\n"
   "Ipe is released under the GNU Public License.\n"
   "See http://ipe.otfried.org for details.\n\n"
-  "You can \"like\" Ipe and follow Ipe announcements on Facebook "
-  "(http://www.facebook.com/drawing.editor.Ipe7).\n\n"
   "If you are an Ipe fan and want to show others, have a look at the "
   "Ipe T-shirts (www.shirtee.com/en/store/ipe).\n\n"
   "Platinum and gold sponsors\n\n"
@@ -1348,7 +1349,8 @@ static const char * const aboutText =
   " * Günter Rote\n"
   " * SCALGO\n"
   " * Martin Ziegler\n\n"
-  "If you enjoy Ipe, you can become a member of the exclusive community of "
+  "If you enjoy Ipe, feel free to treat the author on a cup of coffee at https://ko-fi.com/ipe7author.\n\n"
+  "You can also become a member of the exclusive community of "
   "Ipe patrons (http://patreon.com/otfried). "
   "For the price of a cup of coffee per month you can make a meaningful contribution "
   "to the continuing development of Ipe.";
@@ -1356,13 +1358,13 @@ static const char * const aboutText =
 void AppUi::aboutIpe()
 {
   std::vector<char> buf(strlen(aboutText) + 100);
-  sprintf(&buf[0], aboutText,
+  sprintf(buf.data(), aboutText,
 	  IPELIB_VERSION / 10000,
 	  (IPELIB_VERSION / 100) % 100,
 	  IPELIB_VERSION % 100,
 	  COPYRIGHT_YEAR);
 
-  WString wbuf(&buf[0]);
+  WString wbuf(buf.data());
   MessageBoxW(hwnd, wbuf.data(), L"About Ipe",
 	      MB_OK | MB_ICONINFORMATION | MB_APPLMODAL);
 }
@@ -1433,13 +1435,25 @@ void AppUi::setWindowCaption(bool mod, const char *s)
   setWindowText(hwnd, s);
 }
 
-void AppUi::showWindow(int width, int height)
+void AppUi::showWindow(int width, int height, int x, int y, const Color &pathViewColor)
 {
-  SetWindowPos(hwnd, nullptr, 0, 0, width, height, SWP_NOMOVE);
+  iPathView->setColor(pathViewColor);
+  SetWindowPos(hwnd, nullptr, x, y, width, height, (x < 0) ? SWP_NOMOVE : 0);
   ShowWindow(hwnd, win_nCmdShow);
   UpdateWindow(hwnd);
 }
 
+void AppUi::setFullScreen(int mode)
+{
+  bool wantFull = (mode == 2);
+  if (iFullScreen != wantFull)
+    toggleFullscreen();
+  if (mode == 1)
+    SendMessage(hwnd, WM_SYSCOMMAND, SC_MAXIMIZE, 0);
+  else if (mode == 0 && IsZoomed(hwnd))
+    SendMessage(hwnd, WM_SYSCOMMAND, SC_RESTORE, 0);
+}
+
 // show for t milliseconds, or permanently if t == 0
 void AppUi::explain(const char *s, int t)
 {
@@ -1470,15 +1484,7 @@ void AppUi::setZoom(double zoom)
 
 // --------------------------------------------------------------------
 
-static void listFormats()
-{
-  UINT format = 0;
-  char name[256];
-  while ((format = EnumClipboardFormats(format)) != 0) {
-    GetClipboardFormatNameA(format, name, 256);
-    ipeDebug("Clipboard format %d %s", format, name);
-  }
-}
+// https://docs.microsoft.com/en-us/windows/win32/shell/datascenarios
 
 static int clipboardBitmap(lua_State *L)
 {
@@ -1536,19 +1542,94 @@ static int clipboardBitmap(lua_State *L)
   return 1;
 }
 
+static int clipboardPNG(lua_State *L, UINT pngFormat)
+{
+  HGLOBAL hGlobal = (HGLOBAL) GetClipboardData(pngFormat);
+  if (hGlobal == nullptr) {
+    CloseClipboard();
+    return 0;
+  }
+
+  Buffer data = Buffer((char *) GlobalLock(hGlobal), GlobalSize(hGlobal));
+  GlobalUnlock(hGlobal);
+  CloseClipboard();
+
+  String temp = Platform::latexDirectory() + "clip.png";
+  FILE *f = Platform::fopen(temp.z(), "wb");
+  fwrite(data.data(), 1, data.size(), f);
+  fclose(f);
+
+  Vector dpi;
+  const char *errmsg;
+  Bitmap bm = Bitmap::readPNG(temp.z(), dpi, errmsg);
+  _wunlink(temp.w().data()); // delete temp file
+
+  if (bm.isNull())
+    return 0;
+
+  Rect r(Vector::ZERO, Vector(bm.width(), bm.height()));
+  Image *img = new Image(r, bm);
+  push_object(L, img);
+  return 1;
+}
+
+static int clipboardHDrop(lua_State *L)
+{
+  HGLOBAL hGlobal = (HGLOBAL) GetClipboardData(CF_HDROP);
+  if (hGlobal == nullptr) {
+    CloseClipboard();
+    return 0;
+  }
+
+  DROPFILES * drop = (DROPFILES *) GlobalLock(hGlobal);
+  const uint8_t *p = ((const uint8_t *) drop) + drop->pFiles;
+  String fn;
+  if (drop->fWide) {
+    wchar_t *w = (wchar_t *) p;
+    while (*w)
+      fn.appendUtf8(*w++);
+  } else {
+    while (*p)
+      fn.appendUtf8(*p++);
+  }
+  GlobalUnlock(hGlobal);
+  CloseClipboard();
+
+  return AppUiBase::readImage(L, fn);
+}
+
 int AppUi::clipboard(lua_State *L)
 {
   bool allow_bitmap = lua_toboolean(L, 2);
   OpenClipboard(hwnd);
-  listFormats();
   UINT format = 0;
+  char formatName[256];
+  bool haveText = false;
+  bool haveBitmap = false;
+  bool haveHDrop = false;
+  UINT pngFormat = 0;
   while ((format = EnumClipboardFormats(format)) != 0) {
-    if (format == CF_BITMAP || format == CF_UNICODETEXT)
-      break;
+    if (!GetClipboardFormatNameA(format, formatName, 256))
+      sprintf(formatName, "#%u", format);
+    ipeDebug("Clipboard format %d %s", format, formatName);
+    if (format == CF_UNICODETEXT)
+      haveText = true;
+    if (format == CF_BITMAP)
+      haveBitmap = true;
+    if (format == CF_HDROP)
+      haveHDrop = true;
+    if (!strcmp(formatName, "PNG"))
+      pngFormat = format;
   }
-  if (allow_bitmap && format == CF_BITMAP)
-    return clipboardBitmap(L);
-  if (format == CF_UNICODETEXT){
+  if (allow_bitmap) {
+    if (haveHDrop && clipboardHDrop(L))
+      return 1;
+    if (pngFormat && clipboardPNG(L, pngFormat))
+      return 1;
+    if (haveBitmap)
+      return clipboardBitmap(L);
+  }
+  if (haveText) {
     HGLOBAL hGlobal = GetClipboardData(CF_UNICODETEXT);
     if (hGlobal == nullptr) {
       CloseClipboard();
@@ -1652,8 +1733,9 @@ LRESULT CALLBACK AppUi::wndProc(HWND hwnd, UINT message, WPARAM wParam,
     case  NM_CUSTOMDRAW: {
       switch (((LPNMCUSTOMDRAW) lParam)->dwDrawStage) {
       case CDDS_PREPAINT:
-	return CDRF_NOTIFYITEMDRAW;
-      case CDDS_ITEMPREPAINT: {
+      case CDDS_ITEMPREPAINT:
+	return CDRF_NOTIFYSUBITEMDRAW;
+      case CDDS_ITEMPREPAINT|CDDS_SUBITEM: {
 	LPNMLVCUSTOMDRAW q = (LPNMLVCUSTOMDRAW) lParam;
 	UINT flags = q->nmcd.lItemlParam;
 	if (flags & 0x01)   // active
@@ -1676,7 +1758,7 @@ LRESULT CALLBACK AppUi::wndProc(HWND hwnd, UINT message, WPARAM wParam,
 	HWND from = nm->hdr.hwndFrom;
 	int l = nm->iItem;
 	if (from == ui->hLayers && l >= 0) {
-	  POINT pos = ((LPNMITEMACTIVATE) lParam)->ptAction;
+	  POINT pos = nm->ptAction;
 	  ClientToScreen(ui->hLayers, &pos);
 	  Vector v(pos.x, pos.y);
 	  ui->luaShowLayerBoxPopup(v, ui->iLayerNames[l]);
@@ -1686,19 +1768,20 @@ LRESULT CALLBACK AppUi::wndProc(HWND hwnd, UINT message, WPARAM wParam,
       break;
     case LVN_ITEMACTIVATE:
       if (ui) {
-	int l = ((LPNMITEMACTIVATE) lParam)->iItem;
-	if (l >= 0) {
+	LPNMITEMACTIVATE nm = (LPNMITEMACTIVATE) lParam;
+	int l = nm->iItem;
+	if (l >= 0)
 	  ui->luaLayerAction("active", ui->iLayerNames[l]);
-	}
       }
       break;
     case LVN_ITEMCHANGED:
       if (ui && !ui->iSettingLayers) {
-	int i = ((LPNMLISTVIEW) lParam)->iItem;
-	if (i >= 0) {
-	  bool checked = (ListView_GetCheckState(ui->hLayers, i) != 0);
-	  ui->luaLayerAction(checked ? "selecton" : "selectoff",
-			     ui->iLayerNames[i]);
+	LPNMLISTVIEW nm = (LPNMLISTVIEW) lParam;
+	int l = nm->iItem;
+	// ignore changes in focus and other minor decoration while mouse hovers or right clicks
+	if (l >= 0 && (nm->uOldState & LVIS_STATEIMAGEMASK) != (nm->uNewState & LVIS_STATEIMAGEMASK)) {
+	  bool checked = (ListView_GetCheckState(ui->hLayers, l) != 0);
+	  ui->luaLayerAction(checked ? "selecton" : "selectoff", ui->iLayerNames[l]);
 	}
       }
       break;
