@@ -5,7 +5,7 @@
 /*
 
     This file is part of the extensible drawing editor Ipe.
-    Copyright (c) 1993-2020 Otfried Cheong
+    Copyright (c) 1993-2023 Otfried Cheong
 
     Ipe is free software; you can redistribute it and/or modify it
     under the terms of the GNU General Public License as published by
@@ -65,7 +65,8 @@ public:
   virtual void setActionState(const char *name, bool value) override;
   virtual void setWindowCaption(bool mod, const char *s) override;
   virtual void explain(const char *s, int t) override;
-  virtual void showWindow(int width, int height) override;
+  virtual void showWindow(int width, int height, int x, int y, const Color & pathViewColor) override;
+  virtual void setFullScreen(int mode) override;
 
   virtual void setBookmarks(int no, const String *s) override;
   virtual void setToolVisible(int m, bool vis) override;
@@ -77,6 +78,8 @@ public:
   virtual int clipboard(lua_State *L) override;
   virtual int setClipboard(lua_State *L) override;
 
+  virtual void setRecentFileMenu(const std::vector<String> & names) override;
+
   void toggleFullscreen();
 
 private:
@@ -87,6 +90,10 @@ private:
   void layoutChildren(bool resizeRebar);
   void handleDpiChange(HWND hwnd, WPARAM wParam, LPARAM lParam);
 
+  HBITMAP loadIcon(String action, int w, int h, int r0, int g0, int b0);
+  int loadIcon(String action, HIMAGELIST il, int scale);
+  HBITMAP loadButtonIcon(String action, int scale);
+
   void createAction(String name, String tooltip, bool canWhileDrawing = false);
   int findAction(const char *name) const;
   int actionId(const char *name) const;
@@ -176,6 +183,7 @@ private:
   HWND hLayers;
   bool iSettingLayers;
   std::vector<String> iLayerNames;
+  std::vector<String> iRecentFiles;
   // for fullscreen mode
   bool iFullScreen;
   bool iWasMaximized;
