@@ -5,7 +5,7 @@
 /*
 
     This file is part of the extensible drawing editor Ipe.
-    Copyright (c) 1993-2020 Otfried Cheong
+    Copyright (c) 1993-2023 Otfried Cheong
 
     Ipe is free software; you can redistribute it and/or modify it
     under the terms of the GNU General Public License as published by
@@ -80,11 +80,8 @@ class PathView : public QWidget {
   Q_OBJECT
 
 public:
-#if QT_VERSION < 0x050000
-  PathView(int uiScale, QWidget* parent = nullptr, Qt::WFlags f = nullptr);
-#else
-  PathView(int uiScale, QWidget* parent = nullptr, Qt::WindowFlags f = nullptr);
-#endif
+  PathView(int uiScale, QWidget* parent = nullptr, Qt::WindowFlags f = Qt::Widget);
+  void setColor(const Color & color);
 
   void set(const AllAttributes &all, Cascade *sheet);
 
@@ -104,6 +101,7 @@ private:
   int iUiScale;
   Cascade *iCascade;
   AllAttributes iAll;
+  Color iColor;
 };
 
 // --------------------------------------------------------------------
