@@ -4,7 +4,7 @@
 --[[
 
     This file is part of the extensible drawing editor Ipe.
-    Copyright (c) 1993-2020 Otfried Cheong
+    Copyright (c) 1993-2023 Otfried Cheong
 
     Ipe is free software; you can redistribute it and/or modify it
     under the terms of the GNU General Public License as published by
@@ -141,7 +141,7 @@ function MODEL:selector(prop, value)
   self.ui:setAttributes(self.doc:sheets(), self.attributes)
   if self:page():hasSelection() then
     self:setAttribute(prop, value)
-    if prop == "textsize" and has_text(self:page()) then
+    if prop == "textsize" or (prop == "stroke" and has_text(self:page())) then
       self:autoRunLatex()
     end
   end
@@ -564,6 +564,7 @@ function MODEL:layeraction_rename(layer)
 			      nil, layer)
   if not name then return end
   name = string.gsub(name, "%s+", "_")
+  name = string.gsub(name, "\"+", "_")
   if indexOf(name, p:layers()) then
     self:warning("Cannot rename layer '" .. layer .. "'.",
 		 "The name '" .. name .. "' is already in use.")
@@ -664,6 +665,7 @@ function MODEL:action_ipelet(a)
       else
 	ipelet.run(self, method)
       end
+      self:page():ensurePrimarySelection()
       break
     end
   end
@@ -728,7 +730,12 @@ end
 function MODEL:action_save_as()
   local dir
   if self.file_name then dir = self.file_name:match(prefs.dir_pattern) end
-  if not dir then dir = prefs.save_as_directory end
+  if not dir and #self.recent_files > 0 then
+    dir = self.recent_files[1]:match(prefs.dir_pattern)
+  end
+  if not dir then
+    dir = prefs.save_as_directory
+  end
   local name
   local filter
   if self.file_name then
@@ -867,10 +874,13 @@ end
 -- this function is called without arguments on OSX from ipeAlwaysAction
 function action_manual()
   local url = config.docdir .. "/manual.html"
+  if ipe.fileExists(url) then
+    url = "file:///" ..  url
+  else
+    url = prefs.manual_url
+  end
   if ipeui.startBrowser then
-    if not ipeui.startBrowser("file:///" ..  url) then
-      messageBox(nil, "warning", "Manual not available!", url)
-    end
+    ipeui.startBrowser(url)
   else
     os.execute(prefs.browser:format(url))
   end
@@ -888,6 +898,7 @@ function action_show_libraries(win)
   s = s .. "\n * The font rendering library " .. config.freetype_version
   s = s .. "\n * The rendering library Cairo " .. config.cairo_version
   s = s .. "\n * The clothoid spline library libspiro " .. config.spiro_version
+  s = s .. "\n * The GNU scientific library gsl " .. config.gsl_version
   if config.qt_version then
     s = s .. "\n * The GUI toolkit Qt " .. config.qt_version
   end
@@ -914,6 +925,9 @@ function action_show_configuration(win, dpi)
   if config.platform == "win" then
     s = s .. "\n * Monitor resolution: " .. dpi .. " dpi"
   end
+  if config.toolkit == "qt" then
+    s = s .. "\n * Device pixel ratio: " .. config.device_pixel_ratio
+  end
   s = s .. "\n * External editor: " .. (prefs.external_editor or "none")
   messageBox(win, "information", "Ipe configuration", s)
 end
@@ -1011,7 +1025,7 @@ end
 
 function MODEL:action_cloud_latex()
   local d = ipeui.Dialog(win, "Ipe: Latex in the cloud")
-  local urlFile = config.latexdir .. "url.txt"
+  local urlFile = config.latexdir .. "url1.txt"
   if ipe.fileExists(urlFile) then
     local f = ipe.openFile(urlFile, "r")
     local url = f:read("a")
@@ -1219,6 +1233,11 @@ function MODEL:action_select_all()
   self.ui:update(false)
 end
 
+function MODEL:action_deselect_all()
+  self:page():deselectAll()
+  self.ui:update(false)
+end
+
 function MODEL:action_select_in_active_layer()
   local p = self:page()
   local active = p:active(self.vno)
@@ -1755,6 +1774,7 @@ function MODEL:action_paste_page()
   self:register(t)
   self:nextPage(1)
   self:setPage()
+  self:autoRunLatex()
 end
 
 function MODEL:action_delete_page()
@@ -1790,7 +1810,7 @@ local function update_dialog(d, sec)
 end
 
 function MODEL:action_edit_title()
-  local d = ipeui.Dialog(self.ui:win(), "Ipe: Edit page title and sections")
+  local d = ipeui.Dialog(self.ui:win(), "Ipe: Edit page title and sections", self.doc:properties().language)
   d:add("label1", "label", { label="Page title"}, 1, 1, 1, 4)
   d:add("title", "text", {}, 0, 1, 1, 4)
   d:add("label2", "label", { label="Sections"}, 0, 1, 1, 4)
@@ -1854,7 +1874,7 @@ function MODEL:action_edit_title()
 end
 
 function MODEL:action_edit_notes()
-  local d = ipeui.Dialog(self.ui:win(), "Ipe: Edit page notes")
+  local d = ipeui.Dialog(self.ui:win(), "Ipe: Edit page notes", self.doc:properties().language)
   d:add("label1", "label", { label="Page notes"}, 1, 1)
   d:add("notes", "text", {}, 0, 1)
   d:addButton("ok", "&Ok", "accept")
@@ -2434,6 +2454,7 @@ function MODEL:action_paste()
 	   end
   p:deselectAll()
   self:register(t)
+  self:autoRunLatex()
 end
 
 function MODEL:action_paste_with_layer()
@@ -2491,6 +2512,7 @@ function MODEL:action_paste_with_layer()
 	   end
   p:deselectAll()
   self:register(t)
+  self:autoRunLatex()
 end
 
 ----------------------------------------------------------------------
@@ -2550,6 +2572,7 @@ function MODEL:action_update_style_sheets()
 	   end
   self:register(t)
   self:action_check_style()
+  self:autoRunLatex()
 end
 
 local function sheets_namelist(list)
@@ -2721,7 +2744,7 @@ end
 function MODEL:action_document_properties()
   local engines = { "default", "pdftex", "xetex", "luatex" }
   local p = self.doc:properties()
-  local d = ipeui.Dialog(self.ui:win(), "Ipe document properties")
+  local d = ipeui.Dialog(self.ui:win(), "Ipe document properties", self.doc:properties().language)
   d:add("label1", "label", { label="Title"}, 1, 1)
   d:add("title", "input", {}, -1, 2, 1, 6)
   d:add("label2", "label", { label="Author"}, 0, 1)
@@ -2730,13 +2753,16 @@ function MODEL:action_document_properties()
   d:add("subject", "input", {}, -1, 2, 1, 6)
   d:add("label4", "label", { label="Keywords"}, 0, 1)
   d:add("keywords", "input", {}, -1, 2, 1, 6)
+  d:add("label4b", "label", { label="Language"}, 0, 1)
+  d:add("language", "input", {}, -1, 2, 1, 6)
   d:add("label5", "label", { label="Latex engine"}, 0, 1)
   d:add("tex", "combo", engines, -1, 2, 1, 6)
   d:add("label6", "label", { label="Latex preamble"}, 0, 1)
   d:add("preamble", "text", {}, -1, 2, 2, 6)
-  d:add("label7", "label", { label="Page mode"}, 8, 1)
+  d:add("label7", "label", { label="Page mode"}, 9, 1)
   d:add("fullscreen", "checkbox", { label="&Full screen"}, -1, 2)
   d:add("numberpages", "checkbox", { label="&Number pages"}, -1, 3)
+  d:add("sequentialtext", "checkbox", { label="&Sequential text"}, -1, 4)
   d:add("label8", "label", { label="Created"}, 0, 1)
   d:add("created", "label",  { label="" }, -1, 2)
   d:add("label9", "label", { label="Modified"}, 0, 1)
@@ -2746,7 +2772,7 @@ function MODEL:action_document_properties()
   d:addButton("ok", "&Ok", "accept")
   d:addButton("cancel", "&Cancel", "reject")
   d:setStretch("column", 5, 1)
-  d:setStretch("row", 7, 1)
+  d:setStretch("row", 8, 1)
   addEditorField(d, "preamble")
   for n in pairs(p) do d:set(n, p[n]) end
   if not d:execute() then return end
@@ -2767,6 +2793,31 @@ function MODEL:action_document_properties()
   t.undo = function (t, doc) doc:setProperties(t.original) end
   t.redo = function (t, doc) doc:setProperties(t.final) end
   self:register(t)
+  self:autoRunLatex()
+end
+
+----------------------------------------------------------------------
+
+function MODEL:saction_custom()
+  local p = self:page()
+  local prim = p:primarySelection()
+  local obj = p[prim]
+  local custom = self:getString("Enter custom value", nil, obj:getCustom())
+  if not custom then return end
+  local t = { label="set custom",
+	      pno=self.pno,
+	      vno=self.vno,
+	      primary=prim,
+	      original=obj:getCustom(),
+	      custom=custom,
+	    }
+  t.undo = function (t, doc)
+             doc[t.pno][t.primary]:setCustom(t.original)
+	   end
+  t.redo = function (t, doc)
+             doc[t.pno][t.primary]:setCustom(t.custom)
+           end
+  self:register(t)
 end
 
 ----------------------------------------------------------------------
