@@ -4,7 +4,7 @@
 --[[
 
     This file is part of the extensible drawing editor Ipe.
-    Copyright (c) 1993-2019 Otfried Cheong
+    Copyright (c) 1993-2023 Otfried Cheong
 
     Ipe is free software; you can redistribute it and/or modify it
     under the terms of the GNU General Public License as published by
@@ -61,29 +61,6 @@ function cloneSubpath(sp)
   return nsp
 end
 
-local function recomputeMatrix(seg, cpno)
-  local alpha, beta = seg.arc:angles()
-  local p = ipe.Direction(alpha)
-  local q = ipe.Direction(beta)
-  local cl
-  if cpno == 1 then
-    cl = ipe.LineThrough(V(0,0), q)
-    p = seg.arc:matrix():inverse() * seg[cpno]
-  else -- cpno == 2
-    cl = ipe.LineThrough(V(0,0), p)
-    q = seg.arc:matrix():inverse() * seg[cpno]
-  end
-  local center = ipe.Bisector(p, q):intersects(cl)
-  if center and center:sqLen() < 1e10 then
-    local radius = (p - center):len()
-    alpha = (p - center):angle()
-    beta = (q - center):angle()
-    local m = (seg.arc:matrix() *
-	     ipe.Matrix(radius, 0, 0, radius, center.x, center.y))
-    seg.arc = ipe.Arc(m, alpha, beta)
-  end
-end
-
 function moveCP(shape, spno, segno, cpno, v)
   local sp = shape[spno]
   if sp.type == "closedspline" then
@@ -100,11 +77,11 @@ function moveCP(shape, spno, segno, cpno, v)
       local alpha = u:angle()
       local m = (sp[1] * ipe.Matrix(radius, 0, 0, radius, 0, 0)
 	       * ipe.Rotation(alpha))
-      if not m:isSingular() then sp[1] = m end
+      if not m:isSingular(0.001) then sp[1] = m end
     elseif cpno == 2 then -- minor
       local u = sp[1]:inverse() * v
       local m = sp[1] * ipe.Matrix(1, 0, u.x, u.y, 0, 0)
-      if not m:isSingular() then sp[1] = m end
+      if not m:isSingular(0.001) then sp[1] = m end
     end
   else -- curve
     local seg = sp[segno]
@@ -127,12 +104,12 @@ function moveCP(shape, spno, segno, cpno, v)
       elseif cpno == #seg and segno < #sp then
 	sp[segno + 1][1] = v
       end
-      if seg.type == "arc" then recomputeMatrix(seg, cpno) end
+      if seg.type == "arc" then recomputeArcMatrix(seg, cpno) end
       if segno > 1 and cpno == 1 and sp[segno - 1].type == "arc" then
-	recomputeMatrix(sp[segno - 1], 2)
+	recomputeArcMatrix(sp[segno - 1], 2)
       end
       if segno < #sp and cpno == #seg and sp[segno + 1].type == "arc" then
-	recomputeMatrix(sp[segno + 1], 1)
+	recomputeArcMatrix(sp[segno + 1], 1)
       end
     end
   end
@@ -175,7 +152,8 @@ function EDITTOOL:type()
   end
   cur = cur[self.segNum]
   if self.cpNum == 1 or self.cpNum == #cur then return VERTEX end
-  if cur.type == "spline" or cur.type == "oldspline" then return SPLINECP end
+  if cur.type == "spline" or cur.type == "oldspline" or cur.type == "cardinal"
+     or cur.type == "spiro" then return SPLINECP end
   -- must be center of arc
   return CENTER
 end
@@ -240,7 +218,8 @@ function EDITTOOL:setShapeMarks()
       for _,seg in ipairs(sp) do
 	m[#m + 1] = seg[#seg]
 	m[#m + 1] = VERTEX
-	if seg.type=="spline" or seg.type=="oldspline" then
+	if seg.type=="spline" or seg.type=="oldspline" or seg.type == "cardinal"
+	  or seg.type == "spiro" then
 	  for i = 2,#seg-1 do
 	    m[#m + 1] = seg[i]
 	    m[#m + 1] = SPLINECP
@@ -410,7 +389,7 @@ function EDITTOOL:action_duplicate()
     local seg = sp[self.segNum]
     local cp = seg[self.cpNum]
     if 1 < self.cpNum and self.cpNum < #seg then
-      assert(seg.type == "spline" or seg.type == "oldspline")
+      assert(seg.type == "spline" or seg.type == "oldspline" or seg.type == "cardinal" or seg.type == "spiro")
       table.insert(seg, self.cpNum, cp)
       local pcp = seg[self.cpNum - 1]
       local ncp = seg[self.cpNum + 2]
@@ -504,8 +483,7 @@ function EDITTOOL:action_convert_spline()
     beziers.closed = false
     self.shape[self.spNum] = beziers
   else
-    local beziers = ipe.splineToBeziers(sp[self.segNum], false,
-					sp[self.segNum].type == "oldspline")
+    local beziers = ipe.splineToBeziers(sp[self.segNum], false)
     table.remove(sp, self.segNum)
     for i=1,#beziers do
       table.insert(sp, self.segNum + i - 1, beziers[i])
@@ -514,6 +492,17 @@ function EDITTOOL:action_convert_spline()
   self.undo[#self.undo + 1] = t
 end
 
+function EDITTOOL:action_set_tension()
+  local sp = self.shape[self.spNum]
+  local tension = self.model:getDouble(nil, "Tension", sp[self.segNum].tension, 0.0, 10.0)
+  local t = { label = "change tension",
+	      spNum = self.spNum,
+	      original = cloneSubpath(sp),
+	      undo = revertSubpath }
+  sp[self.segNum].tension = tension
+  self.undo[#self.undo + 1] = t
+end
+
 function EDITTOOL:action_insert_knot()
   local sp = self.shape[self.spNum]
   local seg = sp[self.segNum]
@@ -848,7 +837,7 @@ function EDITTOOL:showMenu()
       m:add("action_convert_spline", "Convert spline to Bezier segments")
       m:add("action_insert_knot_closed", "Cut closed spline at knot " ..
 	    self.cpNum)
-    else
+    elseif sp[self.segNum].type == "spline" then
       local n = #(sp[self.segNum])
       if n > 4 then
 	m:add("action_convert_spline", "Convert spline to Bezier segments")
@@ -860,6 +849,11 @@ function EDITTOOL:showMenu()
       else
 	m:add("action_cut_bezier", "Cut inside Bezier curve")
       end
+    elseif sp[self.segNum].type == "cardinal" then
+      m:add("action_convert_spline", "Convert spline to Bezier segments")
+      m:add("action_set_tension", "Change tension " .. sp[self.segNum].tension)
+    else
+      m:add("action_convert_spline", "Convert spline to Bezier segments")
     end
   end
   if t == CENTER then
