@@ -4,7 +4,7 @@
 --[[
 
     This file is part of the extensible drawing editor Ipe.
-    Copyright (c) 1993-2019 Otfried Cheong
+    Copyright (c) 1993-2023 Otfried Cheong
 
     Ipe is free software; you can redistribute it and/or modify it
     under the terms of the GNU General Public License as published by
@@ -150,6 +150,31 @@ function transformShape(matrix, shape)
   end
 end
 
+-- recompute matrix of an arc after an endpoint has been moved
+function recomputeArcMatrix(seg, cpno)
+  local alpha, beta = seg.arc:angles()
+  local p = ipe.Direction(alpha)
+  local q = ipe.Direction(beta)
+  local cl
+  if (seg[2]-seg[1]):sqLen() == 0 then return end
+  if cpno == 1 then
+    cl = ipe.LineThrough(V(0,0), q)
+    p = seg.arc:matrix():inverse() * seg[cpno]
+  else -- cpno == 2
+    cl = ipe.LineThrough(V(0,0), p)
+    q = seg.arc:matrix():inverse() * seg[cpno]
+  end
+  local center = ipe.Bisector(p, q):intersects(cl)
+  if center and center:sqLen() < 1e10 then
+    local radius = (p - center):len()
+    alpha = (p - center):angle()
+    beta = (q - center):angle()
+    local m = (seg.arc:matrix() *
+	     ipe.Matrix(radius, 0, 0, radius, center.x, center.y))
+    seg.arc = ipe.Arc(m, alpha, beta)
+  end
+end
+
 function findStyle(w, dir)
   if dir and ipe.fileExists(dir .. prefs.fsep .. w) then
     return dir .. prefs.fsep .. w
@@ -428,6 +453,28 @@ load_ipelets()
 
 --------------------------------------------------------------------
 
+recent_files = {}
+
+function load_recent_files()
+  local w = config.latexdir .. "recent_files.lua"
+  if ipe.fileExists(w) then
+    local r = {}
+    local fd = ipe.openFile(w, "r")
+    local ff = load(function () return fd:read("*L") end, w, "bt", r)
+    if ff then
+      ff()
+      if r.recent_files then
+	recent_files = r.recent_files
+      end
+    end
+    fd:close()
+  end
+end
+
+load_recent_files()
+
+--------------------------------------------------------------------
+
 if #argv == 1 and argv[1] == "-show-configuration" then
   show_configuration()
   return
@@ -470,6 +517,7 @@ end
 
 first_model = MODEL:new(first_file)
 first_model:action_fit_top()
+first_model.ui:setScreen(prefs.start_screen)
 
 local acc, accsub = win32_shortcuts(first_model.ui)
 mainloop(acc, accsub)
