@@ -4,7 +4,7 @@
 --[[
 
     This file is part of the extensible drawing editor Ipe.
-    Copyright (c) 1993-2020 Otfried Cheong
+    Copyright (c) 1993-2023 Otfried Cheong
 
     Ipe is free software; you can redistribute it and/or modify it
     under the terms of the GNU General Public License as published by
@@ -71,6 +71,8 @@ function MODEL:init(fname)
 
   self.type3_font = false
 
+  self.recent_files = recent_files
+
   self.auto_latex = prefs.auto_run_latex
   self.ui:setActionState("auto_latex", self.auto_latex)
 
@@ -104,6 +106,8 @@ function MODEL:init(fname)
     self.pristine = true
   end
 
+  self:updateRecentFiles(fname)
+
   if prefs.autosave_interval then
     self.timer = ipeui.Timer(self, "autosave")
     self.timer:setInterval(1000 * prefs.autosave_interval) -- millisecs
@@ -238,6 +242,7 @@ function MODEL:setSnap()
   end
 
   self.ui:setSnap(self.snap)
+  self.ui:setFifiVisible(true)
   local s = ind(self.snap.snapvtx, "", "v")
   s = ind(self.snap.snapctl, s, "c")
   s = ind(self.snap.snapbd, s, "b")
@@ -361,46 +366,31 @@ function MODEL:deselectSecondary()
   end
 end
 
--- If no selected object is close, select closest object.
--- If there is a selected object close enough, returns true.
--- Otherwise, check whether the closest object to the current mouse
--- position is close enough.  If so, unselect everything, make
--- this object the primary selection, and return true.
--- If not, return whether the page has a selection at all.
+-- If no object is selected, find closest object not in a locked layer.
+-- If such an object is found, select it and return true.
+-- In all other cases, return false.
 
--- If primaryOnly is true, the primary selection has to be close enough,
--- otherwise it'll be replaced as above.
-function MODEL:updateCloseSelection(primaryOnly)
-  local bound = prefs.close_distance
+function MODEL:updateCloseSelection()
   local pos = self.ui:unsnappedPos()
   local p = self:page()
 
-  -- is current selection close enough?
-  if primaryOnly then
-    local prim = p:primarySelection()
-    if prim and p:distance(prim, pos, bound) < bound then return true end
-  else
-    for i,obj,sel,layer in p:objects() do
-      if sel and p:distance(i, pos, bound) < bound then return true end
-    end
-  end
+  local prim = p:primarySelection()
+  if prim then return false end
 
-  -- current selection is not close enough: find closest object
+  -- no selection: find closest object
   local closest
+  local bound = 1000000
   for i,obj,sel,layer in p:objects() do
      if p:visible(self.vno, i) and not p:isLocked(layer) then
 	local d = p:distance(i, pos, bound)
 	if d < bound then closest = i; bound = d end
      end
   end
-
   if closest then
-    -- deselect all, and select only closest object
-    p:deselectAll()
     p:setSelect(closest, 1)
     return true
   else
-    return p:hasSelection()
+    return false
   end
 end
 
@@ -477,7 +467,18 @@ end
 function MODEL:runLatex()
   self.ui:type3Font() -- reset flag in canvas
   self.type3_font = false
-  local success, errmsg, result, log = self.doc:runLatex(self.file_name)
+  local success, errmsg, result, log
+  if prefs.freeze_in_latex then
+    success, errmsg, result, log = self.doc:runLatex(self.file_name) -- sync
+  else
+    ipeui.waitDialog(self.ui:win(),
+		     function ()
+		       success, errmsg, result, log = self.doc:runLatex(self.file_name, true) -- async
+		     end, "Compiling Latex")
+    if success and success ~= true then
+      self.doc:completeLatexRun(success) -- on ui thread
+    end
+  end
   if success then
     self.ui:setResources(self.doc)
     self.ui:update()
@@ -486,7 +487,7 @@ function MODEL:runLatex()
     self:latexErrorBox(log)
     return false
   elseif result == "runlatex" then
-    local urlFile = config.latexdir .. prefs.fsep .. "url.txt"
+    local urlFile = config.latexdir .. prefs.fsep .. "url1.txt"
     if ipe.fileExists(urlFile) then
       self:warning("Ipe did not succeed to run Latex to process your text objects.",
 		   "Ipe tried to use Latex in the cloud.\n\n"
@@ -538,8 +539,12 @@ function MODEL:newDocument()
 
   self.doc = ipe.Document()
   for _, w in ipairs(config.styleList) do
-    local sheet = assert(ipe.Sheet(w))
-    self.doc:sheets():insert(1, sheet)
+    local sheet = ipe.Sheet(w)
+    if not sheet then
+      self:warning("Style sheet '" .. w .. "' could not be read.")
+    else
+      self.doc:sheets():insert(1, sheet)
+    end
   end
   local p = self.doc:properties()
   p.tex = prefs.tex_engine
@@ -590,6 +595,8 @@ function MODEL:tryLoadDocument(fname)
 
     self.ui:explain("Document '" .. fname .. "' loaded")
 
+    self:updateRecentFiles(fname)
+
     if self.auto_latex then
       self:runLatex()
     end
@@ -641,6 +648,7 @@ function MODEL:saveDocument(fname)
   self.ui:explain("Saved document '" .. fname .. "'")
   self.file_name = fname
   self:setCaption()
+  self:updateRecentFiles(fname)
   return true
 end
 
@@ -664,21 +672,62 @@ function MODEL:auto_export(fname)
   end
 end
 
+function MODEL:updateRecentFiles(fname)
+  if fname then
+    local i = indexOf(fname, self.recent_files)
+    if i then
+      table.remove(self.recent_files, i)
+    end
+    table.insert(self.recent_files, 1, fname)
+    while #self.recent_files > prefs.num_recent_files do
+      table.remove(self.recent_files)
+    end
+    local w = config.latexdir .. "recent_files.lua"
+    local fd = ipe.openFile(w, "w")
+    if fd then
+      fd:write("recent_files = {\n")
+      for _, f in ipairs(self.recent_files) do
+	fd:write(string.format(" %q,\n", f))
+      end
+      fd:write("}\n")
+      fd:close()
+    end
+  end
+  self.ui:setRecentFiles(self.recent_files)
+end
+
+-- on OSX called without a MODEL
+function action_recent_file(fname)
+  print("action recent file", fname)
+  MODEL.new(nil, fname)
+end
+
+function MODEL:recent_file(fname)
+  if not self:checkModified() then return end
+  self:loadDocument(fname)
+end
+
 ----------------------------------------------------------------------
 
 function MODEL:autosave()
   -- only autosave if document has been modified
   if not self:isModified() then return end
   local f
-  if prefs.autosave_filename:find("%%s") then
-    if self.file_name then
+  if self.file_name then
+    if prefs.autosave_filename:find("%%s") then
       f = self.file_name:match(prefs.basename_pattern) or self.file_name
+      f = string.format(prefs.autosave_filename, f)
     else
-      f = "unnamed"
+      f = prefs.autosave_filename
+    end
+    if f:sub(1, 1) ~= prefs.fsep then -- relative filename
+      local d = self.file_name:match(prefs.dir_pattern)
+      if d then
+        f = d .. prefs.fsep .. f
+      end
     end
-    f = string.format(prefs.autosave_filename, f)
   else
-    f = prefs.autosave_filename
+    f = prefs.autosave_unnamed
   end
   self.ui:explain("Autosaving to " .. f .. "...")
   if not self.doc:save(f, "xml") then
@@ -753,7 +802,7 @@ function MODEL:creation(label, obj)
   self:register(t)
 end
 
-function MODEL:transformation(mode, m)
+function MODEL:transformation(mode, m, deselect)
   local t = { label = mode,
 	      pno = self.pno,
 	      vno = self.vno,
@@ -761,12 +810,16 @@ function MODEL:transformation(mode, m)
 	      original = self:page():clone(),
 	      matrix = m,
 	      undo = revertOriginal,
+	      deselect = deselect,
 	    }
   t.redo = function (t, doc)
 	     local p = doc[t.pno]
 	     for _,i in ipairs(t.selection) do
 	       p:transform(i, t.matrix)
 	     end
+	     if t.deselect then
+	       p:deselectAll()
+	     end
 	   end
   self:register(t)
 end
@@ -886,6 +939,7 @@ function MODEL:action_undo()
     self.ui:setAttributes(self.doc:sheets(), self.attributes)
     self:resetGridSize()
   end
+  self:autoRunLatex()
 end
 
 function MODEL:action_redo()
@@ -915,6 +969,7 @@ function MODEL:action_redo()
     self.ui:setAttributes(self.doc:sheets(), self.attributes)
     self:resetGridSize()
   end
+  self:autoRunLatex()
 end
 
 ----------------------------------------------------------------------
