@@ -4,7 +4,7 @@
 --[[
 
     This file is part of the extensible drawing editor Ipe.
-    Copyright (c) 1993-2019 Otfried Cheong
+    Copyright (c) 1993-2023 Otfried Cheong
 
     Ipe is free software; you can redistribute it and/or modify it
     under the terms of the GNU General Public License as published by
@@ -50,12 +50,16 @@
 mouse = {
   left_alt = "translate",
   left_control = "select",
+  left_command = "select",
   left_shift_control = "select",
+  left_shift_command = "select",
   middle = "pan",
   right = "menu",
   right_alt = "rotate",
   right_control = "stretch",
+  right_command = "stretch",
   right_shift_control = "scale",
+  right_shift_command = "scale",
   right_shift = "pan",
   button8 = "shredder",
   button10 = "translate",
