@@ -4,7 +4,7 @@
 --[[
 
     This file is part of the extensible drawing editor Ipe.
-    Copyright (c) 1993-2019 Otfried Cheong
+    Copyright (c) 1993-2023 Otfried Cheong
 
     Ipe is free software; you can redistribute it and/or modify it
     under the terms of the GNU General Public License as published by
@@ -37,7 +37,7 @@ local function color_icon(sheets, name)
 end
 
 function MODEL:propertiesPopup()
-  self:updateCloseSelection(false)
+  self:updateCloseSelection()
   local sel = self:selection()
   if #sel > 1 then
     self:multiPopup(sel)
@@ -68,6 +68,7 @@ function MODEL:singlePopup(prim)
   local obj = self:page()[prim]
   local m = ipeui.Menu(self.ui:win())
   self["properties_" .. obj:type()](self, obj, m)
+  m:add("action_custom", "Custom: " .. obj:getCustom())
   if prefs.tablet_menu then
     m:add("action_cut", "Cut")
     m:add("action_copy", "Copy")
@@ -342,10 +343,10 @@ function MODEL:properties_text(obj, m)
   else
     local ts = obj:get("labelstyle")
     m:add("labelstyle", "Style: " .. ts, labelstyleL, nil, ts)
-    local ha = obj:get("horizontalalignment")
-    m:add("horizontalalignment", "Horizontal alignment: " .. ha,
-	  {"left", "right", "hcenter"}, nil, ha)
   end
+  local ha = obj:get("horizontalalignment")
+  m:add("horizontalalignment", "Horizontal alignment: " .. ha,
+	{"left", "right", "hcenter"}, nil, ha)
   local va = obj:get("verticalalignment")
   m:add("verticalalignment", "Vertical alignment: " .. va,
 	{"bottom", "baseline", "top", "vcenter"}, nil, va)
