@@ -4,7 +4,7 @@
 --[[
 
     This file is part of the extensible drawing editor Ipe.
-    Copyright (c) 1993-2019 Otfried Cheong
+    Copyright (c) 1993-2023 Otfried Cheong
 
     Ipe is free software; you can redistribute it and/or modify it
     under the terms of the GNU General Public License as published by
@@ -32,6 +32,8 @@
 -- if you really want the Control key, write "Control"
 -- you can also write "Command" for the Command key for clarity.
 
+-- if you want to use the cursor keys, they are "Left", "Right", "Up", "Down".
+
 shortcuts = {
   new_window = nil,
   open = "Ctrl+O",
@@ -48,6 +50,7 @@ shortcuts = {
   redo = "Ctrl+Y",
   copy = "Ctrl+C",
   paste = nil,
+  paste_with_layer = "Ctrl+Alt+V",
   paste_at_cursor = "Ctrl+V",
   cut = "Ctrl+X",
   delete = "delete",
@@ -61,6 +64,7 @@ shortcuts = {
   behind = nil,
   duplicate = "D",
   select_all = "Ctrl+A",
+  deselect_all = "Alt+Ctrl+A",
   join_paths = "Ctrl+J",
   pick_properties = "Q",
   apply_properties = "Ctrl+Q",
@@ -77,6 +81,7 @@ shortcuts = {
   mode_stretch = "E",
   mode_pan = nil,
   mode_shredder = nil,
+  mode_laser = "Shift+Q",
   mode_label = "L",
   mode_math = "Shift+4",
   mode_paragraph = "G",
@@ -101,6 +106,7 @@ shortcuts = {
   snapbd = "F5",
   snapint = "F6",
   snapgrid = "F7",
+  snapcustom = "Shift+F7",
   snapangle = "F8",
   snapauto = "F9",
   set_origin = "F1",
@@ -114,6 +120,7 @@ shortcuts = {
   fullscreen = "F11",
   normal_size = "/",
   grid_visible = "F12",
+  pretty_display = "Ctrl+F12",
   zoom_in = "Ctrl+PgUp",
   zoom_out = "Ctrl+PgDown",
   fit_page = "\\",
@@ -133,7 +140,7 @@ shortcuts = {
   new_layer_view = "Ctrl+Shift+I",
   new_view = nil,
   delete_view = nil,
-  edit_effects = nil,
+  edit_view = "V",
   next_page = "Shift+PgDown",
   previous_page = "Shift+PgUp",
   first_page = "Shift+Home",
@@ -146,22 +153,24 @@ shortcuts = {
   edit_title = "Ctrl+P",
   edit_notes = "Ctrl+N",
   page_sorter = nil,
-  jump_view = "V",
+  jump_view = "Shift+J",
   jump_page = "J",
   ipelet_1_goodies = nil, -- Mirror horizontal
   ipelet_2_goodies = nil, -- Mirror vertical
   ipelet_3_goodies = nil, -- Mirror at x-axis
-  ipelet_4_goodies = nil, -- Turn 90 degrees
-  ipelet_5_goodies = nil, -- Turn 180 degrees
-  ipelet_6_goodies = nil, -- Turn 270 degrees
-  ipelet_7_goodies = "Ctrl+R", -- Precise rotate
-  ipelet_8_goodies = "Ctrl+K", -- Precise stretch
-  ipelet_9_goodies = nil, -- Insert precise box
-  ipelet_10_goodies = nil, -- Insert bounding box
-  ipelet_11_goodies = nil, -- Insert media box
-  ipelet_12_goodies = nil, -- Mark circle center
-  ipelet_13_goodies = nil, -- Make parabolas
-  ipelet_14_goodies = nil, -- Regular k-gon
+  ipelet_4_goodies = nil, -- Mirror at y-axis
+  ipelet_5_goodies = nil, -- Turn 90 degrees
+  ipelet_6_goodies = nil, -- Turn 180 degrees
+  ipelet_7_goodies = nil, -- Turn 270 degrees
+  ipelet_8_goodies = "Ctrl+R", -- Precise rotate
+  ipelet_9_goodies = "Ctrl+K", -- Precise stretch
+  ipelet_10_goodies = nil, -- Rotate coordinate system
+  ipelet_11_goodies = nil, -- Insert precise box
+  ipelet_12_goodies = nil, -- Insert bounding box
+  ipelet_13_goodies = nil, -- Insert media box
+  ipelet_14_goodies = nil, -- Mark circle center
+  ipelet_15_goodies = nil, -- Make parabolas
+  ipelet_16_goodies = nil, -- Regular k-gon
   ipelet_2_symbols = "Alt+Y", -- Use current symbol
 }
 
