@@ -3,7 +3,7 @@
 /*
 
     This file is part of the extensible drawing editor Ipe.
-    Copyright (c) 1993-2020 Otfried Cheong
+    Copyright (c) 1993-2023 Otfried Cheong
 
     Ipe is free software; you can redistribute it and/or modify it
     under the terms of the GNU General Public License as published by
@@ -111,13 +111,10 @@ static int mainloop(lua_State *L)
 
 static const char * const about_text =
   "Copyright (c) 1993-%d Otfried Cheong\n\n"
-  "The extensible drawing editor Ipe creates figures "
-  "in Postscript and PDF format, "
+  "The extensible drawing editor Ipe creates figures in PDF format, "
   "using LaTeX to format the text in the figures.\n"
   "Ipe is released under the GNU Public License.\n"
   "See http://ipe.otfried.org for details.\n"
-  "You can \"like\" Ipe and follow Ipe announcements on Facebook "
-  "(http://www.facebook.com/drawing.editor.Ipe7).\n\n"
   "If you are an Ipe fan and want to show others, have a look at the "
   "Ipe T-shirts (www.shirtee.com/en/store/ipe).\n\n"
   "Platinum and gold sponsors\n\n"
@@ -125,7 +122,8 @@ static const char * const about_text =
   " * Günter Rote\n"
   " * SCALGO\n"
   " * Martin Ziegler\n\n"
-  "If you enjoy Ipe, you can become a member of the exclusive community of "
+  "If you enjoy Ipe, feel free to treat the author on a cup of coffee at https://ko-fi.com/ipe7author.\n\n"
+  "You can also become a member of the exclusive community of "
   "Ipe patrons (http://patreon.com/otfried). "
   "For the price of a cup of coffee per month you can make a meaningful contribution "
   "to the continuing development of Ipe.";
@@ -173,9 +171,17 @@ static const char * const about_text =
   String method = "action_";
   method += N2I([sender ipeAction]);
   lua_getglobal(L, method.z());
+  // push_string(L, N2I([sender title]));
   lua_call(L, 0, 0);
 }
 
+- (void) ipeRecentFileAction:(id) sender
+{
+  lua_getglobal(L, "action_recent_file");
+  push_string(L, N2I([sender title]));
+  lua_call(L, 1, 0);
+}
+
 - (BOOL) applicationShouldTerminateAfterLastWindowClosed:(NSApplication *) app {
   lua_getglobal(L, "prefs");
   lua_getfield(L, -1, "terminate_on_close");
