@@ -4,7 +4,7 @@
 /*
 
     This file is part of the extensible drawing editor Ipe.
-    Copyright (c) 1993-2020 Otfried Cheong
+    Copyright (c) 1993-2023 Otfried Cheong
 
     Ipe is free software; you can redistribute it and/or modify it
     under the terms of the GNU General Public License as published by
@@ -37,7 +37,7 @@
 #include "appui_qt.h"
 #include <QLocale>
 #include <QDir>
-#include <QDesktopWidget>
+#include <QScreen>
 
 using namespace ipe;
 using namespace ipelua;
@@ -46,7 +46,7 @@ using namespace ipelua;
 
 // --------------------------------------------------------------------
 
-static void setup_globals(lua_State *L, int width, int height)
+static void setup_globals(lua_State *L, int width, int height, double devicePixelRatio)
 {
   lua_getglobal(L, "package");
   const char *luapath = getenv("IPELUAPATH");
@@ -88,6 +88,9 @@ static void setup_globals(lua_State *L, int width, int height)
   lua_rawseti(L, -2, 2);
   lua_setfield(L, -2, "screen_geometry");
 
+  lua_pushnumber(L, devicePixelRatio);
+  lua_setfield(L, -2, "device_pixel_ratio");
+
   lua_setglobal(L, "config");
 
   lua_pushcfunction(L, ipe_tonumber);
@@ -104,10 +107,6 @@ int mainloop(lua_State *L)
 
 int main(int argc, char *argv[])
 {
-  // try to work around bug in Ubuntu, see issue #42.
-  unsetenv("QT_QPA_PLATFORMTHEME");
-  unsetenv("UBUNTU_MENUPROXY");
-
   Platform::initLib(IPELIB_VERSION);
   lua_State *L = setup_lua();
 
@@ -122,8 +121,9 @@ int main(int argc, char *argv[])
   }
   lua_setglobal(L, "argv");
 
-  QRect r = a.desktop()->screenGeometry();
-  setup_globals(L, r.width(), r.height());
+  auto & screen = a.screens().at(0);
+  QRect r = screen->availableGeometry();
+  setup_globals(L, r.width(), r.height(), screen->devicePixelRatio());
 
   lua_run_ipe(L, mainloop);
 
