@@ -4,7 +4,7 @@
 /*
 
     This file is part of the extensible drawing editor Ipe.
-    Copyright (c) 1993-2019 Otfried Cheong
+    Copyright (c) 1993-2023 Otfried Cheong
 
     Ipe is free software; you can redistribute it and/or modify it
     under the terms of the GNU General Public License as published by
@@ -124,7 +124,7 @@ static HACCEL makeAccel(lua_State *L, int arg)
     lua_pop(L, 2);
   }
 
-  HACCEL hAccel = CreateAcceleratorTable(&accel[0], accel.size());
+  HACCEL hAccel = CreateAcceleratorTable(accel.data(), accel.size());
   return hAccel;
 }
 
@@ -176,6 +176,22 @@ int WINAPI WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance,
 
   setup_globals(L);
 
+  lua_getglobal(L, "package");
+  lua_getfield(L, -1, "path");
+  const char *s = lua_tolstring(L, -1, nullptr);
+  lua_pop(L, 3);
+  ipeDebug("package.path: %s", s);
+  for (const char *t = s; *t; ++t) {
+    if (*t < 0 || *t >= 0x80) {
+      MessageBoxA(nullptr,
+		  "Ipe has been installed on a path containing non-ASCII characters.\n\n"
+		  "Unfortunately, the Lua library does not support this.",
+		  "Error!",
+		  MB_ICONEXCLAMATION | MB_OK);
+      return 9;
+    }
+  }
+
   lua_run_ipe(L, mainloop);
 
   lua_close(L);
