@@ -4,7 +4,7 @@
 /*
 
     This file is part of the extensible drawing editor Ipe.
-    Copyright (c) 1993-2019 Otfried Cheong
+    Copyright (c) 1993-2023 Otfried Cheong
 
     Ipe is free software; you can redistribute it and/or modify it
     under the terms of the GNU General Public License as published by
@@ -60,6 +60,12 @@ PathView::PathView(HWND parent, int id)
   assert(GetWindowLongPtr(hwnd, GWLP_USERDATA));
 }
 
+void PathView::setColor(const Color & color)
+{
+  iColor = color;
+  InvalidateRect(hwnd, nullptr, FALSE);
+}
+
 void PathView::set(const AllAttributes &all, Cascade *sheet)
 {
   iCascade = sheet;
@@ -120,7 +126,9 @@ void PathView::wndPaint()
   cairo_surface_t *surface = cairo_win32_surface_create(hdc);
   cairo_t *cc = cairo_create(surface);
 
-  cairo_set_source_rgb(cc, 1, 1, 0.8);
+  cairo_set_source_rgb(cc, iColor.iRed.toDouble(),
+		       iColor.iGreen.toDouble(),
+		       iColor.iBlue.toDouble());
   cairo_rectangle(cc, 0, 0, w, h);
   cairo_fill(cc);
 
@@ -134,8 +142,11 @@ void PathView::wndPaint()
     Vector u2 = (1.0/zoom) * Vector(0.80 * w, 0.5 * h);
     Vector u3 = (1.0/zoom) * Vector(0.88 * w, 0.2 * h);
     Vector u4 = (1.0/zoom) * Vector(0.96 * w, 0.5 * h);
+    Vector mid = 0.5 * (v0 + v1);
+    Vector vf = iAll.iFArrowShape.isMidArrow() ? mid : v1;
+    Vector vr = iAll.iRArrowShape.isMidArrow() ? mid : v0;
 
-    CairoPainter painter(iCascade, nullptr, cc, 3.0, false);
+    CairoPainter painter(iCascade, nullptr, cc, 3.0, false, false);
     painter.setPen(iAll.iPen);
     painter.setDashStyle(iAll.iDashStyle);
     painter.setStroke(iAll.iStroke);
@@ -146,10 +157,10 @@ void PathView::wndPaint()
     painter.lineTo(v1);
     painter.drawPath(EStrokedOnly);
     if (iAll.iFArrow)
-      Path::drawArrow(painter, v1, Angle(0),
+      Path::drawArrow(painter, vf, Angle(0),
 		      iAll.iFArrowShape, iAll.iFArrowSize,  80.0);
     if (iAll.iRArrow)
-      Path::drawArrow(painter, v0, Angle(IpePi),
+      Path::drawArrow(painter, vr, Angle(IpePi),
 		      iAll.iRArrowShape, iAll.iRArrowSize, -80.0);
     painter.setDashStyle(Attribute::NORMAL());
     painter.setTiling(iAll.iTiling);
