@@ -4,7 +4,7 @@
 /*
 
     This file is part of the extensible drawing editor Ipe.
-    Copyright (c) 1993-2020 Otfried Cheong
+    Copyright (c) 1993-2023 Otfried Cheong
 
     Ipe is free software; you can redistribute it and/or modify it
     under the terms of the GNU General Public License as published by
@@ -286,6 +286,15 @@ static int appui_setPretty(lua_State *L)
   return 0;
 }
 
+static int appui_setScreen(lua_State *L)
+{
+  AppUiBase **ui = check_appui(L, 1);
+  static const char * const screen_names[] = { "normal", "maximized", "full", nullptr };
+  int t = luaL_checkoption(L, 2, nullptr, screen_names);
+  (*ui)->setFullScreen(t);
+  return 0;
+}
+
 static int appui_type3Font(lua_State *L)
 {
   CanvasBase *canvas = check_canvas(L, 1);
@@ -563,7 +572,7 @@ static int appui_setBookmarks(lua_State *L)
     bm.push_back(String(lua_tolstring(L, -1, nullptr)));
     lua_pop(L, 1);
   }
-  (*ui)->setBookmarks(no, &bm[0]);
+  (*ui)->setBookmarks(no, bm.data());
   return 0;
 }
 
@@ -584,6 +593,22 @@ static int appui_setNotes(lua_State *L)
   return 0;
 }
 
+static int appui_setRecentFiles(lua_State *L)
+{
+  AppUiBase **ui = check_appui(L, 1);
+  luaL_argcheck(L, lua_istable(L, 2), 2, "argument is not a table");
+  int no = lua_rawlen(L, 2);
+  std::vector<String> s;
+  for (int i = 1; i <= no; ++i) {
+    lua_rawgeti(L, 2, i);
+    luaL_argcheck(L, lua_isstring(L, -1), 2, "item is not a string");
+    s.push_back(String(lua_tolstring(L, -1, nullptr)));
+    lua_pop(L, 1);
+  }
+  (*ui)->setRecentFileMenu(s);
+  return 0;
+}
+
 static int appui_showTool(lua_State *L)
 {
   static const char * const option_names[] = {
@@ -696,11 +721,11 @@ static int appui_renderPage(lua_State *L)
     Thumbnail::TargetFormat(luaL_checkoption(L, 5, nullptr, render_formats));
   const char *dst = luaL_checklstring(L, 6, nullptr);
   double zoom = luaL_checknumber(L, 7);
-  bool transparent = lua_toboolean(L, 8);
-  bool nocrop = lua_toboolean(L, 9);
   Thumbnail tn(*doc, 0);
+  tn.setTransparent(lua_toboolean(L, 8));
+  tn.setNoCrop(lua_toboolean(L, 9));
   const Page *page = (*doc)->page(pageno - 1);
-  tn.saveRender(fm, dst, page, viewno - 1, zoom, transparent, nocrop);
+  tn.saveRender(fm, dst, page, viewno - 1, zoom);
   return 0;
 }
 
@@ -732,6 +757,7 @@ static const struct luaL_Reg appui_methods[] = {
   { "setCursor", appui_setCursor },
   { "setNumbering", appui_setNumbering },
   { "setPretty", appui_setPretty },
+  { "setScreen", appui_setScreen },
   { "type3Font", appui_type3Font },
   // --------------------------------------------------------------------
   { "panTool", appui_pantool },
@@ -756,6 +782,7 @@ static const struct luaL_Reg appui_methods[] = {
   { "setNumbers", appui_setNumbers },
   { "setBookmarks", appui_setBookmarks },
   { "setNotes", appui_setNotes },
+  { "setRecentFiles", appui_setRecentFiles },
   { "showTool", appui_showTool },
   { "selectPage", appui_selectPage },
   { "pageSorter", appui_pageSorter },
@@ -783,6 +810,11 @@ static int appui_constructor(lua_State *L)
   style.paperColor = Color(1000,1000,1000);
   style.primarySelectionColor = Color(1000,0,0);
   style.secondarySelectionColor = Color(1000,0,1000);
+  style.selectionSurroundColor = Color(1000, 1000, 0);
+  style.primarySelectionWidth = 3.0;
+  style.secondarySelectionWidth = 2.0;
+  style.selectionSurroundWidth = 6.0;
+  style.gridLineColor = Color(300, 300, 300);
   style.classicGrid = false;
   style.thinLine = 0.2;
   style.thickLine = 0.9;
@@ -791,6 +823,8 @@ static int appui_constructor(lua_State *L)
   style.paperClip = false;
   style.numberPages = false;
 
+  Color pathViewColor = Color(1000, 1000, 800);
+
   lua_getglobal(L, "prefs");
 
   lua_getfield(L, -1, "canvas_style");
@@ -810,6 +844,36 @@ static int appui_constructor(lua_State *L)
       style.secondarySelectionColor = check_color(L, lua_gettop(L));
     lua_pop(L, 1); // secondary_color
 
+    lua_getfield(L, -1, "surround_color");
+    if (!lua_isnil(L, -1))
+      style.selectionSurroundColor = check_color(L, lua_gettop(L));
+    lua_pop(L, 1); // surround_color
+
+    lua_getfield(L, -1, "primary_width");
+    if (lua_isnumber(L, -1))
+      style.primarySelectionWidth = lua_tonumberx(L, -1, nullptr);
+    lua_pop(L, 1); // primary_width
+
+    lua_getfield(L, -1, "secondary_width");
+    if (lua_isnumber(L, -1))
+      style.secondarySelectionWidth = lua_tonumberx(L, -1, nullptr);
+    lua_pop(L, 1); // secondary_width
+
+    lua_getfield(L, -1, "surround_width");
+    if (lua_isnumber(L, -1))
+      style.selectionSurroundWidth = lua_tonumberx(L, -1, nullptr);
+    lua_pop(L, 1); // surround_width
+
+    lua_getfield(L, -1, "grid_line_color");
+    if (!lua_isnil(L, -1))
+      style.gridLineColor = check_color(L, lua_gettop(L));
+    lua_pop(L, 1); // grid_line_color
+
+    lua_getfield(L, -1, "path_view_color");
+    if (!lua_isnil(L, -1))
+      pathViewColor = check_color(L, lua_gettop(L));
+    lua_pop(L, 1); // path_view_color
+
     lua_getfield(L, -1, "classic_grid");
     style.classicGrid = lua_toboolean(L, -1);
     lua_pop(L, 1); // classic_grid
@@ -836,7 +900,7 @@ static int appui_constructor(lua_State *L)
   }
   lua_pop(L, 1); // canvas_style
 
-  int width = -1, height = -1;
+  int width = -1, height = -1, x = -1, y = -1;
   lua_getfield(L, -1, "window_size");
   if (lua_istable(L, -1)) {
     lua_rawgeti(L, -1, 1);
@@ -845,7 +909,13 @@ static int appui_constructor(lua_State *L)
     lua_rawgeti(L, -2, 2);
     if (lua_isnumber(L, -1))
       height = lua_tointegerx(L, -1, nullptr);
-    lua_pop(L, 2);
+    lua_rawgeti(L, -3, 3);
+    if (lua_isnumber(L, -1))
+      x = lua_tointegerx(L, -1, nullptr);
+    lua_rawgeti(L, -4, 4);
+    if (lua_isnumber(L, -1))
+      y = lua_tointegerx(L, -1, nullptr);
+    lua_pop(L, 4);
   }
   lua_pop(L, 1); // window_size
 
@@ -853,7 +923,7 @@ static int appui_constructor(lua_State *L)
 
   (*ui)->canvas()->setCanvasStyle(style);
 
-  (*ui)->showWindow(width, height);
+  (*ui)->showWindow(width, height, x, y, pathViewColor);
 
   return 1;
 }
