@@ -5,7 +5,7 @@
 /*
 
     This file is part of the extensible drawing editor Ipe.
-    Copyright (c) 1993-2019 Otfried Cheong
+    Copyright (c) 1993-2023 Otfried Cheong
 
     Ipe is free software; you can redistribute it and/or modify it
     under the terms of the GNU General Public License as published by
@@ -48,27 +48,29 @@ namespace ipe {
   class CairoPainter : public Painter {
   public:
     CairoPainter(const Cascade *sheet, Fonts *fonts, cairo_t *cc,
-		 double zoom, bool pretty);
+		 double zoom, bool pretty, bool filterBest);
     virtual ~CairoPainter() { }
 
     void setDimmed(bool dim) { iDimmed = dim; }
 
     void executeStream(const PdfDict *stream, const PdfDict *resources);
 
-  protected:
-    virtual void doPush();
-    virtual void doPop();
-    virtual void doMoveTo(const Vector &v);
-    virtual void doLineTo(const Vector &v);
-    virtual void doCurveTo(const Vector &v1, const Vector &v2,
-			   const Vector &v3);
-    virtual void doClosePath();
-    virtual void doDrawArc(const Arc &arc);
+    bool type3Font() const { return iType3Font; }
 
-    virtual void doAddClipPath();
-    virtual void doDrawPath(TPathMode mode);
-    virtual void doDrawBitmap(Bitmap bitmap);
-    virtual void doDrawText(const Text *text);
+  protected:
+    virtual void doPush() override;
+    virtual void doPop() override;
+    virtual void doMoveTo(const Vector &v) override;
+    virtual void doLineTo(const Vector &v) override;
+    virtual void doCurveTo(const Vector &v1, const Vector &v2,
+			   const Vector &v3) override;
+    virtual void doClosePath() override;
+    virtual void doDrawArc(const Arc &arc) override;
+
+    virtual void doAddClipPath() override;
+    virtual void doDrawPath(TPathMode mode) override;
+    virtual void doDrawBitmap(Bitmap bitmap) override;
+    virtual void doDrawText(const Text *text) override;
 
   private:
     const PdfDict *findResource(String kind, String name);
@@ -125,6 +127,10 @@ namespace ipe {
     bool iDimmed;
     bool iAfterMoveTo;
 
+    bool iFilterBest;
+
+    bool iType3Font;
+
     // PDF operator drawing
     std::vector<const PdfObj *> iArgs;
 
