@@ -4,7 +4,7 @@
 /*
 
     This file is part of the extensible drawing editor Ipe.
-    Copyright (c) 1993-2020 Otfried Cheong
+    Copyright (c) 1993-2023 Otfried Cheong
 
     Ipe is free software; you can redistribute it and/or modify it
     under the terms of the GNU General Public License as published by
@@ -237,6 +237,7 @@ Face::Face(const PdfDict *d, const PdfResourceBase *resources) noexcept
     iType = FontType::CIDType2;
   else if (t == "Type3") {
     iType = FontType::Type3;
+    getType3Width(d);
     return;
   } else {
     iType = FontType::Unsupported;
@@ -282,6 +283,8 @@ Face::Face(const PdfDict *d, const PdfResourceBase *resources) noexcept
     String encoding = enc->name()->value();
     if (encoding != "Identity-H")
       ipeDebug("Unsupported encoding: %s", encoding.z());
+    if (iType == FontType::CIDType2)
+      getCIDToGIDMap(d);
     // ipeDebug("FT Face has %d charmaps, is cid-keyed: %d",
     // iFace->num_charmaps, FT_IS_CID_KEYED(iFace));
   } else {
@@ -338,8 +341,11 @@ int Face::glyphIndex(int ch) noexcept
     return iEncoding[ch];
   case FontType::Truetype:
     return FT_Get_Char_Index(iFace, (FT_ULong) ch);
-  case FontType::CIDType0:
   case FontType::CIDType2:
+    if (0 <= ch && ch < size(iCID2GID))
+      return iCID2GID[ch];
+    return ch; // cid-2-gid map is identity
+  case FontType::CIDType0:
     return ch; // for cid-keyed font, this is a cid
   default:
     return 0;
@@ -388,6 +394,21 @@ void Face::getCIDWidth(const PdfDict *d) noexcept
   }
 }
 
+void Face::getCIDToGIDMap(const PdfDict *d) noexcept
+{
+  const PdfObj *map = getPdf(d, "CIDToGIDMap");
+  if (!map || !map->dict()) return;
+  Buffer buf = map->dict()->inflate();
+  const uint8_t *p = (const uint8_t *) buf.data();
+  const uint8_t *fin = p + buf.size();
+  while (p < fin) {
+    uint16_t gid = *p++ << 8;
+    gid |= *p++;
+    iCID2GID.push_back(gid);
+  }
+  ipeDebug("Font '%s' has CIDToGIDMap with %d entries", iName.z(), iCID2GID.size());
+}
+
 void Face::getSimpleWidth(const PdfDict *d) noexcept
 {
   const PdfObj *fc = getPdf(d, "FirstChar");
@@ -404,6 +425,16 @@ void Face::getSimpleWidth(const PdfDict *d) noexcept
   // ipeDebug("Got %d widths entries", iWidth.size());
 }
 
+void Face::getType3Width(const PdfDict *d) noexcept
+{
+  getSimpleWidth(d);
+  std::vector<double> a;
+  if (!d->getNumberArray("FontMatrix", nullptr, a) || a.size() != 6)
+    return;
+  for (int i = 2; i < size(iWidth); ++i)
+    iWidth[i] *= 1000 * a[0];
+}
+
 // --------------------------------------------------------------------
 
 void Face::getType1Encoding(const PdfDict *d) noexcept
