@@ -5,7 +5,7 @@
 /*
 
     This file is part of the extensible drawing editor Ipe.
-    Copyright (c) 1993-2020 Otfried Cheong
+    Copyright (c) 1993-2023 Otfried Cheong
 
     Ipe is free software; you can redistribute it and/or modify it
     under the terms of the GNU General Public License as published by
@@ -65,9 +65,11 @@ namespace ipe {
     const PdfObj *getPdf(const PdfDict *d, String key) const noexcept;
     bool getFontFile(const PdfDict *d, Buffer &data) noexcept;
     void getSimpleWidth(const PdfDict *d) noexcept;
+    void getType3Width(const PdfDict *d) noexcept;
     void getType1Encoding(const PdfDict *d) noexcept;
     void setupTruetypeEncoding() noexcept;
     void getCIDWidth(const PdfDict *d) noexcept;
+    void getCIDToGIDMap(const PdfDict *d) noexcept;
 
   private:
     const PdfDict *iFontDict;
@@ -78,6 +80,7 @@ namespace ipe {
     FT_FaceRec_ *iFace { nullptr };
     std::vector<int> iEncoding;
     std::vector<int> iWidth;
+    std::vector<uint16_t> iCID2GID;
     int iDefaultWidth { 1000 };
   };
 
