@@ -4,7 +4,7 @@
 /*
 
     This file is part of the extensible drawing editor Ipe.
-    Copyright (c) 1993-2020 Otfried Cheong
+    Copyright (c) 1993-2023 Otfried Cheong
 
     Ipe is free software; you can redistribute it and/or modify it
     under the terms of the GNU General Public License as published by
@@ -50,10 +50,8 @@ using namespace ipe;
 // --------------------------------------------------------------------
 
 Thumbnail::Thumbnail(const Document *doc, int width)
+  : iDoc{doc}, iTransparent{false}, iNoCrop{false}, iWidth{width}
 {
-  iDoc = doc;
-  iWidth = width;
-
   iLayout = iDoc->cascade()->findLayout();
   Rect paper = iLayout->paper();
   iHeight = int(iWidth * paper.height() / paper.width());
@@ -64,7 +62,7 @@ Thumbnail::Thumbnail(const Document *doc, int width)
 Buffer Thumbnail::render(const Page *page, int view)
 {
   Buffer buffer(iWidth * iHeight * 4);
-  memset(buffer.data(), 0xff, iWidth * iHeight * 4);
+  memset(buffer.data(), (iTransparent ? 0x00 : 0xff), iWidth * iHeight * 4);
 
   cairo_surface_t* surface =
     cairo_image_surface_create_for_data((uint8_t *) buffer.data(),
@@ -75,7 +73,7 @@ Buffer Thumbnail::render(const Page *page, int view)
   Vector offset = - iLayout->paper().topLeft();
   cairo_translate(cc, offset.x, offset.y);
 
-  CairoPainter painter(iDoc->cascade(), iFonts.get(), cc, iZoom, true);
+  CairoPainter painter(iDoc->cascade(), iFonts.get(), cc, iZoom, true, false);
   painter.setAttributeMap(&page->viewMap(view));
   std::vector<Matrix> layerMatrices = page->layerMatrices(view);
   painter.pushMatrix();
@@ -106,14 +104,14 @@ static cairo_status_t stream_writer(void *closure, const unsigned char *data, un
 // zoom and transparent are ignored for formats other than EPNG
 bool Thumbnail::saveRender(TargetFormat fm, const char *dst,
 			   const Page *page, int view, double zoom,
-			   bool transparent, bool nocrop)
+			   double tolerance)
 {
   if (fm != EPNG)
     zoom = 1.0;
 
   Rect bbox;
   int wid, ht;
-  if (nocrop || fm == EPS) {
+  if (iNoCrop) {
     bbox = iLayout->paper();
     wid = int(bbox.width() * zoom);
     ht = int(bbox.height() * zoom);
@@ -133,10 +131,7 @@ bool Thumbnail::saveRender(TargetFormat fm, const char *dst,
     if (wid * ht > 20000000)
       return false;
     data = Buffer(wid * ht * 4);
-    if (transparent)
-      memset(data.data(), 0x00, wid * ht * 4);
-    else
-      memset(data.data(), 0xff, wid * ht * 4);
+    memset(data.data(), (iTransparent ? 0x00 : 0xff), wid * ht * 4);
     surface = cairo_image_surface_create_for_data((uint8_t *) data.data(),
 						  CAIRO_FORMAT_ARGB32,
 						  wid, ht, wid * 4);
@@ -165,12 +160,13 @@ bool Thumbnail::saveRender(TargetFormat fm, const char *dst,
   }
   cairo_translate(cc, -offset.x, -offset.y);
 
-  CairoPainter painter(iDoc->cascade(), iFonts.get(), cc, zoom, true);
+  cairo_set_tolerance(cc, tolerance);
+  CairoPainter painter(iDoc->cascade(), iFonts.get(), cc, zoom, true, true);
   painter.setAttributeMap(&page->viewMap(view));
   std::vector<Matrix> layerMatrices = page->layerMatrices(view);
   painter.pushMatrix();
 
-  if (nocrop) {
+  if (iNoCrop) {
     const Symbol *background =
       iDoc->cascade()->findSymbol(Attribute::BACKGROUND());
     if (background && page->findLayer("BACKGROUND") < 0)
@@ -243,7 +239,7 @@ Buffer PdfThumbnail::render(const PdfDict *page)
   cairo_scale(cc, zoom, -zoom);
 
   if (stream) {
-    CairoPainter painter(iCascade.get(), iFonts.get(), cc, 1.0, false);
+    CairoPainter painter(iCascade.get(), iFonts.get(), cc, 1.0, false, false);
     painter.executeStream(stream, page);
   }
   cairo_surface_flush(surface);
