@@ -5,7 +5,7 @@
 /*
 
     This file is part of the extensible drawing editor Ipe.
-    Copyright (c) 1993-2020 Otfried Cheong
+    Copyright (c) 1993-2023 Otfried Cheong
 
     Ipe is free software; you can redistribute it and/or modify it
     under the terms of the GNU General Public License as published by
@@ -45,14 +45,19 @@ namespace ipe {
 
     Thumbnail(const Document *doc, int width);
 
+    void setTransparent(bool t) { iTransparent = t; }
+    void setNoCrop(bool n) {iNoCrop = n; }
+
     int width() const { return iWidth; }
     int height() const { return iHeight; }
     Buffer render(const Page *page, int view);
     bool saveRender(TargetFormat fm, const char *dst,
 		    const Page *page, int view, double zoom,
-		    bool transparent, bool nocrop);
+		    double tolerance = 0.1);
   private:
     const Document *iDoc;
+    bool iTransparent;
+    bool iNoCrop;
     int iWidth;
     int iHeight;
     double iZoom;
