@@ -4,7 +4,7 @@
 /*
 
     This file is part of the extensible drawing editor Ipe.
-    Copyright (c) 1993-2019 Otfried Cheong
+    Copyright (c) 1993-2023 Otfried Cheong
 
     Ipe is free software; you can redistribute it and/or modify it
     under the terms of the GNU General Public License as published by
@@ -85,12 +85,19 @@ CanvasBase::CanvasBase()
   iFifiMode = Snap::ESnapNone;
   iSelectionVisible = true;
 
+  iType3Font = false;
+
   isInkMode = false;
   iAdditionalModifiers = 0;
 
   iStyle.paperColor = Color(1000,1000,1000);
   iStyle.primarySelectionColor = Color(1000, 0, 0);
   iStyle.secondarySelectionColor= Color(1000, 0, 1000);
+  iStyle.selectionSurroundColor = Color(1000, 1000, 0);
+  iStyle.gridLineColor = Color(300, 300, 300);
+  iStyle.primarySelectionWidth = 3.0;
+  iStyle.secondarySelectionWidth = 2.0;
+  iStyle.selectionSurroundWidth = 6.0;
   iStyle.pretty = false;
   iStyle.classicGrid = false;
   iStyle.thinLine = 0.2;
@@ -129,6 +136,13 @@ void CanvasBase::setResources(const PdfResources *resources)
   iFonts = std::make_unique<Fonts>(resources);
 }
 
+bool CanvasBase::type3Font()
+{
+  bool result = iType3Font;
+  iType3Font = false;
+  return result;
+}
+
 // --------------------------------------------------------------------
 
 //! Set the page to be displayed.
@@ -271,7 +285,9 @@ void CanvasBase::drawGrid(cairo_t *cc)
   Vector screenLR = devToUser(Vector(iWidth, iHeight));
 
   cairo_save(cc);
-  cairo_set_source_rgb(cc, 0.3, 0.3, 0.3);
+  cairo_set_source_rgb(cc, iStyle.gridLineColor.iRed.toDouble(),
+		       iStyle.gridLineColor.iGreen.toDouble(),
+		       iStyle.gridLineColor.iBlue.toDouble());
 
   if (iStyle.classicGrid) {
     double lw = iStyle.thinLine / iZoom;
@@ -356,9 +372,10 @@ void CanvasBase::drawObjects(cairo_t *cc)
     cairo_clip(cc);
   }
 
-  CairoPainter painter(iCascade, iFonts.get(), cc, iZoom, iStyle.pretty);
+  CairoPainter painter(iCascade, iFonts.get(), cc, iZoom, iStyle.pretty, false);
   painter.setDimmed(iDimmed);
-  // painter.Transform(CanvasTfm());
+  painter.setAttributeMap(&iPage->viewMap(iView));
+  std::vector<Matrix> layerMatrices = iPage->layerMatrices(iView);
   painter.pushMatrix();
 
   const Symbol *background =
@@ -377,10 +394,16 @@ void CanvasBase::drawObjects(cairo_t *cc)
     title->draw(painter);
 
   for (int i = 0; i < iPage->count(); ++i) {
-    if (iPage->objectVisible(iView, i))
+    if (iPage->objectVisible(iView, i)) {
+      painter.pushMatrix();
+      painter.transform(layerMatrices[iPage->layerOf(i)]);
       iPage->object(i)->draw(painter);
+      painter.popMatrix();
+    }
   }
   painter.popMatrix();
+  if (painter.type3Font())
+    iType3Font = true;
 }
 
 // --------------------------------------------------------------------
@@ -483,12 +506,18 @@ void CanvasBase::drawTool(Painter &painter)
     for (int i = 0; i < iPage->count(); ++i) {
       if (iPage->objectVisible(iView, i)) {
 	if (iPage->select(i) == EPrimarySelected) {
+	  painter.setStroke(Attribute(iStyle.selectionSurroundColor));
+	  painter.setPen(Attribute(Fixed(iStyle.selectionSurroundWidth)));
+	  iPage->object(i)->drawSimple(painter);
 	  painter.setStroke(Attribute(iStyle.primarySelectionColor));
-	  painter.setPen(Attribute(Fixed(2)));
+	  painter.setPen(Attribute(Fixed(iStyle.primarySelectionWidth)));
 	  iPage->object(i)->drawSimple(painter);
 	} else if (iPage->select(i) == ESecondarySelected) {
+	  painter.setStroke(Attribute(iStyle.selectionSurroundColor));
+	  painter.setPen(Attribute(Fixed(iStyle.selectionSurroundWidth)));
+	  iPage->object(i)->drawSimple(painter);
 	  painter.setStroke(Attribute(iStyle.secondarySelectionColor));
-	  painter.setPen(Attribute(Fixed(1)));
+	  painter.setPen(Attribute(Fixed(iStyle.secondarySelectionWidth)));
 	  iPage->object(i)->drawSimple(painter);
 	}
       }
