@@ -5,7 +5,7 @@
 /*
 
     This file is part of the extensible drawing editor Ipe.
-    Copyright (c) 1993-2020 Otfried Cheong
+    Copyright (c) 1993-2023 Otfried Cheong
 
     Ipe is free software; you can redistribute it and/or modify it
     under the terms of the GNU General Public License as published by
@@ -81,6 +81,11 @@ namespace ipe {
       Color paperColor;
       Color primarySelectionColor;
       Color secondarySelectionColor;
+      Color selectionSurroundColor;
+      double primarySelectionWidth;
+      double secondarySelectionWidth;
+      double selectionSurroundWidth;
+      Color gridLineColor;
       bool pretty;
       bool classicGrid;
       double thinLine;
