@@ -5,7 +5,7 @@
 /*
 
     This file is part of the extensible drawing editor Ipe.
-    Copyright (C) 1993-2015  Otfried Cheong
+    Copyright (c) 1993-2023 Otfried Cheong
 
     Ipe is free software; you can redistribute it and/or modify it
     under the terms of the GNU General Public License as published by
@@ -48,7 +48,7 @@ namespace ipe {
 
   private:
     virtual void setCursor(TCursor cursor, double w = 1.0,
-			   Color *color = 0);
+			   Color *color = nullptr);
 
     virtual void invalidate();
     virtual void invalidate(int x, int y, int w, int h);
