@@ -4,7 +4,7 @@
 /*
 
     This file is part of the extensible drawing editor Ipe.
-    Copyright (c) 1993-2020 Otfried Cheong
+    Copyright (c) 1993-2023 Otfried Cheong
 
     Ipe is free software; you can redistribute it and/or modify it
     under the terms of the GNU General Public License as published by
@@ -36,6 +36,8 @@
 
 #include <QPainter>
 #include <QPaintEvent>
+#include <QPainterPath>
+#include <QPointingDevice>
 
 using namespace ipe;
 
@@ -114,14 +116,10 @@ void IpeQtPainter::doDrawPath(TPathMode mode)
 // --------------------------------------------------------------------
 
 //! Construct a new canvas.
-#if QT_VERSION < 0x050000
-Canvas::Canvas(QWidget* parent, Qt::WFlags f)
-#else
 Canvas::Canvas(QWidget* parent, Qt::WindowFlags f)
-#endif
   : QWidget(parent, f)
 {
-  setAttribute(Qt::WA_NoBackground);
+  setAttribute(Qt::WA_OpaquePaintEvent);
   setMouseTracking(true);
   setFocusPolicy(Qt::ClickFocus);
 }
@@ -207,8 +205,8 @@ void Canvas::keyPressEvent(QKeyEvent *ev)
 
 void Canvas::mouseButton(QMouseEvent *ev, int button, bool press)
 {
-  iGlobalPos = Vector(ev->globalPos().x(), ev->globalPos().y());
-  computeFifi(ev->x(), ev->y());
+  iGlobalPos = Vector(ev->globalPosition().x(), ev->globalPosition().y());
+  computeFifi(ev->position().x(), ev->position().y());
   int mod = convertModifiers(ev->modifiers()) | iAdditionalModifiers;
   if (iTool)
     iTool->mouseButton(button | mod, press);
@@ -233,7 +231,7 @@ void Canvas::mouseReleaseEvent(QMouseEvent *ev)
 
 void Canvas::mouseMoveEvent(QMouseEvent *ev)
 {
-  computeFifi(ev->x(), ev->y());
+  computeFifi(ev->position().x(), ev->position().y());
   if (iTool)
     iTool->mouseMove();
   if (iObserver)
@@ -242,21 +240,23 @@ void Canvas::mouseMoveEvent(QMouseEvent *ev)
 
 void Canvas::tabletEvent(QTabletEvent *ev)
 {
-#if QT_VERSION < 0x050000
-  Vector globalPos(ev->hiResGlobalPos().x(), ev->hiResGlobalPos().y());
-  QPointF hiPos = ev->hiResGlobalPos() - (ev->globalPos() - ev->pos());
-#else
-  Vector globalPos(ev->hiResGlobalX(), ev->hiResGlobalY());
-  QPointF hiPos = ev->posF();
-#endif
-  ev->accept();
+  static bool pressed = false;
+  Vector globalPos(ev->globalPosition().x(), ev->globalPosition().y());
+  QPointF hiPos = ev->position();
 
   switch (ev->type()) {
   case QEvent::TabletPress:
-    ipeDebug("TabletPress: %d %d", ev->button(), ev->pointerType());
+    if (pressed) {
+      ev->ignore();
+      pressed = false;
+      return;
+    }
+    ipeDebug("TabletPress: %d %d %ld", ev->button(), ev->pointerType(), ev->timestamp());
+    pressed = true;
+    ev->accept();
     iGlobalPos = globalPos;
     computeFifi(hiPos.x(), hiPos.y());
-    if (ev->pointerType() == QTabletEvent::Eraser) {
+    if (ev->pointerType() == QPointingDevice::PointerType::Eraser) {
       if (iObserver)
 	iObserver->canvasObserverMouseAction(Qt::XButton1 |
 					     iAdditionalModifiers);
@@ -268,6 +268,7 @@ void Canvas::tabletEvent(QTabletEvent *ev)
     break;
   case QEvent::TabletMove:
     if (ev->pressure() > 0.01) {
+      ev->accept();
       computeFifi(hiPos.x(), hiPos.y());
       if (iTool)
 	iTool->mouseMove();
@@ -277,6 +278,8 @@ void Canvas::tabletEvent(QTabletEvent *ev)
     }
     // else fall through and consider it a release event
   case QEvent::TabletRelease:
+    pressed = false;
+    ev->accept();
     iGlobalPos = globalPos;
     computeFifi(hiPos.x(), hiPos.y());
     if (iTool)
@@ -292,8 +295,13 @@ void Canvas::wheelEvent(QWheelEvent *ev)
 {
   QPoint p = ev->angleDelta();
   int kind = (ev->modifiers() & Qt::ControlModifier) ? 2 : 0;
-  if (iObserver)
-    iObserver->canvasObserverWheelMoved(p.x() / 8.0, p.y() / 8.0, kind);
+  if (iObserver) {
+    if (ev->modifiers() & Qt::ShiftModifier)
+      // switch x and y
+      iObserver->canvasObserverWheelMoved(p.y() / 8.0, p.x() / 8.0, kind);
+    else
+      iObserver->canvasObserverWheelMoved(p.x() / 8.0, p.y() / 8.0, kind);
+  }
   ev->accept();
 }
 
@@ -380,19 +388,22 @@ void Canvas::drawFifi(QPainter &q)
 
 void Canvas::paintEvent(QPaintEvent * ev)
 {
-  iBWidth = iWidth = width();
-  iBHeight = iHeight = height();
+  iWidth = width();
+  iHeight = height();
+
+  double dpr = devicePixelRatio();
+  iBWidth = iWidth * dpr;
+  iBHeight = iHeight * dpr;
 
   refreshSurface();
 
   QPainter qPainter;
   qPainter.begin(this);
   QRect r = ev->rect();
-  QRect source(r.left(), r.top(), r.width(), r.height());
+  QRect source(r.left() * dpr, r.top() * dpr, r.width() * dpr, r.height() * dpr);
   QImage bits(cairo_image_surface_get_data(iSurface),
-	      iWidth, iHeight, QImage::Format_RGB32);
+	      iBWidth, iBHeight, QImage::Format_RGB32);
   qPainter.drawImage(r, bits, source);
-  qPainter.translate(-1, -1);
   if (iFifiVisible)
     drawFifi(qPainter);
   if (iPage) {
