@@ -5,7 +5,7 @@
 /*
 
     This file is part of the extensible drawing editor Ipe.
-    Copyright (c) 1993-2020 Otfried Cheong
+    Copyright (c) 1993-2023 Otfried Cheong
 
     Ipe is free software; you can redistribute it and/or modify it
     under the terms of the GNU General Public License as published by
@@ -73,11 +73,7 @@ namespace ipe {
   class Canvas : public QWidget, public CanvasBase {
     Q_OBJECT
   public:
-#if QT_VERSION < 0x050000
-    Canvas(QWidget* parent, Qt::WFlags f=0);
-#else
-    Canvas(QWidget* parent, Qt::WindowFlags f=nullptr);
-#endif
+    Canvas(QWidget* parent, Qt::WindowFlags f=Qt::Widget);
 
     virtual void setCursor(TCursor cursor, double w = 1.0,
 			   Color *color = nullptr);
