@@ -4,7 +4,7 @@
 /*
 
     This file is part of the extensible drawing editor Ipe.
-    Copyright (c) 1993-2020 Otfried Cheong
+    Copyright (c) 1993-2023 Otfried Cheong
 
     Ipe is free software; you can redistribute it and/or modify it
     under the terms of the GNU General Public License as published by
@@ -240,7 +240,7 @@ void Canvas::wndPaint()
     drawFifi(cr);
 
   if (iPage) {
-    CairoPainter cp(iCascade, iFonts.get(), cr, iZoom, false);
+    CairoPainter cp(iCascade, iFonts.get(), cr, iZoom, false, false);
     cp.transform(canvasTfm());
     cp.pushMatrix();
     drawTool(cp);
