@@ -4,7 +4,7 @@
 /*
 
     This file is part of the extensible drawing editor Ipe.
-    Copyright (c) 1993-2020 Otfried Cheong
+    Copyright (c) 1993-2023 Otfried Cheong
 
     Ipe is free software; you can redistribute it and/or modify it
     under the terms of the GNU General Public License as published by
@@ -118,6 +118,10 @@ LRESULT CALLBACK PdfView::wndProc(HWND hwnd, UINT message, WPARAM wParam,
       SetWindowLongPtr(hwnd, GWLP_USERDATA, (LONG_PTR) view);
     }
     break;
+  case WM_COMMAND:
+    if (view)
+      PostMessage(view->target, WM_COMMAND, wParam, lParam);
+    break;
   case WM_PAINT:
     if (view)
       view->wndPaint();
