@@ -3,7 +3,7 @@
 /*
 
     This file is part of the extensible drawing editor Ipe.
-    Copyright (c) 1993-2019 Otfried Cheong
+    Copyright (c) 1993-2023 Otfried Cheong
 
     Ipe is free software; you can redistribute it and/or modify it
     under the terms of the GNU General Public License as published by
@@ -44,11 +44,14 @@
 @property ipe::Thumbnail *thumb;
 @property int page;
 @property NSSize tnSize;
+@property NSMutableArray <NSNumber *> *marks;
 
 - (int) count;
 - (NSString *) title:(int) index;
 - (NSImage *) image:(int) index;
+- (BOOL) marked:(int) index;
 
+- (void) createMarks;
 - (ipe::Buffer) renderImage:(int) index;
 - (NSImage *) createImage:(ipe::Buffer) b;
 
