@@ -4,7 +4,7 @@
 /*
 
     This file is part of the extensible drawing editor Ipe.
-    Copyright (c) 1993-2020 Otfried Cheong
+    Copyright (c) 1993-2023 Otfried Cheong
 
     Ipe is free software; you can redistribute it and/or modify it
     under the terms of the GNU General Public License as published by
@@ -99,6 +99,12 @@ static INT_PTR CALLBACK dialogProc(HWND hwnd, UINT message,
     SendMessage(h, LVM_SETIMAGELIST, (WPARAM) LVSIL_NORMAL, (LPARAM) d->hImageList);
     populateView(h, d);
     return (INT_PTR) TRUE; }
+  case WM_COMMAND:
+    if (LOWORD(wParam) == IDCANCEL) {
+      EndDialog(hwnd, FALSE);
+      return TRUE;
+    }
+    return FALSE;
   case WM_NOTIFY:
     switch (((LPNMHDR)lParam)->code) {
     case LVN_ITEMACTIVATE:
@@ -165,7 +171,7 @@ int showPageSelectDialog(int width, int height, const char * title,
   t.push_back(0);
   t.push_back(0);
 
-  int res = DialogBoxIndirectParamW(nullptr, (LPCDLGTEMPLATE) &t[0],
+  int res = DialogBoxIndirectParamW(nullptr, (LPCDLGTEMPLATE) t.data(),
 				    nullptr, dialogProc, (LPARAM) &sData);
   return res;
 }
