@@ -4,7 +4,7 @@
 /*
 
     This file is part of the extensible drawing editor Ipe.
-    Copyright (c) 1993-2019 Otfried Cheong
+    Copyright (c) 1993-2023 Otfried Cheong
 
     Ipe is free software; you can redistribute it and/or modify it
     under the terms of the GNU General Public License as published by
@@ -348,7 +348,7 @@ static bool extractPdf(DataSource &source, std::FILE *out)
     return false;
   }
 
-  const PdfObj *type = obj->dict()->get("Type", nullptr);
+  const PdfObj *type = obj->dict()->get("Type");
   if (!type || !type->name() || type->name()->value() != "Ipe") {
     fprintf(stderr, "Input file does not contain an Ipe XML stream.\n");
     return false;
