@@ -4,7 +4,7 @@
 --[[
 
     This file is part of the extensible drawing editor Ipe.
-    Copyright (c) 1993-2020 Otfried Cheong
+    Copyright (c) 1993-2023 Otfried Cheong
 
     Ipe is free software; you can redistribute it and/or modify it
     under the terms of the GNU General Public License as published by
@@ -201,12 +201,21 @@ function preciseBox(model)
     return
   end
   local sx, sy = tonumber(ssx), tonumber(ssy)
-  local shape = { boxshape(V(0,0), V(sx * dpmm, sy * dpmm)) }
+  local corner = V(sx * dpmm, sy * dpmm)
+  local origin = V(0,0)
+  if model.snap.with_axes then
+    origin = model.snap.origin
+  end
+  local shape = { boxshape(origin, origin + corner) }
   local obj = ipe.Path(model.attributes, shape)
   model:creation("create precise box", obj)
 end
 
 function boundingBox(model)
+  if not model:page():hasSelection() then
+    model.ui:explain("no selection")
+    return
+  end
   local box = bounding_box(model:page())
   local shape = { boxshape(box:bottomLeft(), box:topRight()) }
   local obj = ipe.Path(model.attributes, shape)
