@@ -4,7 +4,7 @@
 /*
 
     This file is part of the extensible drawing editor Ipe.
-    Copyright (c) 1993-2019 Otfried Cheong
+    Copyright (c) 1993-2023 Otfried Cheong
 
     Ipe is free software; you can redistribute it and/or modify it
     under the terms of the GNU General Public License as published by
@@ -39,6 +39,10 @@ extern "C" {
 
 using namespace ipe;
 
+// some slack when deciding if facets are vertical,
+// to handle degenerate situations
+static constexpr double eps = 1e-5;
+
 // --------------------------------------------------------------------
 
 struct DelaunayEdge {
@@ -228,9 +232,9 @@ void VoronoiIpelet::addInfiniteEdge(facetT *facet, facetT *neighbor)
   Vector dir;
   Vector v = voronoi_vertex(facet);
 
-  if (neighbor->normal[2] == 0.0) {
+  if (fabs(neighbor->normal[2]) < eps) {
     // neighboring facet is vertical
-    dir = Vector(neighbor->normal[0], neighbor->normal[2]);
+    dir = Vector(neighbor->normal[0], neighbor->normal[1]);
   } else {
     dir = v - voronoi_vertex(neighbor);
   }
@@ -247,11 +251,11 @@ void VoronoiIpelet::voronoiTreatFacet(qhT *qh, facetT *facet)
   if (facet == qh_MERGEridge) return;
   if (facet == qh_DUPLICATEridge) return;
 
-  if (iVoronoiSign * facet->normal[2] >= 0.0) return;
+  if (iVoronoiSign * facet->normal[2] >= -eps) return;
 
   FOREACHneighbor_(facet) {
     if (neighbor != qh_MERGEridge && neighbor != qh_DUPLICATEridge) {
-      if (iVoronoiSign * neighbor->normal[2] < 0.0) {
+      if (iVoronoiSign * neighbor->normal[2] < -eps) {
 	// make Voronoi edge between the two facets
 	addVoronoiEdge(facet, neighbor);
       } else {
