@@ -4,7 +4,7 @@
 /*
 
     This file is part of the extensible drawing editor Ipe.
-    Copyright (c) 1993-2020 Otfried Cheong
+    Copyright (c) 1993-2023 Otfried Cheong
 
     Ipe is free software; you can redistribute it and/or modify it
     under the terms of the GNU General Public License as published by
@@ -426,12 +426,20 @@ void Document::saveAsXml(Stream &stream, bool usePdfBitmaps) const
     infoStr.putXmlString(iProperties.iKeywords);
     infoStr << "\"";
   }
+  if (!iProperties.iLanguage.empty()) {
+    infoStr << " language=\"";
+    infoStr.putXmlString(iProperties.iLanguage);
+    infoStr << "\"";
+  }
   if (iProperties.iFullScreen) {
     infoStr << " pagemode=\"fullscreen\"";
   }
   if (iProperties.iNumberPages) {
     infoStr << " numberpages=\"yes\"";
   }
+  if (iProperties.iSequentialText) {
+    infoStr << " sequentialtext=\"yes\"";
+  }
   switch (iProperties.iTexEngine) {
   case LatexType::Pdftex:
     infoStr << " tex=\"pdftex\"";
@@ -596,33 +604,34 @@ Page *Document::remove(int no)
 // --------------------------------------------------------------------
 
 //! Run PdfLatex or Xelatex
-int Document::runLatex(String docname, String &texLog)
+int Document::runLatexAsync(String docname, String &texLog, Latex **pConverter)
 {
+  *pConverter = nullptr;
   texLog = "";
-  Latex converter(cascade(), iProperties.iTexEngine);
-
+  std::unique_ptr<Latex> converter(new Latex(cascade(),
+					     iProperties.iTexEngine,
+					     iProperties.iSequentialText));
   AttributeSeq seq;
   cascade()->allNames(ESymbol, seq);
 
   for (AttributeSeq::iterator it = seq.begin(); it != seq.end(); ++it) {
     const Symbol *sym = cascade()->findSymbol(*it);
     if (sym)
-      converter.scanObject(sym->iObject);
+      converter->scanObject(sym->iObject);
   }
 
   int count = 0;
   for (int i = 0; i < countPages(); ++i)
-    count = converter.scanPage(page(i));
-  if (count == 0)
-    return ErrNoText;
+    count = converter->scanPage(page(i));
 
   if (iProperties.iNumberPages) {
     for (int i = 0; i < countPages(); ++i) {
       int nviews = page(i)->countViews();
       for (int j = 0; j < nviews; ++j)
-	converter.addPageNumber(i, j, countPages(), nviews);
+	converter->addPageNumber(i, j, countPages(), nviews);
     }
-  }
+  } else if (count == 0)
+    return ErrNoText;
 
   // First we need a directory
   String latexDir = Platform::latexDirectory();
@@ -634,12 +643,13 @@ int Document::runLatex(String docname, String &texLog)
   String logFile = latexDir + "ipetemp.log";
 
   std::remove(logFile.z());
+  std::remove(pdfFile.z());
 
   std::FILE *file = Platform::fopen(texFile.z(), "wb");
   if (!file)
     return ErrWritingSource;
   FileStream stream(file);
-  int err = converter.createLatexSource(stream, properties().iPreamble);
+  int err = converter->createLatexSource(stream, properties().iPreamble);
   std::fclose(file);
 
   if (err < 0)
@@ -652,11 +662,7 @@ int Document::runLatex(String docname, String &texLog)
 
   // Check log file for Pdflatex version and errors
   texLog = Platform::readFile(logFile);
-  if (texLog.left(14) != "This is pdfTeX" &&
-      texLog.left(15) != "This is pdfeTeX" &&
-      texLog.left(13) != "This is XeTeX" &&
-      texLog.left(14) != "This is LuaTeX" &&
-      texLog.left(22) != "entering extended mode")
+  if (!texLog.hasPrefix("This is ") && !texLog.hasPrefix("entering extended mode"))
     return ErrRunLatex;
   int i = texLog.find('\n');
   if (i < 0)
@@ -671,17 +677,35 @@ int Document::runLatex(String docname, String &texLog)
   if (!pdfF)
     return ErrLatex;
   FileSource source(pdfF);
-  bool okay = (converter.readPdf(source) && converter.updateTextObjects());
+  bool okay = converter->readPdf(source);
   std::fclose(pdfF);
-
   if (okay) {
-    setResources(converter.takeResources());
-    // resources()->show();
+    *pConverter = converter.release();
     return ErrNone;
   } else
     return ErrLatexOutput;
 }
 
+int Document::completeLatexRun(Latex *converter)
+{
+  bool okay = converter->updateTextObjects();
+  if (okay) {
+    setResources(converter->takeResources());
+    // resources()->show();
+  }
+  delete converter;
+  return okay ? ErrNone : ErrLatexOutput;
+}
+
+int Document::runLatex(String docname, String &texLog)
+{
+  Latex *converter;
+  int err = runLatexAsync(docname, texLog, &converter);
+  if (err)
+    return err;
+  return completeLatexRun(converter);
+}
+
 //! Run Pdflatex (suitable for console applications)
 /*! Success/error is reported on stderr. */
 int Document::runLatex(String docname)
