@@ -4,7 +4,7 @@
 /*
 
     This file is part of the extensible drawing editor Ipe.
-    Copyright (c) 1993-2020 Otfried Cheong
+    Copyright (c) 1993-2023 Otfried Cheong
 
     Ipe is free software; you can redistribute it and/or modify it
     under the terms of the GNU General Public License as published by
@@ -59,15 +59,10 @@ inline double sq(double x)
 
 // --------------------------------------------------------------------
 
-struct _bezctx {
-  Vector pos;
-  std::vector<Bezier> &bez;
-};
-
-
+extern "C" {  // old version of the header does not have cplusplus wrapper
 #include "spiroentrypoints.h"
-#include "bezctx_intf.h"
-#include "spiro.h"
+#include "bezctx.h"
+}
 
 // --------------------------------------------------------------------
 
@@ -825,37 +820,60 @@ void Bezier::cardinalSpline(int n,  const Vector *v, double tension, std::vector
 // ------------------------------------------------------------------------------------------
 // These functions are called by spiro to generate Bezier output
 
-void bezctx_moveto(bezctx *bc, double x, double y, int is_open) {
-  // ipeDebug("bezctx_moveto(%g,%g)_%d\n",x,y,is_open);
+struct BezierContext {
+  struct _bezctx base;
+  Vector pos;
+  std::vector<Bezier> &bez;
+
+  BezierContext(const Vector &pos, std::vector<Bezier> &bez);
+};
+
+extern "C" void bezctx_ipe_moveto(bezctx *bc0, double x, double y, int is_open) {
+  // ipeDebug("bezctx_moveto(%g,%g)_%d",x,y,is_open);
+  BezierContext *bc = (BezierContext *) bc0;
   bc->pos = Vector(x, y);
 }
 
-void bezctx_lineto(bezctx *bc, double x, double y) {
-  // ipeDebug("bezctx_lineto(%g,%g)\n",x,y);
+extern "C" void bezctx_ipe_lineto(bezctx *bc0, double x, double y) {
+  // ipeDebug("bezctx_lineto(%g,%g)",x,y);
+  BezierContext *bc = (BezierContext *) bc0;
   Vector pos{x, y};
   bc->bez.push_back(Bezier(bc->pos, bc->pos, pos, pos));
   bc->pos = pos;
 }
 
 // most likely this will not be called with our settings
-void bezctx_quadto(bezctx *bc, double x1, double y1, double x2, double y2) {
-  // ipeDebug("bezctx_quadto(%g,%g, %g,%g)\n",x1,y1,x2,y2);
+extern "C" void bezctx_ipe_quadto(bezctx *bc0, double x1, double y1,
+			      double x2, double y2) {
+  // ipeDebug("bezctx_quadto(%g,%g, %g,%g)",x1,y1,x2,y2);
+  BezierContext *bc = (BezierContext *) bc0;
   Vector cp1{x1, y1}, cp2{x2, y2};
   bc->bez.push_back(Bezier::quadBezier(bc->pos, cp1, cp2));
   bc->pos = cp2;
 }
 
-void bezctx_curveto(bezctx *bc, double x1, double y1, double x2, double y2,
-		    double x3, double y3) {
-  // ipeDebug("bezctx_curveto(%g,%g, %g,%g, %g,%g)\n",x1,y1,x2,y2,x3,y3);
+extern "C" void bezctx_ipe_curveto(bezctx *bc0, double x1, double y1,
+			       double x2, double y2, double x3, double y3) {
+  // ipeDebug("bezctx_curveto(%g,%g, %g,%g, %g,%g)",x1,y1,x2,y2,x3,y3);
+  BezierContext *bc = (BezierContext *) bc0;
   Vector cp1{x1, y1}, cp2{x2, y2}, cp3{x3, y3};
   bc->bez.push_back(Bezier(bc->pos, cp1, cp2, cp3));
   bc->pos = cp3;
 }
 
 // apparently this is called to indicate when the curve reaches each knot
-void bezctx_mark_knot(bezctx *bc, int knot_idx) {
-  // ipeDebug("bezctx_mark_knot()_%d\n",knot_idx);
+extern "C" void bezctx_ipe_mark_knot(bezctx *bc, int knot_idx) {
+  // ipeDebug("bezctx_mark_knot: %d",knot_idx);
+}
+
+BezierContext::BezierContext(const Vector &pos, std::vector<Bezier> &bez) :
+  pos{pos}, bez{bez}
+{
+  base.moveto = bezctx_ipe_moveto;
+  base.lineto = bezctx_ipe_lineto;
+  base.quadto = bezctx_ipe_quadto;
+  base.curveto = bezctx_ipe_curveto;
+  base.mark_knot = bezctx_ipe_mark_knot;
 }
 
 // ------------------------------------------------------------------------------------------
@@ -873,12 +891,17 @@ void Bezier::spiroSpline(int n,  const Vector *v, std::vector<Bezier> &result)
     return;
   }
 
-  bezctx context{v[0], result};
+  BezierContext context{v[0], result};
+#ifdef SPIRO_CUBIC_TO_BEZIER
   if (SpiroCPsToBezier2(spiroCp.data(), spiroCp.size(),
-			SPIRO_CUBIC_TO_BEZIER, false, &context) != 1 ) {
+			SPIRO_CUBIC_TO_BEZIER, false, (bezctx *) &context) != 1 ) {
     // spiro failed, use another spline instead
     Bezier::spline(n, v, result);
   }
+#else
+  // old version of libspiro
+  SpiroCPsToBezier0(spiroCp.data(), spiroCp.size(), false, (bezctx *) &context);
+#endif
 }
 
 //! Return distance to Bezier spline.
@@ -1034,22 +1057,23 @@ static Vector nearbyBezierIntersection(const Vector &q, const Bezier &a, const B
 
   const gsl_multimin_fminimizer_type *T = gsl_multimin_fminimizer_nmsimplex;
   gsl_multimin_fminimizer *s = gsl_multimin_fminimizer_alloc(T, 2);
-  gsl_multimin_fminimizer_set(s, &minFunc, x, ss);
+  int status = gsl_multimin_fminimizer_set(s, &minFunc, x, ss);
 
-  size_t iter = 0;
-  int status;
-  do {
-    iter++;
-    status = gsl_multimin_fminimizer_iterate(s);
-    if (status) break;
-    double size = gsl_multimin_fminimizer_size(s);
-    status = gsl_multimin_test_size(size, 1e-4);
+  if (status == GSL_SUCCESS) {
+    size_t iter = 0;
+    do {
+      iter++;
+      status = gsl_multimin_fminimizer_iterate(s);
+      if (status) break;
+      double size = gsl_multimin_fminimizer_size(s);
+      status = gsl_multimin_test_size(size, 1e-4);
 #ifdef GSL_OUTPUT
-    if (status == GSL_SUCCESS) ipeDebug("Minimum obtained");
-    ipeDebug("minimize: %3ld t1=%.9f t2=%.9f sqDist=%.3e size=%.3e", iter,
-	     gsl_vector_get(s->x, 0), gsl_vector_get(s->x, 1), s->fval, size);
+      if (status == GSL_SUCCESS) ipeDebug("Minimum obtained");
+      ipeDebug("minimize: %3ld t1=%.9f t2=%.9f sqDist=%.3e size=%.3e", iter,
+	       gsl_vector_get(s->x, 0), gsl_vector_get(s->x, 1), s->fval, size);
 #endif
-  } while (status == GSL_CONTINUE && iter < 100);
+    } while (status == GSL_CONTINUE && iter < 100);
+  }
   Vector result = (status == GSL_SUCCESS) ? a.point(gsl_vector_get(s->x, 0)) : q;
   gsl_vector_free(x);
   gsl_vector_free(ss);
@@ -1079,11 +1103,12 @@ static Vector nearbyBezierLineIntersection(const Vector &q, const Bezier &a, con
 
   const gsl_min_fminimizer_type *T = gsl_min_fminimizer_brent;
   gsl_min_fminimizer *s = gsl_min_fminimizer_alloc(T);
-  gsl_min_fminimizer_set(s, &minFunc, (q-a.iV[0]).len() / (a.iV[3] - a.iV[0]).len(), 0.0, 1.0);
 
-  size_t iter = 0;
-  int status;
-  do {
+  double t = (q-a.iV[0]).len() / (a.iV[3] - a.iV[0]).len();
+  int status = gsl_min_fminimizer_set(s, &minFunc, t, 0.0, 1.0);
+  if (status == GSL_SUCCESS) {
+    size_t iter = 0;
+    do {
       iter++;
       status = gsl_min_fminimizer_iterate(s);
       if (status) break;
@@ -1094,7 +1119,9 @@ static Vector nearbyBezierLineIntersection(const Vector &q, const Bezier &a, con
       if (status == GSL_SUCCESS) ipeDebug("Minimum obtained");
       ipeDebug("minimize: %3ld [%.7f, %.7f] %.7f", iter, x0, x1, gsl_min_fminimizer_x_minimum(s));
 #endif
-  } while (status == GSL_CONTINUE && iter < 100);
+    } while (status == GSL_CONTINUE && iter < 100);
+  }
+
   Vector result = (status == GSL_SUCCESS) ? a.point(gsl_min_fminimizer_x_minimum(s)) : q;
   gsl_min_fminimizer_free(s);
   return result;
@@ -1122,12 +1149,14 @@ static Vector nearbyBezierEllipseIntersection(const Vector &q, const Bezier &a,
 
   const gsl_min_fminimizer_type *T = gsl_min_fminimizer_brent;
   gsl_min_fminimizer *s = gsl_min_fminimizer_alloc(T);
-  gsl_min_fminimizer_set(s, &minFunc, (q-a.iV[0]).len() / (a.iV[3] - a.iV[0]).len(), 0.0, 1.0);
 
-  size_t iter = 0;
-  int status;
-  do {
+  double t = (q-a.iV[0]).len() / (a.iV[3] - a.iV[0]).len();
+  int status = gsl_min_fminimizer_set(s, &minFunc, t, 0.0, 1.0);
+  if (status == GSL_SUCCESS) {
+    size_t iter = 0;
+    do {
       iter++;
+      ipeDebug("About it iterate");
       status = gsl_min_fminimizer_iterate(s);
       if (status) break;
       double x0 = gsl_min_fminimizer_x_lower(s);
@@ -1137,7 +1166,9 @@ static Vector nearbyBezierEllipseIntersection(const Vector &q, const Bezier &a,
       if (status == GSL_SUCCESS) ipeDebug("Minimum obtained");
       ipeDebug("minimize: %3ld [%.7f, %.7f] %.7f", iter, x0, x1, gsl_min_fminimizer_x_minimum(s));
 #endif
-  } while (status == GSL_CONTINUE && iter < 100);
+    } while (status == GSL_CONTINUE && iter < 100);
+  }
+
   Vector result = (status == GSL_SUCCESS) ? a.point(gsl_min_fminimizer_x_minimum(s)) : q;
   gsl_min_fminimizer_free(s);
   return result;
@@ -1433,10 +1464,13 @@ static void ellipseIntersections(const Matrix &m, double ysign, std::vector<Vect
   gsl_poly_complex_workspace *w = gsl_poly_complex_workspace_alloc(deg+1);
 
   double z[8];
-  gsl_poly_complex_solve(coeff, deg+1, w, z);
+  int status = gsl_poly_complex_solve(coeff, deg+1, w, z);
 
   gsl_poly_complex_workspace_free(w);
 
+  if (status != GSL_SUCCESS)
+    return;  // report no intersections
+
   for (int i = 0; i < deg; i++) {
 #ifdef GSL_OUTPUT
     ipeDebug("z%d = %+.18f %+.18f", i, z[2*i], z[2*i+1]);
