@@ -4,7 +4,7 @@
 /*
 
     This file is part of the extensible drawing editor Ipe.
-    Copyright (c) 1993-2019 Otfried Cheong
+    Copyright (c) 1993-2023 Otfried Cheong
 
     Ipe is free software; you can redistribute it and/or modify it
     under the terms of the GNU General Public License as published by
@@ -64,7 +64,7 @@ Group::Group(const XmlAttributes &attr)
   String str;
   if (attr.has("clip", str)) {
     Shape clip;
-    if (clip.load(str))
+    if (clip.load(str) && clip.countSubPaths() > 0)
       iClip = clip;
   }
   iUrl = attr["url"];
