@@ -4,7 +4,7 @@
 /*
 
     This file is part of the extensible drawing editor Ipe.
-    Copyright (c) 1993-2019 Otfried Cheong
+    Copyright (c) 1993-2023 Otfried Cheong
 
     Ipe is free software; you can redistribute it and/or modify it
     under the terms of the GNU General Public License as published by
@@ -110,8 +110,10 @@ int ImlParser::parseDocument(Document &doc)
     properties.iAuthor = att["author"];
     properties.iSubject = att["subject"];
     properties.iKeywords = att["keywords"];
+    properties.iLanguage = att["language"];
     properties.iFullScreen = (att["pagemode"] == "fullscreen");
     properties.iNumberPages = (att["numberpages"] == "yes");
+    properties.iSequentialText = (att["sequentialtext"] == "yes");
     properties.iCreated = att["created"];
     properties.iModified = att["modified"];
     String tex = att["tex"];
@@ -239,6 +241,16 @@ bool ImlParser::parsePage(Page &page)
     page.addLayer(att["name"]);
     if (att["edit"] == "no")
       page.setLocked(page.countLayers() - 1, true);
+    String snapMode;
+    if (att.has("snap", snapMode)) {
+      if (snapMode == "never")
+	page.setSnapping(page.countLayers() - 1, Page::SnapMode::Never);
+      else if (snapMode == "always")
+	page.setSnapping(page.countLayers() - 1, Page::SnapMode::Always);
+    }
+    String data;
+    if (att.has("data", data))
+      page.setLayerData(page.countLayers() - 1, data);
     tag = parseToTag();
   }
   // default layer: 'alpha'
@@ -277,6 +289,12 @@ bool ImlParser::parsePage(Page &page)
     if (att.has("name", str))
       page.setViewName(page.countViews() - 1, str);
 
+    if (!att.slash()) {
+      AttributeMap map;
+      if (!parseView(page, map))
+	return false;
+      page.setViewMap(page.countViews() - 1, map);
+    }
     tag = parseToTag();
   }
 
@@ -339,42 +357,27 @@ Page *ImlParser::parsePageSelection()
   return page.release();
 }
 
-#if 0
-//! parse an \c <ipeselection> element.
-/*! An PgObjectSeq is used to own the objects, but selection mode
-  and layer are not set. */
-bool ImlParser::parseSelection(PageObjectSeq &seq)
-{
-  String tag = parseToTag();
-  if (tag != "ipeselection")
-    return false;
-  XmlAttributes attr;
-  if (!parseAttributes(attr))
-    return nullptr;
-  tag = parseToTag();
-
-  while (tag == "bitmap") {
-    if (!parseBitmap())
-      return false;
-    tag = parseToTag();
-  }
-
-  for (;;) {
-    if (tag == "/ipeselection")
-      return true;
-    Object *obj = parseObject(tag);
-    if (!obj)
-      return false;
-    seq.push_back(PageObject(ENotSelected, 0, obj));
-    tag = parseToTag();
-  }
-}
-#endif
-
 //! parse an Object.
 /*! On calling, stream must be just past the tag. */
 Object *ImlParser::parseObject(String tag, Page *page,
 			       int *currentLayer)
+{
+  String layer;
+  Object *obj = parseObject(tag, layer);
+  if (obj && page && currentLayer && !layer.empty()) {
+    for (int i = 0; i < page->countLayers(); ++i) {
+      if (page->layer(i) == layer) {
+	*currentLayer = i;
+	break;
+      }
+    }
+  }
+  return obj;
+}
+
+//! parse an Object.
+/*! On calling, stream must be just past the tag. */
+Object *ImlParser::parseObject(String tag, String &layer)
 {
   if (tag[0] == '/')
     return nullptr;
@@ -383,15 +386,9 @@ Object *ImlParser::parseObject(String tag, Page *page,
   if (!parseAttributes(attr))
     return nullptr;
 
-  String layer;
-  if (page && currentLayer && attr.has("layer", layer)) {
-    for (int i = 0; i < page->countLayers(); ++i) {
-      if (page->layer(i) == layer) {
-	*currentLayer = i;
-	break;
-      }
-    }
-  }
+  String l;
+  if (attr.has("layer", l))
+    layer = l;
 
   if (tag == "group") {
     Group group(attr);
@@ -433,6 +430,61 @@ static inline bool symbolName(String s)
 			 ('A' <= s[0] && s[0] <= 'Z')));
 }
 
+//! Parse an attribute map.
+/*! On calling, stream must be before the first mapping element. */
+bool ImlParser::parseView(Page &page, AttributeMap &map)
+{
+  String tag = parseToTag();
+  while (tag != "/view") {
+    if (tag == "transform") {
+      XmlAttributes att;
+      if (!parseAttributes(att) || !att.slash())
+	return false;
+      String layer = att["layer"];
+      String mstr;
+      if (layer.empty() || !att.has("matrix", mstr))
+	return false;
+      int layerNum = page.findLayer(layer);
+      if (layerNum < 0)
+	return false;
+      page.setLayerMatrix(page.countViews() - 1, layerNum, Matrix(mstr));
+    } else {
+      XmlAttributes att;
+      if (!parseAttributes(att) || !att.slash())
+	return false;
+      String from = att["from"];
+      String kindStr = att["kind"];
+      Kind kind;
+      if (tag != "map") {  // deprecated format (briefly allowed in 7.2.16)
+	kindStr = tag;
+	from = att["map"];
+      }
+      if (kindStr == "pen")
+	kind = EPen;
+      else if (kindStr == "symbolsize")
+	kind = ESymbolSize;
+      else if (kindStr == "arrowsize")
+	kind = EArrowSize;
+      else if (kindStr == "opacity")
+	kind = EOpacity;
+      else if (kindStr == "color")
+	kind = EColor;
+      else if (kindStr == "dashstyle")
+	kind = EDashStyle;
+      else if (kindStr == "symbol")
+	kind = ESymbol;
+      else
+	return false; // error
+      String to = att["to"];
+      if (from.empty() || to.empty())
+	return false;
+      map.add({ kind, Attribute(true, from), Attribute(true, to) });
+    }
+    tag = parseToTag();
+  }
+  return true;
+}
+
 //! Parse a style sheet.
 /*! On calling, stream must be just past the style tag. */
 bool ImlParser::parseStyle(StyleSheet &sheet)
@@ -699,10 +751,11 @@ bool ImlParser::parseStyle(StyleSheet &sheet)
 					      Attribute::NORMAL());
       if (name.empty() || value.isSymbolic())
 	return false;
-      if (kind == EGridSize &&
-	  (!value.isNumber() || !value.number().isInteger()))
-	return false; // refuse non-integer gridsize
-      sheet.add(kind, Attribute(true, name), value);
+      if ((kind == EGridSize || kind == EAngleSize) && !value.isNumber())
+	return false;
+      // silently ignore non-integer grid size
+      if (kind != EGridSize || value.number().isInteger())
+	sheet.add(kind, Attribute(true, name), value);
     }
     tag = parseToTag();
   }
