@@ -4,7 +4,7 @@
 /*
 
     This file is part of the extensible drawing editor Ipe.
-    Copyright (c) 1993-2019 Otfried Cheong
+    Copyright (c) 1993-2023 Otfried Cheong
 
     Ipe is free software; you can redistribute it and/or modify it
     under the terms of the GNU General Public License as published by
@@ -47,12 +47,13 @@ using namespace ipe;
 */
 
 //! Create a converter object.
-Latex::Latex(const Cascade *sheet, LatexType latexType)
+Latex::Latex(const Cascade *sheet, LatexType latexType, bool sequentialText)
 {
   iCascade = sheet;
   iResources = new PdfResources;
   iLatexType = latexType;
   iXetex = (latexType == LatexType::Xetex);
+  iSequentialText = sequentialText;
 }
 
 //! Destructor.
@@ -181,16 +182,12 @@ void Latex::addPageNumber(int pno, int vno, int npages, int nviews)
 */
 int Latex::createLatexSource(Stream &stream, String preamble)
 {
-  bool ancient = (getenv("IPEANCIENTPDFTEX") != nullptr);
   int count = 0;
   stream << "\\nonstopmode\n";
   if (!iXetex) {
+    // for parser testing: \\pdfobjcompresslevel2\\pdfminorversion5
     stream << "\\expandafter\\ifx\\csname pdfobjcompresslevel\\endcsname"
 	   << "\\relax\\else\\pdfobjcompresslevel0\\fi\n";
-    if (!ancient && iLatexType != LatexType::Luatex)
-      stream << "\\ifnum\\the\\pdftexversion<140"
-	     << "\\errmessage{Pdftex is too old. "
-	     << "Set IPEANCIENTPDFTEX environment variable!}\\fi\n";
     if (iLatexType == LatexType::Luatex)
       // load luatex85 for new versions of Luatex
       stream << "\\expandafter\\ifx\\csname pdfcolorstack\\endcsname\\relax"
@@ -223,15 +220,12 @@ int Latex::createLatexSource(Stream &stream, String preamble)
   if (iXetex) {
     stream << "\\def\\ipesetcolor#1#2#3{\\special{pdf:bc [#1 #2 #3]}}\n"
 	   << "\\def\\iperesetcolor{\\special{pdf:ec}}\n";
-  } else if (!ancient) {
+  } else {
     stream << "\\makeatletter\n"
 	   << "\\def\\ipesetcolor#1#2#3{\\def\\current@color{#1 #2 #3 rg #1 #2 #3 RG}"
 	   << "\\pdfcolorstack\\@pdfcolorstack push{\\current@color}}\n"
 	   << "\\def\\iperesetcolor{\\pdfcolorstack\\@pdfcolorstack pop}\n"
 	   << "\\makeatother\n";
-  } else {
-    stream << "\\def\\ipesetcolor#1#2#3{\\color[rgb]{#1,#2,#3}}\n"
-	   << "\\def\\iperesetcolor{}\n";
   }
   stream << iCascade->findPreamble() << "\n"
 	 << preamble << "\n"
@@ -241,10 +235,13 @@ int Latex::createLatexSource(Stream &stream, String preamble)
 	 << "\\begin{document}\n"
 	 << "\\begin{picture}(500,500)\n";
 
-  int curnum = 1;
   if (iXetex)
     stream << "\\special{pdf:obj @ipeforms []}\n";
+
+  // generate Latex source for each text object
   for (auto &it : iTextObjects) {
+    StringStream source(it.iSource);
+
     const Text *text = it.iText;
 
     if (!text->getXForm())
@@ -253,24 +250,22 @@ int Latex::createLatexSource(Stream &stream, String preamble)
     Attribute fsAttr = iCascade->find(ETextSize, it.iSize);
 
     // compute x-stretch factor from textstretch
-    Fixed stretch(1);
+    it.iStretch = Fixed(1);
     if (it.iSize.isSymbolic())
-      stretch = iCascade->find(ETextStretch, it.iSize).number();
-
-    stream << "\\setbox0=\\hbox{";
+      it.iStretch = iCascade->find(ETextStretch, it.iSize).number();
     if (text->isMinipage()) {
-      stream << "\\begin{minipage}{" <<
-	text->width()/stretch.toDouble() << "bp}";
+      source << "\\begin{minipage}{" <<
+	text->width()/it.iStretch.toDouble() << "bp}";
     }
 
     if (fsAttr.isNumber()) {
       Fixed fs = fsAttr.number();
-      stream << "\\fontsize{" << fs << "}"
+      source << "\\fontsize{" << fs << "}"
 	     << "{" << fs.mult(6, 5) << "bp}\\selectfont\n";
     } else
-      stream << fsAttr.string() << "\n";
+      source << fsAttr.string() << "\n";
     Color col = iCascade->find(EColor, text->stroke()).color();
-    stream << "\\ipesetcolor{" << col.iRed.toDouble()
+    source << "\\ipesetcolor{" << col.iRed.toDouble()
 	   << "}{" << col.iGreen.toDouble()
 	   << "}{" << col.iBlue.toDouble()
 	   << "}%\n";
@@ -282,21 +277,35 @@ int Latex::createLatexSource(Stream &stream, String preamble)
     int sp = 0;
     while (sp < style.size() && style[sp] != '\0')
       ++sp;
-    stream << style.substr(0, sp);
+    source << style.substr(0, sp);
 
     String txt = text->text();
-    stream << txt;
+    source << txt;
 
     if (text->isMinipage()) {
       if (!txt.empty() && txt[txt.size() - 1] != '\n')
-	stream << "\n";
-      stream << style.substr(sp + 1);
-      stream << "\\end{minipage}";
+	source << "\n";
+      source << style.substr(sp + 1);
+      source << "\\end{minipage}";
     } else
-      stream << style.substr(sp + 1) << "%\n";
+      source << style.substr(sp + 1) << "%\n";
+  }
 
+  if (!iSequentialText)
+    std::sort(iTextObjects.begin(), iTextObjects.end(),
+	      [](const SText &a, const SText &b) {
+		return a.iSource < b.iSource;
+	      });
+
+  for (int i = 0; i < size(iTextObjects); ++i) {
+    auto & it = iTextObjects[i];
+    if (!iSequentialText && i > 0 && it.iSource == iTextObjects[i-1].iSource)
+      continue;
+    stream << "\\setbox0=\\hbox{";
+    stream << it.iSource;
     stream << "\\iperesetcolor}\n"
 	   << "\\count0=\\dp0\\divide\\count0 by \\bigpoint\n";
+    int curnum = i + 1;
     if (iXetex) {
       stream << "\\special{ pdf:bxobj @ipeform" << curnum << "\n"
 	     << "width \\the\\wd0 \\space "
@@ -306,7 +315,7 @@ int Latex::createLatexSource(Stream &stream, String preamble)
 	     << "\\special{pdf:exobj}%\n"
 	     << "\\special{pdf:obj @ipeinfo" << curnum << " <<"
 	     << " /IpeId " << curnum
-	     << " /IpeStretch " << stretch.toDouble()
+	     << " /IpeStretch " << it.iStretch.toDouble()
 	     << " /IpeDepth \\the\\count0"
 	     << " /IpeXForm @ipeform" << curnum << " >>}\n"
 	     << "\\special{pdf:close @ipeinfo" << curnum << "}\n"
@@ -314,11 +323,10 @@ int Latex::createLatexSource(Stream &stream, String preamble)
 	     << "\\put(0,0){\\special{pdf:uxobj @ipeform" << curnum << "}}\n";
     } else {
       stream << "\\pdfxform attr{/IpeId " << curnum
-	     << " /IpeStretch " << stretch.toDouble()
+	     << " /IpeStretch " << it.iStretch.toDouble()
 	     << " /IpeDepth \\the\\count0}"
 	     << "0\\put(0,0){\\pdfrefxform\\pdflastxform}\n";
     }
-    ++curnum;
   }
   stream << "\\end{picture}\n";
   if (iXetex)
@@ -343,7 +351,7 @@ bool Latex::getXForm(String key, const PdfDict *ipeInfo)
   */
   Text::XForm *xf = new Text::XForm;
   iXForms.push_back(xf);
-  const PdfObj *xform = iXetex ? ipeInfo->get("IpeXForm", nullptr) :
+  const PdfObj *xform = iXetex ? ipeInfo->get("IpeXForm") :
     iResources->findResource("XObject", key);
   int xformNum = -1;
   if (xform && xform->ref()) {
@@ -369,14 +377,14 @@ bool Latex::getXForm(String key, const PdfDict *ipeInfo)
     xf->iName = key;
     ipeInfo = xformd;
   }
-  double val;
   // Get  id
-  if (!ipeInfo->getNumber("IpeId", val, &iPdf))
+  int ipeId = ipeInfo->getInteger("IpeId", &iPdf);
+  int ipeDepth = ipeInfo->getInteger("IpeDepth", &iPdf);
+  if (ipeId < 0 || ipeDepth < 0)
     return false;
-  xf->iRefCount = val;  // abusing refcount field
-  if (!ipeInfo->getNumber("IpeDepth", val, &iPdf))
-    return false;
-  xf->iDepth = int(val);
+  xf->iRefCount = ipeId;  // abusing refcount field
+  xf->iDepth = ipeDepth;
+  double val;
   if (!ipeInfo->getNumber("IpeStretch", val, &iPdf))
     return false;
   xf->iStretch = val;
@@ -428,6 +436,10 @@ bool Latex::readPdf(DataSource &source)
       const PdfObj *info = obj->array()->obj(i, &iPdf);
       if (!info || !info->dict())
 	return false;
+      const PdfObj *ref = info->dict()->get("IpeXForm");
+      if (!ref || !ref->ref())
+	return false;
+      iResources->setIpeXForm(ref->ref()->value());
       if (!getXForm(String(), info->dict()))
 	return false;
     }
@@ -437,12 +449,17 @@ bool Latex::readPdf(DataSource &source)
       warn("Page 1 has no XForms.");
       return false;
     }
-    for (int i = 0; i < obj->dict()->count(); i++) {
-      String key = obj->dict()->key(i);
+    const PdfDict *xo = obj->dict();
+    for (int i = 0; i < xo->count(); i++) {
+      String key = xo->key(i);
+      if (!xo->value(i)->ref())
+	return false;
+      iResources->setIpeXForm(xo->value(i)->ref()->value());
       if (!getXForm(key, nullptr))
 	return false;
     }
   }
+  // iResources->show();
   return true;
 }
 
@@ -450,17 +467,27 @@ bool Latex::readPdf(DataSource &source)
 /*! Returns true if successful. */
 bool Latex::updateTextObjects()
 {
-  int curnum = 1;
-  for (auto &it : iTextObjects) {
-    auto xf = std::find_if(iXForms.begin(), iXForms.end(),
-			   [curnum](Text::XForm *f)
-			   { return f->iRefCount == curnum; } );
-    if (xf == iXForms.end())
-      return false;
-    Text::XForm *xform = *xf;
-    iXForms.erase(xf);
-    it.iText->setXForm(xform);
-    ++curnum;
+  std::sort(iXForms.begin(), iXForms.end(),
+	    [](const Text::XForm *a, const Text::XForm *b) {
+	      return a->iRefCount < b->iRefCount;
+	    });
+
+  int curXForm = 0;
+  Text::XForm *xf = nullptr;
+  for (int i = 0; i < size(iTextObjects); ++i) {
+    if (!iSequentialText && i > 0 && iTextObjects[i].iSource == iTextObjects[i-1].iSource) {
+      if (xf == nullptr)
+	return false;
+      iTextObjects[i].iText->setXForm(xf);
+    } else {
+      if (i + 1 != iXForms[curXForm]->iRefCount)
+	return false;
+      xf = iXForms[curXForm];
+      iXForms[curXForm] = nullptr;
+      xf->iRefCount = 0;
+      iTextObjects[i].iText->setXForm(xf);
+      ++curXForm;
+    }
   }
   return true;
 }
