@@ -4,7 +4,7 @@
 /*
 
     This file is part of the extensible drawing editor Ipe.
-    Copyright (C) 1993-2016  Otfried Cheong
+    Copyright (c) 1993-2023 Otfried Cheong
 
     Ipe is free software; you can redistribute it and/or modify it
     under the terms of the GNU General Public License as published by
@@ -43,14 +43,18 @@
    ipe "ipe" namespace, those that aren't start with the letters
    "Ipe".
 
+   Before using Ipelib in your own program, make sure to initialize
+   the library by calling ipe::Platform::initLib().
+
    Many of the Ipelib classes are also made available as Lua objects.
    \subpage lua describes the Lua bindings to Ipelib, to ipeui, and to
    the Ipe program itself.  The Ipe program itself is mostly written
    in Lua and uses these Lua bindings.
 
-   All filenames passed to Ipelib are assumed to be in the local
-   system's encoding.  All Lua strings are assumed to be UTF-8
-   (filenames are converted by the ipelua bindings).
+   On Unix, all filenames passed to Ipelib are assumed to be in the
+   local system's encoding.  On Windows, all filenames passed to
+   Ipelib are assumed to be UTF-8.  All Lua strings are assumed to be
+   UTF-8 (filenames are converted by the ipelua bindings).
 
    \subpage ipelets explains how to write ipelets, that is, extensions
    to Ipe.  Ipelets are either written in Lua or in C++ (using a small
@@ -177,6 +181,9 @@ Object::Object(const XmlAttributes &attr)
     else if (str == "translations")
       iTransformations = ETransformationsTranslations;
   }
+  iCustom = Attribute::UNDEFINED();
+  if (attr.has("custom", str) && !str.empty())
+    iCustom = Attribute(false, str);
 }
 
 /*! Create object by taking pinning/transforming from \a attr and
@@ -185,6 +192,7 @@ Object::Object(const AllAttributes &attr)
 {
   iPinned = attr.iPinned;
   iTransformations = attr.iTransformations;
+  iCustom = Attribute::UNDEFINED();
 }
 
 /*! Create object with identity matrix, no pinning, all transformations. */
@@ -192,6 +200,7 @@ Object::Object()
 {
   iPinned = ENoPin;
   iTransformations = ETransformationsAffine;
+  iCustom = Attribute::UNDEFINED();
 }
 
 //! Copy constructor.
@@ -200,6 +209,7 @@ Object::Object(const Object &rhs)
   iMatrix = rhs.iMatrix;
   iPinned = rhs.iPinned;
   iTransformations = rhs.iTransformations;
+  iCustom = rhs.iCustom;
 }
 
 //! Pure virtual destructor.
@@ -233,6 +243,8 @@ void Object::saveAttributesAsXml(Stream &stream, String layer) const
     stream << " transformations=\"translations\"";
   else if (iTransformations == ETransformationsRigidMotions)
     stream << " transformations=\"rigid\"";
+  if (iCustom != Attribute::UNDEFINED())
+    stream << " custom=\"" << iCustom.string() << "\"";
 }
 
 //! Return pointer to this object if it is an Group, nullptr otherwise.
@@ -339,6 +351,19 @@ Attribute Object::getAttribute(Property prop) const noexcept
   }
 }
 
+//! Set the 'custom' attribute (not used by Ipe, for users and ipelets)
+void Object::setCustom(Attribute value)
+{
+  assert(value.isString());
+  iCustom = value;
+}
+
+//! Return value of the 'custom' attribute
+Attribute Object::getCustom() const noexcept
+{
+  return iCustom;
+}
+
 // --------------------------------------------------------------------
 
 //! Check all symbolic attributes.
