@@ -4,7 +4,7 @@
 /*
 
     This file is part of the extensible drawing editor Ipe.
-    Copyright (c) 1993-2019 Otfried Cheong
+    Copyright (c) 1993-2023 Otfried Cheong
 
     Ipe is free software; you can redistribute it and/or modify it
     under the terms of the GNU General Public License as published by
@@ -128,8 +128,17 @@ void Page::saveAsXml(Stream &stream) const
   }
   for (int i = 0; i < countLayers(); ++i) {
     stream << "<layer name=\"" << iLayers[i].iName << "\"";
-    if (iLayers[i].iFlags & ELocked)
+    if (iLayers[i].locked)
       stream << " edit=\"no\"";
+    if (iLayers[i].snapMode == SnapMode::Never)
+      stream << " snap=\"never\"";
+    else if (iLayers[i].snapMode == SnapMode::Always)
+      stream << " snap=\"always\"";
+    if (!iLayers[i].iData.empty()) {
+      stream << " data=\"";
+      stream.putXmlString(iLayers[i].iData);
+      stream << "\"";
+    }
     stream << "/>\n";
   }
   for (int i = 0; i < countViews(); ++i) {
@@ -150,7 +159,18 @@ void Page::saveAsXml(Stream &stream) const
       stream << " marked=\"yes\"";
     if (!viewName(i).empty())
       stream << " name=\"" << viewName(i) << "\"";
-    stream << "/>\n";
+    const AttributeMap &map = viewMap(i);
+    if (map.count() == 0 && iViews[i].iLayerMatrices.empty()) {
+      stream << "/>\n";
+    } else {
+      stream << ">\n";
+      map.saveAsXml(stream);
+      for (const auto &s : iViews[i].iLayerMatrices) {
+	stream << "<transform layer=\"" << s.iLayer << "\" matrix=\""
+	       << s.iMatrix << "\"/>\n";
+      }
+      stream << "</view>\n";
+    }
   }
   int currentLayer = -1;
   for (ObjSeq::const_iterator it = iObjects.begin();
@@ -170,21 +190,26 @@ void Page::saveAsXml(Stream &stream) const
 Page::SLayer::SLayer(String name)
 {
   iName = name;
-  iFlags = 0;
+  locked = false;
+  snapMode = SnapMode::Visible;
+}
+
+//! Set free data field of the layer.
+void Page::setLayerData(int index, String data)
+{
+  iLayers[index].iData = data;
 }
 
 //! Set locking of layer \a i.
 void Page::setLocked(int i, bool flag)
 {
-  iLayers[i].iFlags &= ~ELocked;
-  if (flag) iLayers[i].iFlags |= ELocked;
+  iLayers[i].locked = flag;
 }
 
 //! Set snapping of layer \a i.
-void Page::setSnapping(int i, bool flag)
+void Page::setSnapping(int i, SnapMode mode)
 {
-  iLayers[i].iFlags &= ~ENoSnapping;
-  if (!flag) iLayers[i].iFlags |= ENoSnapping;
+  iLayers[i].snapMode = mode;
 }
 
 //! Add a new layer.
@@ -341,7 +366,15 @@ Rect Page::viewBBox(const Cascade *sheet, int view) const
 bool Page::objSnapsInView(int objNo, int view) const noexcept
 {
   int layer = layerOf(objNo);
-  return hasSnapping(layer) && visible(view, layer);
+  switch (snapping(layer)) {
+  case SnapMode::Visible:
+    return visible(view, layer);
+  case SnapMode::Always:
+    return true;
+  case SnapMode::Never:
+  default:
+    return false;
+  }
 }
 
 // --------------------------------------------------------------------
@@ -432,6 +465,41 @@ int Page::findView(String s) const
   return -1;
 }
 
+//! Return matrices for all layers.
+std::vector<Matrix> Page::layerMatrices(int view) const
+{
+  std::vector<Matrix> m(countLayers());
+  for (int i = 0; i < size(iViews[view].iLayerMatrices); ++i) {
+    int l = findLayer(iViews[view].iLayerMatrices[i].iLayer);
+    if (l >= 0)
+      m[l] = iViews[view].iLayerMatrices[i].iMatrix;
+  }
+  return m;
+}
+
+//! Set matrix for the given layer in this view.
+void Page::setLayerMatrix(int view, int layer, const Matrix &m)
+{
+  String name = iLayers[layer].iName;
+  std::vector<SLayerMatrix> &ms = iViews[view].iLayerMatrices;
+  std::vector<SLayerMatrix>::iterator it =
+    std::find_if(ms.begin(), ms.end(), [&name](const SLayerMatrix &a) {return a.iLayer == name;});
+  if (m.isIdentity()) {
+    if (it != ms.end()) ms.erase(it);
+  } else {
+    if (it != ms.end())
+      it->iMatrix = m;
+    else
+      ms.push_back({name, m});
+  }
+}
+
+//! Set the attribute mapping for the view.
+void Page::setViewMap(int index, const AttributeMap &map)
+{
+  iViews[index].iAttributeMap = map;
+}
+
 // --------------------------------------------------------------------
 
 Page::SObject::SObject()
@@ -753,7 +821,7 @@ void Page::saveSelection(Stream &stream) const
   }
   for (int i = 0; i < count(); ++i) {
     if (select(i))
-      object(i)->saveAsXml(stream, String());
+      object(i)->saveAsXml(stream, layer(layerOf(i)));
   }
   stream << "</ipeselection>\n";
 }
