@@ -5,7 +5,7 @@
 /*
 
     This file is part of the extensible drawing editor Ipe.
-    Copyright (c) 1993-2019 Otfried Cheong
+    Copyright (c) 1993-2023 Otfried Cheong
 
     Ipe is free software; you can redistribute it and/or modify it
     under the terms of the GNU General Public License as published by
@@ -96,6 +96,7 @@ Painter::Painter(const Cascade *style)
   state.iGradient = Attribute::NORMAL();
   iState.push_back(state);
   iMatrix.push_back(Matrix()); // identity
+  iAttributeMap = nullptr;
   iInPath = 0;
 }
 
@@ -105,6 +106,22 @@ Painter::~Painter()
   // nothing
 }
 
+//! Set a new attribute map.
+void Painter::setAttributeMap(const AttributeMap *map)
+{
+  iAttributeMap = map;
+}
+
+//! Lookup a symbolic attribute
+/*! Uses first the attribute map and then the stylesheet. */
+Attribute Painter::lookup(Kind kind, Attribute sym) const
+{
+  if (iAttributeMap && sym.isSymbolic())
+    return iCascade->find(kind, iAttributeMap->map(kind, sym));
+  else
+    return iCascade->find(kind,sym);
+}
+
 //! Concatenate a matrix to current transformation matrix.
 void Painter::transform(const Matrix &m)
 {
@@ -287,7 +304,7 @@ void Painter::setStroke(Attribute color)
   else if (color == Attribute::SYM_FILL())
     iState.back().iStroke = iState.back().iSymFill;
   else
-    iState.back().iStroke = iCascade->find(EColor, color).color();
+    iState.back().iStroke = lookup(EColor, color).color();
 }
 
 //! Set fill color, resolving symbolic color.
@@ -299,7 +316,7 @@ void Painter::setFill(Attribute color)
   else if (color == Attribute::SYM_FILL())
     iState.back().iFill = iState.back().iSymFill;
   else
-    iState.back().iFill = iCascade->find(EColor, color).color();
+    iState.back().iFill = lookup(EColor, color).color();
 }
 
 //! Set pen, resolving symbolic value.
@@ -309,14 +326,14 @@ void Painter::setPen(Attribute pen)
   if (pen == Attribute::SYM_PEN())
     iState.back().iPen = iState.back().iSymPen;
   else
-    iState.back().iPen = iCascade->find(EPen, pen).number();
+    iState.back().iPen = lookup(EPen, pen).number();
 }
 
 //! Set dash style, resolving symbolic value.
 void Painter::setDashStyle(Attribute dash)
 {
   assert(!iInPath);
-  iState.back().iDashStyle = iCascade->find(EDashStyle, dash).string();
+  iState.back().iDashStyle = lookup(EDashStyle, dash).string();
 }
 
 //! Return dashstyle as a double sequence.
@@ -368,14 +385,14 @@ void Painter::setFillRule(TFillRule rule)
 void Painter::setOpacity(Attribute opaq)
 {
   assert(!iInPath);
-  iState.back().iOpacity = iCascade->find(EOpacity, opaq).number();
+  iState.back().iOpacity = lookup(EOpacity, opaq).number();
 }
 
 //! Set stroke opacity.
 void Painter::setStrokeOpacity(Attribute opaq)
 {
   assert(!iInPath);
-  iState.back().iStrokeOpacity = iCascade->find(EOpacity, opaq).number();
+  iState.back().iStrokeOpacity = lookup(EOpacity, opaq).number();
 }
 
 //! Set tiling pattern.
@@ -407,7 +424,7 @@ void Painter::setSymStroke(Attribute color)
   else if (color == Attribute::SYM_FILL())
     iState.back().iSymStroke = (++iState.rbegin())->iSymFill;
   else
-    iState.back().iSymStroke = iCascade->find(EColor, color).color();
+    iState.back().iSymStroke = lookup(EColor, color).color();
 }
 
 //! Set symbol fill color, resolving symbolic color.
@@ -419,7 +436,7 @@ void Painter::setSymFill(Attribute color)
   else if (color == Attribute::SYM_FILL())
     iState.back().iSymFill = (++iState.rbegin())->iSymFill;
   else
-    iState.back().iSymFill = iCascade->find(EColor, color).color();
+    iState.back().iSymFill = lookup(EColor, color).color();
 }
 
 //! Set symbol pen, resolving symbolic pen.
@@ -429,7 +446,7 @@ void Painter::setSymPen(Attribute pen)
   if (pen == Attribute::SYM_PEN())
     iState.back().iSymPen = (++iState.rbegin())->iSymPen;
   else
-    iState.back().iSymPen = iCascade->find(EPen, pen).number();
+    iState.back().iSymPen = lookup(EPen, pen).number();
 }
 
 //! Set full graphics state at once.
@@ -598,7 +615,9 @@ void Painter::doDrawText(const Text *)
     object. Only PDF drawing overrides this to reuse a PDF XForm. */
 void Painter::doDrawSymbol(Attribute symbol)
 {
-  const Symbol *sym = cascade()->findSymbol(symbol);
+  const Symbol *sym = iAttributeMap ?
+    cascade()->findSymbol(iAttributeMap->map(ESymbol, symbol)) :
+    cascade()->findSymbol(symbol);
   if (sym)
     sym->iObject->draw(*this);
 }
