@@ -4,7 +4,7 @@
 /*
 
     This file is part of the extensible drawing editor Ipe.
-    Copyright (c) 1993-2019 Otfried Cheong
+    Copyright (c) 1993-2023 Otfried Cheong
 
     Ipe is free software; you can redistribute it and/or modify it
     under the terms of the GNU General Public License as published by
@@ -134,6 +134,8 @@ Path::Path(const XmlAttributes &attr)
   iHasRArrow = false;
   iFArrowShape = iRArrowShape = Attribute::ARROW_NORMAL();
   iFArrowSize = iRArrowSize = Attribute::NORMAL();
+  iFArrowIsM = false;
+  iRArrowIsM = false;
 
   if (attr.has("arrow", str)) {
     iHasFArrow = true;
@@ -141,6 +143,7 @@ Path::Path(const XmlAttributes &attr)
     if (i >= 0) {
       iFArrowShape = Attribute(true, String("arrow/") + str.left(i) + "(spx)");
       iFArrowSize = Attribute::makeScalar(str.substr(i+1), Attribute::NORMAL());
+      iFArrowIsM = iFArrowShape.isMidArrow();
     } else
       iFArrowSize = Attribute::makeScalar(str, Attribute::NORMAL());
   }
@@ -151,6 +154,7 @@ Path::Path(const XmlAttributes &attr)
     if (i >= 0) {
       iRArrowShape = Attribute(true, String("arrow/") + str.left(i) + "(spx)");
       iRArrowSize = Attribute::makeScalar(str.substr(i+1), Attribute::NORMAL());
+      iRArrowIsM = iRArrowShape.isMidArrow();
     } else
       iRArrowSize = Attribute::makeScalar(str, Attribute::NORMAL());
   }
@@ -177,6 +181,7 @@ void Path::init(const AllAttributes &attr, bool withArrows)
   iHasRArrow = false;
   iFArrowShape = iRArrowShape = Attribute::ARROW_NORMAL();
   iFArrowSize = iRArrowSize = Attribute::NORMAL();
+  iFArrowIsM = iRArrowIsM = false;
   if (withArrows) {
     iFArrowShape = attr.iFArrowShape;
     iRArrowShape = attr.iRArrowShape;
@@ -184,6 +189,8 @@ void Path::init(const AllAttributes &attr, bool withArrows)
     iRArrowSize = attr.iRArrowSize;
     iHasFArrow = attr.iFArrow;
     iHasRArrow = attr.iRArrow;
+    iFArrowIsM = iFArrowShape.isMidArrow();
+    iRArrowIsM = iRArrowShape.isMidArrow();
   }
 }
 
@@ -204,13 +211,14 @@ Object *Path::clone() const
 //! Compute the arrow information.
 void Path::makeArrowData()
 {
-  assert(iShape.countSubPaths() > 0);
-  if (iShape.countSubPaths() > 1 || iShape.subPath(0)->closed()) {
+  if (iShape.countSubPaths() == 0 || iShape.countSubPaths() > 1 || iShape.subPath(0)->closed()) {
     iFArrowOk = false;
     iRArrowOk = false;
+    iMArrowOk = false;
   } else {
-    CurveSegment seg = iShape.subPath(0)->asCurve()->segment(0);
+    const Curve *curve = iShape.subPath(0)->asCurve();
 
+    CurveSegment seg = curve->segment(0);
     iRArrowOk = true;
     iRArrowPos = seg.cp(0);
     iRArrowArc = 0;
@@ -226,7 +234,7 @@ void Path::makeArrowData()
 	iRArrowDir = (iRArrowPos - seg.cp(1)).angle();
     }
 
-    seg = iShape.subPath(0)->asCurve()->segment(-1);
+    seg = curve->segment(-1);
     iFArrowOk = true;
     iFArrowPos = seg.last();
     iFArrowArc = 0;
@@ -236,10 +244,36 @@ void Path::makeArrowData()
       Linear m = seg.matrix().linear();
       iFArrowDir = (m * Vector(Angle(alpha + IpeHalfPi))).angle();
     } else {
-      if (seg.cp(seg.countCP() - 2) == seg.last())
+      Vector p = seg.cp(seg.countCP() - 2);
+      if (p == seg.last())
 	iFArrowOk = false;
       else
-	iFArrowDir = (iFArrowPos - seg.cp(seg.countCP() - 2)).angle();
+	iFArrowDir = (iFArrowPos - p).angle();
+    }
+
+    iMArrowOk = false;
+    Matrix m = matrix();
+    double len = 0.0;
+    for (int i = 0; i < curve->countSegments(); ++i) {
+      seg = curve->segment(i);
+      if (seg.type() != CurveSegment::ESegment)
+	return;
+      len += (m * seg.cp(0) - m * seg.cp(1)).len();
+    }
+    double mid = len / 2.0;
+    len = 0.0;
+    for (int i = 0; i < curve->countSegments(); ++i) {
+      seg = curve->segment(i);
+      double d = (m * seg.cp(0) - m * seg.cp(1)).len();
+      if (len < mid && mid <= len + d) {
+	// found the position
+	iMArrowOk = true;
+	double lambda = (mid - len) / d;
+	iMArrowPos = seg.cp(0) + lambda * (seg.cp(1) - seg.cp(0));
+	iMArrowDir = (seg.cp(1) - seg.cp(0)).angle();
+	return;
+      }
+      len += d;
     }
   }
 }
@@ -443,7 +477,10 @@ void Path::draw(Painter &painter) const
 	if ((painter.matrix().linear() * seg.matrix().linear()).determinant()<0)
 	  r = -r;
       }
-      drawArrow(painter, iFArrowPos, iFArrowDir, iFArrowShape, iFArrowSize, r);
+      if (iFArrowIsM && iMArrowOk)
+	drawArrow(painter, iMArrowPos, iMArrowDir, iFArrowShape, iFArrowSize, r);
+      else
+	drawArrow(painter, iFArrowPos, iFArrowDir, iFArrowShape, iFArrowSize, r);
     }
     if (iHasRArrow && iRArrowOk) {
       double r = 0.0;
@@ -454,7 +491,10 @@ void Path::draw(Painter &painter) const
 	if ((painter.matrix().linear() * seg.matrix().linear()).determinant()>0)
 	  r = -r;
       }
-      drawArrow(painter, iRArrowPos, iRArrowDir, iRArrowShape, iRArrowSize, r);
+      if (iRArrowIsM && iMArrowOk)
+	drawArrow(painter, iMArrowPos, iMArrowDir + IPE_PI, iRArrowShape, iRArrowSize, r);
+      else
+	drawArrow(painter, iRArrowPos, iRArrowDir, iRArrowShape, iRArrowSize, r);
     }
   }
   painter.popMatrix();
@@ -500,6 +540,12 @@ void Path::snapBnd(const Vector &mouse, const Matrix &m,
   iShape.snapBnd(mouse, m * matrix(), pos, bound);
 }
 
+void Path::setMatrix(const Matrix &matrix)
+{
+  Object::setMatrix(matrix);
+  makeArrowData();
+}
+
 //! Set whether object will be stroked and filled.
 void Path::setPathMode(TPathMode pm)
 {
@@ -564,6 +610,7 @@ void Path::setArrow(bool arrow, Attribute shape, Attribute size)
   iHasFArrow = arrow;
   iFArrowShape = shape;
   iFArrowSize = size;
+  iFArrowIsM = iFArrowShape.isMidArrow();
 }
 
 //! Set backward arrow (if the object can take it).
@@ -572,6 +619,7 @@ void Path::setRarrow(bool arrow, Attribute shape, Attribute size)
   iHasRArrow = arrow;
   iRArrowShape = shape;
   iRArrowSize = size;
+  iRArrowIsM = iRArrowShape.isMidArrow();
 }
 
 //! Set line cap style.
@@ -694,12 +742,14 @@ bool Path::setAttribute(Property prop, Attribute value)
   case EPropFArrowShape:
     if (value != iFArrowShape) {
       iFArrowShape = value;
+      iFArrowIsM = iFArrowShape.isMidArrow();
       return true;
     }
     break;
   case EPropRArrowShape:
     if (value != iRArrowShape) {
       iRArrowShape = value;
+      iRArrowIsM = iRArrowShape.isMidArrow();
       return true;
     }
     break;
