@@ -4,7 +4,7 @@
 /*
 
     This file is part of the extensible drawing editor Ipe.
-    Copyright (c) 1993-2020 Otfried Cheong
+    Copyright (c) 1993-2023 Otfried Cheong
 
     Ipe is free software; you can redistribute it and/or modify it
     under the terms of the GNU General Public License as published by
@@ -437,7 +437,13 @@ bool PdfDict::getNumberArray(String key, const PdfFile *file,
 bool PdfDict::deflated() const noexcept
 {
   const PdfObj *f = get("Filter", nullptr);
-  return !(!f || !f->name() || f->name()->value() != "FlateDecode");
+  if (!f) return false;
+  if (f->array()) {
+    const PdfArray *a = f->array();
+    if (a->count() != 1) return false;
+    f = a->obj(0, nullptr);
+  }
+  return f->name() && f->name()->value() == "FlateDecode";
 }
 
 //! Return the (uncompressed) stream data.
