@@ -4,7 +4,7 @@
 /*
 
     This file is part of the extensible drawing editor Ipe.
-    Copyright (c) 1993-2020 Otfried Cheong
+    Copyright (c) 1993-2023 Otfried Cheong
 
     Ipe is free software; you can redistribute it and/or modify it
     under the terms of the GNU General Public License as published by
@@ -289,7 +289,7 @@ PdfWriter::PdfWriter(TellStream &stream, const Document *doc, const PdfResources
     --id;
   }
 
-  iStream << "%PDF-1.4\n";
+  iStream << "%PDF-1.5\n";
 
   // embed all fonts and other resources from Pdflatex
   embedResources();
@@ -785,6 +785,8 @@ void PdfWriter::createPageView(int pno, int view)
   bm.scanPage(page);
   // ipeDebug("# of bitmaps: %d", bm.iBitmaps.size());
   embedBitmaps(bm);
+  if (page->findLayer("NOPDF") >= 0)
+    return;
   // create page stream
   String pagedata;
   StringStream sstream(pagedata);
@@ -806,6 +808,7 @@ void PdfWriter::createPageView(int pno, int view)
 	      << "/Type /Annot\n"
 	      << "/Subtype /Link\n"
 	      << "/H /N\n"
+	      << "/Border [0 0 0]\n"
 	      << "/Rect [" << page->bbox(i) << "]\n"
 	      << "/A <</Type/Action/S";
       String url = g->url();
@@ -951,10 +954,11 @@ void PdfWriter::writeString(String text)
 
 // --------------------------------------------------------------------
 
-int PdfWriter::pageObjectNumber(int page, int view)
+// returns object number of the first view of the requested page
+int PdfWriter::pageObjectNumber(int page)
 {
   auto it = std::find_if(iPageObjectNumbers.begin(), iPageObjectNumbers.end(),
-			 [=](const PON &pon) {return pon.page == page && pon.view == view;});
+			 [=](const PON &pon) {return pon.page == page;});
   if (it != iPageObjectNumbers.end())
     return it->objNum;
   ipeDebug("pageObjectNumber not found, this is a bug!");
@@ -975,11 +979,14 @@ void PdfWriter::createBookmarks()
   for (int pg = iFromPage; pg <= iToPage; ++pg) {
     if ((iSaveFlags & SaveFlag::MarkedView) && !iDoc->page(pg)->marked())
       continue;
+    if (iDoc->page(pg)->findLayer("NOPDF") >= 0)
+      continue;
     String s = iDoc->page(pg)->section(0);
     String ss = iDoc->page(pg)->section(1);
     if (!s.empty()) {
       Section sec;
       sec.iPage = pg;
+      sec.iObjNum = -1; // TODO: sensible value
       sections.push_back(sec);
     }
     if (!sections.empty() && !ss.empty())
@@ -1007,7 +1014,7 @@ void PdfWriter::createBookmarks()
     iStream << "<<\n/Title ";
     writeString(iDoc->page(sections[s].iPage)->section(0));
     iStream << "\n/Parent " << iBookmarks << " 0 R\n"
-	    << "/Dest [ " << pageObjectNumber(sections[s].iPage, 0)
+	    << "/Dest [ " << pageObjectNumber(sections[s].iPage)
 	    << " 0 R /XYZ null null null ]\n";
     if (s > 0)
       iStream << "/Prev " << sections[s-1].iObjNum << " 0 R\n";
@@ -1025,7 +1032,7 @@ void PdfWriter::createBookmarks()
       iStream << "<<\n/Title ";
       writeString(iDoc->page(pageNo)->section(1));
       iStream << "\n/Parent " << obj << " 0 R\n"
-	      << "/Dest [ " << pageObjectNumber(pageNo, 0)
+	      << "/Dest [ " << pageObjectNumber(pageNo)
 	      << " 0 R /XYZ null null null ]\n";
       if (ss > 0)
 	iStream << "/Prev " << (obj + ss) << " 0 R\n";
@@ -1043,9 +1050,11 @@ void PdfWriter::createNamedDests()
   for (int pg = iFromPage; pg <= iToPage; ++pg) {
     if ((iSaveFlags & SaveFlag::MarkedView) && !iDoc->page(pg)->marked())
       continue;
+    if (iDoc->page(pg)->findLayer("NOPDF") >= 0)
+      continue;
     String s = iDoc->page(pg)->section(0);
     if (!s.empty())
-      dests.push_back(std::make_pair(s, pageObjectNumber(pg, 0)));
+      dests.push_back(std::make_pair(s, pageObjectNumber(pg)));
   }
   if (dests.empty())
     return;
