@@ -4,7 +4,7 @@
 /*
 
     This file is part of the extensible drawing editor Ipe.
-    Copyright (c) 1993-2019 Otfried Cheong
+    Copyright (c) 1993-2023 Otfried Cheong
 
     Ipe is free software; you can redistribute it and/or modify it
     under the terms of the GNU General Public License as published by
@@ -163,7 +163,7 @@ void Reference::draw(Painter &painter) const
   const Symbol *symbol = painter.cascade()->findSymbol(iName);
   if (symbol) {
     iSnap = symbol->iSnap;  // cache snap point information
-    Attribute si = painter.cascade()->find(ESymbolSize, iSize);
+    Attribute si = painter.lookup(ESymbolSize, iSize);
     double s = si.number().toDouble();
     painter.pushMatrix();
     painter.transform(matrix());
