@@ -4,7 +4,7 @@
 /*
 
     This file is part of the extensible drawing editor Ipe.
-    Copyright (c) 1993-2020 Otfried Cheong
+    Copyright (c) 1993-2023 Otfried Cheong
 
     Ipe is free software; you can redistribute it and/or modify it
     under the terms of the GNU General Public License as published by
@@ -531,9 +531,9 @@ void Curve::snapVtx(const Vector &mouse, const Matrix &m,
 {
   if (!ctl)
     snapVertex(mouse, m * segment(0).cp(0), pos, bound);
-  else if (iClosed)
+  else if (closed())
     // midpoint of closing segment
-    snapVertex(mouse, m * (0.5 * (iCP.back() + iCP.front())), pos, bound);
+    closingSegment().snapVtx(mouse, m, pos, bound, ctl);
   for (int i = 0; i < countSegments(); ++i)
     segment(i).snapVtx(mouse, m, pos, bound, ctl);
 }
@@ -1074,7 +1074,10 @@ bool Shape::load(String data)
       stream.nextToken();
       sp = nullptr;
       mid = -1;
-      Ellipse *e = new Ellipse(getMatrix(args));
+      Matrix m = getMatrix(args);
+      if (m.determinant() == 0)
+	return false; // don't accept zero-radius arc
+      Ellipse *e = new Ellipse(m);
       appendSubPath(e);
     } else if (stream.token() == "u") {
       if (args.size() < 6 || (args.size() % 2 != 0))
