@@ -4,7 +4,7 @@
 /*
 
     This file is part of the extensible drawing editor Ipe.
-    Copyright (c) 1993-2019 Otfried Cheong
+    Copyright (c) 1993-2023 Otfried Cheong
 
     Ipe is free software; you can redistribute it and/or modify it
     under the terms of the GNU General Public License as published by
@@ -73,17 +73,25 @@ private:
   std::vector<Matrix> iMatrices;
   Vector iMouse;
   double iDist;
-  int iView;
 };
 
 CollectSegs::CollectSegs(const Vector &mouse, double snapDist,
 			 const Page *page, int view)
-  : iMouse(mouse), iDist(snapDist), iView(view)
+  : iMouse(mouse), iDist(snapDist)
 {
   iMatrices.push_back(Matrix()); // identity matrix
-  for (int i = 0; i < page->count(); ++i) {
-    if (page->objSnapsInView(i, iView))
-      page->object(i)->accept(*this);
+  if (view < 0) {
+    int gridLayer = page->findLayer("GRID");
+    if (gridLayer < 0) return;  // nothing
+    for (int i = 0; i < page->count(); ++i) {
+      if (page->layerOf(i) == gridLayer)
+	page->object(i)->accept(*this);
+    }
+  } else {
+    for (int i = 0; i < page->count(); ++i) {
+      if (page->objSnapsInView(i, view))
+	page->object(i)->accept(*this);
+    }
   }
 }
 
@@ -124,10 +132,8 @@ void CollectSegs::visitPath(const Path *obj)
       break; }
     case SubPath::ECurve: {
       const Curve *ssp = sp->asCurve();
-      int ns = ssp->closed() ? -1 : 0;
-      Vector u[2];
-      for (int j = ns; j < ssp->countSegments(); ++j) {
-	CurveSegment seg = j < 0 ? ssp->closingSegment(u) : ssp->segment(j);
+      for (int j = 0; j < ssp->countSegmentsClosing(); ++j) {
+	CurveSegment seg = ssp->segment(j);
 	switch (seg.type()) {
 	case CurveSegment::ESegment:
 	  if (seg.distance(iMouse, m, iDist) < iDist)
@@ -139,7 +145,9 @@ void CollectSegs::visitPath(const Path *obj)
 	    iArcs.push_back(arc);
 	  break;
 	case CurveSegment::EOldSpline:
-	case CurveSegment::ESpline: {
+	case CurveSegment::ESpline:
+	case CurveSegment::ESpiroSpline:
+	case CurveSegment::ECardinalSpline: {
 	  std::vector<Bezier> bez;
 	  seg.beziers(bez);
 	  bool cont = false;
@@ -331,6 +339,15 @@ Snap::TSnapModes Snap::simpleSnap(Vector &pos, const Page *page, int view,
     }
   }
 
+  // custom grid snapping
+  if (iSnap & ESnapCustom) {
+    intersectionSnap(pos, fifi, page, -1, d);
+    if (d < snapDist) {
+      pos = fifi;
+      return ESnapCustom;
+    }
+  }
+
   // grid snapping: always occurs
   if (iSnap & ESnapGrid) {
     int grid = iGridSize;
@@ -339,6 +356,7 @@ Snap::TSnapModes Snap::simpleSnap(Vector &pos, const Page *page, int view,
     pos = fifi;
     return ESnapGrid;
   }
+
   return ESnapNone;
 }
 
