@@ -4,7 +4,7 @@
 /*
 
     This file is part of the extensible drawing editor Ipe.
-    Copyright (c) 1993-2020 Otfried Cheong
+    Copyright (c) 1993-2023 Otfried Cheong
 
     Ipe is free software; you can redistribute it and/or modify it
     under the terms of the GNU General Public License as published by
@@ -426,6 +426,7 @@ void StyleSheet::saveAsXml(Stream &stream, bool saveBitmaps) const
 	 it != iSymbols.end(); ++it)
       it->second.iObject->accept(bm);
     if (!bm.iBitmaps.empty()) {
+      std::sort(bm.iBitmaps.begin(), bm.iBitmaps.end());
       int id = 1;
       Bitmap prev;
       for (std::vector<Bitmap>::iterator it = bm.iBitmaps.begin();
@@ -466,21 +467,29 @@ void StyleSheet::saveAsXml(Stream &stream, bool saveBitmaps) const
     stream << "</symbol>\n";
   }
 
+  std::vector<String> mappings;
   for (Map::const_iterator it = iMap.begin(); it != iMap.end(); ++it) {
+    String mapping;
+    StringStream out(mapping);
     int kind = (it->first >> SHIFT);
     int kind1 = (kind == ELabelStyle) ? ETextStyle : kind;
-    stream << "<" << kind_names[kind1]
-	   << " name=\"" << rep->toString(it->first & MASK) << "\"";
+    out << "<" << kind_names[kind1]
+	<< " name=\"" << rep->toString(it->first & MASK) << "\"";
     if (kind1 == ETextStyle) {
       String s = it->second.string();
       int i = s.find('\0');
       if (kind == ELabelStyle)
-	stream << " type=\"label\"";
-      stream << " begin=\"" << s.substr(0, i) << "\" end=\""
-	     << s.substr(i+1) << "\"/>\n";
+	out << " type=\"label\"";
+      out << " begin=\"" << s.substr(0, i) << "\" end=\""
+	  << s.substr(i+1) << "\"/>\n";
     } else
-      stream << " value=\"" << it->second.string() << "\"/>\n";
+      out << " value=\"" << it->second.string() << "\"/>\n";
+    mappings.push_back(mapping);
   }
+  // sort symbolic mapping table to make sure it is deterministic
+  std::sort(mappings.begin(), mappings.end());
+  for (auto & mapping : mappings)
+    stream << mapping;
   if (!iPreamble.empty()) {
     stream << "<preamble>";
     stream.putXmlString(iPreamble);
