@@ -4,7 +4,7 @@
 /*
 
     This file is part of the extensible drawing editor Ipe.
-    Copyright (c) 1993-2019 Otfried Cheong
+    Copyright (c) 1993-2023 Otfried Cheong
 
     Ipe is free software; you can redistribute it and/or modify it
     under the terms of the GNU General Public License as published by
@@ -597,7 +597,7 @@ void Text::setXForm(XForm *xform) const
     delete iXForm;
   iXForm = xform;
   if (iXForm) {
-    iXForm->iRefCount = 1;
+    ++iXForm->iRefCount;
     iDepth = iXForm->iStretch * iXForm->iDepth / 100.0;
     iHeight = iXForm->iStretch * iXForm->iBBox.height() - iDepth;
     if (!isMinipage())
@@ -625,17 +625,15 @@ Vector Text::align() const
     align.y = depth();
     break;
   }
-  if (!isMinipage()) {
-    switch (horizontalAlignment()) {
-    case EAlignLeft:
-      break;
-    case EAlignRight:
-      align.x = width();
-      break;
-    case EAlignHCenter:
-      align.x = 0.5 * width();
-      break;
-    }
+  switch (horizontalAlignment()) {
+  case EAlignLeft:
+    break;
+  case EAlignRight:
+    align.x = width();
+    break;
+  case EAlignHCenter:
+    align.x = 0.5 * width();
+    break;
   }
   return align;
 }
