@@ -4,7 +4,7 @@
 /*
 
     This file is part of the extensible drawing editor Ipe.
-    Copyright (c) 1993-2020 Otfried Cheong
+    Copyright (c) 1993-2023 Otfried Cheong
 
     Ipe is free software; you can redistribute it and/or modify it
     under the terms of the GNU General Public License as published by
@@ -41,7 +41,7 @@ using namespace ipelua;
 // --------------------------------------------------------------------
 
 static const char * const format_name[] =
-  { "xml", "pdf", "eps", "ipe5", "unknown" };
+  { "xml", "pdf", "unknown" };
 
 void ipelua::make_metatable(lua_State *L, const char *name,
 			    const struct luaL_Reg *methods)
@@ -298,11 +298,17 @@ static int document_runLatex(lua_State *L)
   String docname;
   if (!lua_isnoneornil(L, 2))
     docname = luaL_checklstring(L, 2, nullptr);
+  bool async = lua_toboolean(L, 3);
   String log;
-  int result = (*d)->runLatex(docname, log);
+  ipe::Latex *converter;
+  int result = async ? (*d)->runLatexAsync(docname, log, &converter)
+    : (*d)->runLatex(docname, log);
 
   if (result == Document::ErrNone) {
-    lua_pushboolean(L, true);
+    if (async)
+      lua_pushlightuserdata(L, converter);
+    else
+      lua_pushboolean(L, true);
     lua_pushnil(L);
     lua_pushnil(L);
   } else if (result == Document::ErrNoText) {
@@ -338,6 +344,16 @@ static int document_runLatex(lua_State *L)
   return 4;
 }
 
+static int document_completeLatexRun(lua_State *L)
+{
+  Document **d = check_document(L, 1);
+  ipe::Latex *converter = (ipe::Latex *) lua_touserdata(L, 2);
+  if (converter == nullptr)
+    luaL_error(L, "no Latex converter given");
+  lua_pushboolean(L, (*d)->completeLatexRun(converter));
+  return 1;
+}
+
 static int document_checkStyle(lua_State *L)
 {
   Document **d = check_document(L, 1);
@@ -403,6 +419,8 @@ static int document_properties(lua_State *L)
   lua_setfield(L, -2, "subject");
   push_string(L, prop.iKeywords);
   lua_setfield(L, -2, "keywords");
+  push_string(L, prop.iLanguage);
+  lua_setfield(L, -2, "language");
   push_string(L, prop.iPreamble);
   lua_setfield(L, -2, "preamble");
   push_string(L, prop.iCreated);
@@ -415,6 +433,8 @@ static int document_properties(lua_State *L)
   lua_setfield(L, -2, "fullscreen");
   lua_pushboolean(L, prop.iNumberPages);
   lua_setfield(L, -2, "numberpages");
+  lua_pushboolean(L, prop.iSequentialText);
+  lua_setfield(L, -2, "sequentialtext");
   lua_pushstring(L, tex_engine_names[int(prop.iTexEngine)]);
   lua_setfield(L, -2, "tex");
   return 1;
@@ -443,11 +463,13 @@ static int document_setProperties(lua_State *L)
   Document::SProperties prop = (*d)->properties();
   // take from table
   propFlag(L, "numberpages", prop.iNumberPages);
+  propFlag(L, "sequentialtext", prop.iSequentialText);
   propFlag(L, "fullscreen", prop.iFullScreen);
   propString(L, "title", prop.iTitle);
   propString(L, "author", prop.iAuthor);
   propString(L, "subject", prop.iSubject);
   propString(L, "keywords", prop.iKeywords);
+  propString(L, "language", prop.iLanguage);
   propString(L, "preamble", prop.iPreamble);
   propString(L, "created", prop.iCreated);
   propString(L, "modified", prop.iModified);
@@ -481,6 +503,7 @@ static const struct luaL_Reg document_methods[] = {
   { "sheets", document_sheets },
   { "replaceSheets", document_replaceSheets },
   { "runLatex", document_runLatex },
+  { "completeLatexRun", document_completeLatexRun },
   { "checkStyle", document_checkStyle },
   { "properties", document_properties },
   { "setProperties", document_setProperties },
@@ -526,7 +549,7 @@ static int ipe_splinetobeziers(lua_State *L)
   bool closed = lua_toboolean(L, 2);
   std::vector<Bezier> result;
   if (closed) {
-    Bezier::closedSpline(v.size(), &v[0], result);
+    Bezier::closedSpline(v.size(), v.data(), result);
   } else {
     // get spline type
     lua_getfield(L, 1, "type");
@@ -538,21 +561,21 @@ static int ipe_splinetobeziers(lua_State *L)
     lua_pop(L, 1); // pop type
     switch (type) {
     case CurveSegment::ESpline:
-      Bezier::spline(v.size(), &v[0], result);
+      Bezier::spline(v.size(), v.data(), result);
       break;
     case CurveSegment::EOldSpline:
-      Bezier::oldSpline(v.size(), &v[0], result);
+      Bezier::oldSpline(v.size(), v.data(), result);
       break;
     case CurveSegment::ECardinalSpline: {
       lua_getfield(L, 1, "tension");
       if (!lua_isnumber(L, -1))
 	luaL_error(L, "spline has no tension");
       float tension = lua_tonumberx(L, -1, nullptr);
-      Bezier::cardinalSpline(v.size(), &v[0], tension, result);
+      Bezier::cardinalSpline(v.size(), v.data(), tension, result);
       lua_pop(L, 1); // tension
       break; }
     case CurveSegment::ESpiroSpline:
-      Bezier::spiroSpline(v.size(), &v[0], result);
+      Bezier::spiroSpline(v.size(), v.data(), result);
       break;
     default:
       break;
