@@ -4,7 +4,7 @@
 /*
 
     This file is part of the extensible drawing editor Ipe.
-    Copyright (c) 1993-2020 Otfried Cheong
+    Copyright (c) 1993-2023 Otfried Cheong
 
     Ipe is free software; you can redistribute it and/or modify it
     under the terms of the GNU General Public License as published by
@@ -145,6 +145,8 @@ namespace ipelua {
   extern const char *const linecap_names[];
   extern const char *const fillrule_names[];
   extern const char *const segtype_names[];
+  extern const char *const horizontal_alignment_names[];
+  extern const char *const vertical_alignment_names[];
 
   extern ipe::String check_filename(lua_State *L, int index);
 
