@@ -4,7 +4,7 @@
 /*
 
     This file is part of the extensible drawing editor Ipe.
-    Copyright (c) 1993-2019 Otfried Cheong
+    Copyright (c) 1993-2023 Otfried Cheong
 
     Ipe is free software; you can redistribute it and/or modify it
     under the terms of the GNU General Public License as published by
@@ -288,8 +288,9 @@ static int matrix_isIdentity(lua_State *L)
 static int matrix_isSingular(lua_State *L)
 {
   Matrix *m = check_matrix(L, 1);
+  double x = luaL_checknumber(L, 2);
   double t = m->a[0]*m->a[3]-m->a[1]*m->a[2];
-  lua_pushboolean(L, t == 0);
+  lua_pushboolean(L, t < 0 ? -t <= x : t <= x);
   return 1;
 }
 
