@@ -4,7 +4,7 @@
 /*
 
     This file is part of the extensible drawing editor Ipe.
-    Copyright (c) 1993-2020 Otfried Cheong
+    Copyright (c) 1993-2023 Otfried Cheong
 
     Ipe is free software; you can redistribute it and/or modify it
     under the terms of the GNU General Public License as published by
@@ -54,10 +54,10 @@ static const char *const pathmode_names[] =
 static const char *const transformation_names[] =
   { "translations", "rigid", "affine", nullptr };
 
-static const char *const horizontal_alignment_names[] =
+const char *const ipelua::horizontal_alignment_names[] =
   { "left", "right", "hcenter", nullptr };
 
-static const char *const vertical_alignment_names[] =
+const char *const ipelua::vertical_alignment_names[] =
   { "bottom", "baseline", "top", "vcenter", nullptr };
 
 const char *const ipelua::linejoin_names[] =
@@ -495,6 +495,22 @@ static int object_get(lua_State *L)
   return 1;
 }
 
+static int object_get_custom(lua_State *L)
+{
+  SObject *s = check_object(L, 1);
+  Attribute value = s->obj->getCustom();
+  push_attribute(L, value);
+  return 1;
+}
+
+static int object_set_custom(lua_State *L)
+{
+  SObject *s = check_object(L, 1);
+  const char *value = luaL_checklstring(L, 2, nullptr);
+  s->obj->setCustom(Attribute(false, value));
+  return 0;
+}
+
 static int object_position(lua_State *L)
 {
   Object *obj = check_object(L, 1)->obj;
@@ -536,6 +552,17 @@ static int object_setText(lua_State *L)
   return 0;
 }
 
+static int object_text_dimensions(lua_State *L)
+{
+  Object *obj = check_object(L, 1)->obj;
+  luaL_argcheck(L, obj->type() == Object::EText, 1, "not a text object");
+  Text *t = obj->asText();
+  lua_pushnumber(L, t->width());
+  lua_pushnumber(L, t->height());
+  lua_pushnumber(L, t->depth());
+  return 3;
+}
+
 static int object_clone(lua_State *L)
 {
   SObject *s = check_object(L, 1);
@@ -932,6 +959,8 @@ static const struct luaL_Reg object_methods[] = {
   { "type", object_type },
   { "set", object_set },
   { "get", object_get },
+  { "setCustom", object_set_custom },
+  { "getCustom", object_get_custom },
   { "xml", object_xml },
   { "clone", object_clone },
   { "matrix", object_matrix },
@@ -949,6 +978,7 @@ static const struct luaL_Reg object_methods[] = {
   { "position", object_position },
   { "text", object_text },
   { "setText", object_setText },
+  { "dimensions", object_text_dimensions },
   { "elements", object_elements },
   { "element", object_element },
   { "elementType", object_elementType },
