@@ -4,7 +4,7 @@
 /*
 
     This file is part of the extensible drawing editor Ipe.
-    Copyright (c) 1993-2019 Otfried Cheong
+    Copyright (c) 1993-2023 Otfried Cheong
 
     Ipe is free software; you can redistribute it and/or modify it
     under the terms of the GNU General Public License as published by
@@ -230,14 +230,6 @@ static int page_isLocked(lua_State *L)
   return 1;
 }
 
-static int page_hasSnapping(lua_State *L)
-{
-  Page *p = check_page(L, 1)->page;
-  int n = check_layer(L, 2, p);
-  lua_pushboolean(L, p->hasSnapping(n));
-  return 1;
-}
-
 static int page_setLocked(lua_State *L)
 {
   Page *p = check_page(L, 1)->page;
@@ -246,14 +238,44 @@ static int page_setLocked(lua_State *L)
   return 0;
 }
 
+static const char * const snapmode_names[] =
+  { "never", "visible", "always", nullptr };
+
 static int page_setSnapping(lua_State *L)
 {
   Page *p = check_page(L, 1)->page;
   int n = check_layer(L, 2, p);
-  p->setSnapping(n, lua_toboolean(L, 3));
+  int mode = luaL_checkoption(L, 3, nullptr, snapmode_names);
+  p->setSnapping(n, Page::SnapMode(mode));
   return 0;
 }
 
+static int page_snapping(lua_State *L)
+{
+  Page *p = check_page(L, 1)->page;
+  int n = check_layer(L, 2, p);
+  Page::SnapMode mode = p->snapping(n);
+  lua_pushstring(L, snapmode_names[int(mode)]);
+  return 1;
+}
+
+static int page_setLayerData(lua_State *L)
+{
+  Page *p = check_page(L, 1)->page;
+  int n = check_layer(L, 2, p);
+  String s = luaL_checklstring(L, 3, nullptr);
+  p->setLayerData(n, s);
+  return 0;
+}
+
+static int page_layerData(lua_State *L)
+{
+  Page *p = check_page(L, 1)->page;
+  int n = check_layer(L, 2, p);
+  push_string(L, p->layerData(n));
+  return 1;
+}
+
 static int page_renameLayer(lua_State *L)
 {
   Page *p = check_page(L, 1)->page;
@@ -610,6 +632,112 @@ static int page_setViewName(lua_State *L)
   return 0;
 }
 
+static int page_viewMap(lua_State *L)
+{
+  Page *p = check_page(L, 1)->page;
+  int n = check_viewno(L, 2, p);
+  const AttributeMap &map = p->viewMap(n);
+  lua_createtable(L, map.count(), 0);
+  for (int i = 0; i < map.count(); ++i) {
+    lua_createtable(L, 0, 3);
+    lua_pushstring(L, kind_names[map.iMap[i].kind]);
+    lua_setfield(L, -2, "kind");
+    push_string(L, map.iMap[i].from.string());
+    lua_setfield(L, -2, "map");
+    push_string(L, map.iMap[i].to.string());
+    lua_setfield(L, -2, "to");
+    lua_rawseti(L, -2, i+1);
+  }
+  return 1;
+}
+
+static const char * const map_kind_names[] =
+  { "pen", "symbolsize", "arrowsize", "opacity", "color", "dashstyle", "symbol", nullptr };
+static const Kind map_kind_kinds[] =
+  { EPen, ESymbolSize, EArrowSize, EOpacity, EColor, EDashStyle, ESymbol };
+
+// index must be positive
+static AttributeMap check_attribute_map(lua_State *L, int index)
+{
+  luaL_checktype(L, index, LUA_TTABLE);
+  AttributeMap map;
+  for (int i = 0; ; ++i) {
+    lua_rawgeti(L, index, i+1);
+    if (lua_isnil(L, -1)) {
+      lua_pop(L, 1);
+      return map;
+    }
+    if (!lua_istable(L, -1))
+      luaL_error(L, "element %d is not a table", i+1);
+    lua_getfield(L, -1, "kind");
+    if (!lua_isstring(L, -1))
+      luaL_error(L, "element %d has no kind", i+1);
+    int k = test_option(L, -1, map_kind_names);
+    if (k < 0)
+      luaL_error(L, "element %d has invalid kind", i+1);
+    Kind kind = map_kind_kinds[k];
+    lua_pop(L, 1); // kind
+    lua_getfield(L, -1, "map");
+    lua_getfield(L, -2, "to");
+    if (!lua_isstring(L, -2) || !lua_isstring(L, -1))
+      luaL_error(L, "element %d has no 'map' or no 'to' component", i+1);
+    String from = lua_tolstring(L, -2, nullptr);
+    String to = lua_tolstring(L, -1, nullptr);
+    map.add({ kind, Attribute(true, from), Attribute(true, to) });
+    lua_pop(L, 3); // to, from, element i
+  }
+}
+
+static int page_setViewMap(lua_State *L)
+{
+  Page *p = check_page(L, 1)->page;
+  int n = check_viewno(L, 2, p);
+  AttributeMap map = check_attribute_map(L, 3);
+  p->setViewMap(n, map);
+  return 0;
+}
+
+static int page_layerMatrices(lua_State *L)
+{
+  Page *p = check_page(L, 1)->page;
+  int n = check_viewno(L, 2, p);
+  std::vector<Matrix> ms = p->layerMatrices(n);
+  int count = 0;
+  for (const auto & m : ms) {
+    if (!m.isIdentity()) ++count;
+  }
+  lua_createtable(L, 0, count);
+  for (int i = 0; i < size(ms); ++i) {
+    if (!ms[i].isIdentity()) {
+      push_string(L, p->layer(i));
+      push_matrix(L, ms[i]);
+      lua_settable(L, -3);
+    }
+  }
+  return 1;
+}
+
+static int page_setLayerMatrices(lua_State *L)
+{
+  Page *p = check_page(L, 1)->page;
+  int n = check_viewno(L, 2, p);
+  luaL_checktype(L, 3, LUA_TTABLE);
+  lua_pushnil(L);  // first key
+  p->clearLayerMatrices(n);
+  while (lua_next(L, 3) != 0) {
+    // uses 'key' (at index -2) and 'value' (at index -1)
+    String layer = luaL_checklstring(L, -2, nullptr);
+    Matrix *m = check_matrix(L, -1);
+    int layerNum = p->findLayer(layer);
+    if (layerNum < 0)
+      luaL_error(L, "page has no such layer");
+    p->setLayerMatrix(n, layerNum, *m);
+    // removes 'value'; keeps 'key' for next iteration
+    lua_pop(L, 1);
+  }
+  return 0;
+}
+
 // Either: view & layername
 // Or: view & object index
 static int page_visible(lua_State *L)
@@ -665,9 +793,11 @@ static const struct luaL_Reg page_methods[] = {
   { "xml", page_xml },
   { "layers", page_layers },
   { "isLocked", page_isLocked },
-  { "hasSnapping", page_hasSnapping },
+  { "snapping", page_snapping },
   { "setLocked", page_setLocked },
   { "setSnapping", page_setSnapping },
+  { "layerData", page_layerData },
+  { "setLayerData", page_setLayerData },
   { "renameLayer", page_renameLayer },
   { "addLayer", page_addLayer },
   { "removeLayer", page_removeLayer },
@@ -687,6 +817,10 @@ static const struct luaL_Reg page_methods[] = {
   { "setMarkedView", page_setMarkedView },
   { "viewName", page_viewName },
   { "setViewName", page_setViewName },
+  { "viewMap", page_viewMap },
+  { "setViewMap", page_setViewMap },
+  { "layerMatrices", page_layerMatrices },
+  { "setLayerMatrices", page_setLayerMatrices },
   { "visible", page_visible },
   { "setVisible", page_setVisible },
   { "bbox", page_bbox },
