@@ -5,7 +5,7 @@
 /*
 
     This file is part of the extensible drawing editor Ipe.
-    Copyright (c) 1993-2020 Otfried Cheong
+    Copyright (c) 1993-2023 Otfried Cheong
 
     Ipe is free software; you can redistribute it and/or modify it
     under the terms of the GNU General Public License as published by
@@ -308,14 +308,12 @@ static void setItemShortcut(NSMenu *menu, int index, unichar code)
 
 static const char * const about_text =
   "IpePresenter %d.%d.%d\n\n"
-  "Copyright (c) 2020 Otfried Cheong\n\n"
+  "Copyright (c) 2020-2023 Otfried Cheong\n\n"
   "A presentation tool for giving PDF presentations "
   "created in Ipe or using beamer.\n"
   "Originally invented by Dmitriy Morozov, "
   "IpePresenter is now developed together with Ipe and released under the GNU Public License.\n"
   "See http://ipepresenter.otfried.org for details.\n\n"
-  "You can \"like\" IpePresenter and follow IpePresenter announcements on Facebook "
-  "(http://www.facebook.com/drawing.editor.Ipe7).\n\n"
   "If you are an IpePresenter fan and want to show others, have a look at the "
   "Ipe T-shirts (www.shirtee.com/en/store/ipe).\n\n"
   "Platinum and gold sponsors\n\n"
@@ -323,7 +321,8 @@ static const char * const about_text =
   " * Günter Rote\n"
   " * SCALGO\n"
   " * Martin Ziegler\n\n"
-  "If you enjoy IpePresenter, you can become a member of the exclusive community of "
+  "If you enjoy IpePresenter, feel free to treat the author on a cup of coffee at https://ko-fi.com/ipe7author.\n\n"
+  "You can also become a member of the exclusive community of "
   "Ipe patrons (http://patreon.com/otfried). "
   "For the price of a cup of coffee per month you can make a meaningful contribution "
   "to the continuing development of IpePresenter and Ipe.";
