@@ -4,7 +4,7 @@
 /*
 
     This file is part of the extensible drawing editor Ipe.
-    Copyright (c) 1993-2020 Otfried Cheong
+    Copyright (c) 1993-2023 Otfried Cheong
 
     Ipe is free software; you can redistribute it and/or modify it
     under the terms of the GNU General Public License as published by
@@ -61,7 +61,7 @@ IpeAction::IpeAction(int cmd, const QString &text, const char *shortcut,
 {
   if (shortcut)
     setShortcut(QKeySequence(shortcut));
-  connect(this, &QAction::triggered, [=] () { parent->cmd(iCommand); });
+  connect(this, &QAction::triggered, [parent,this] () { parent->cmd(iCommand); });
 }
 
 // --------------------------------------------------------------------
@@ -174,7 +174,7 @@ MainWindow::MainWindow(BeamerView* bv, Qt::WindowFlags f) :
   iViewMenu->addAction(iBlackoutAction);
 
   connect(iViewMenu, &QMenu::aboutToShow,
-	  [=] () {
+	  [this] () {
 	    iShowPresentationAction->setChecked(iScreen->isVisible());
 	    iFullScreenAction->setChecked((iScreen->windowState() & Qt::WindowFullScreen) != 0);
 	    iBlackoutAction->setChecked(iScreen->pdfView()->blackout());
@@ -208,14 +208,14 @@ MainWindow::MainWindow(BeamerView* bv, Qt::WindowFlags f) :
   iHelpMenu->addAction(new IpeAction(EAbout, "About IpePresenter", nullptr, this));
 
   connect(iScreen->pdfView(), &PdfView::sizeChanged,
-	  [=] () { fitBox(mediaBox(-1), iScreen->pdfView()); });
+	  [this] () { fitBox(mediaBox(-1), iScreen->pdfView()); });
   connect(iCurrent, &PdfView::sizeChanged,
-	  [=] () { fitBox(mediaBox(-1), iCurrent); });
+	  [this] () { fitBox(mediaBox(-1), iCurrent); });
   connect(iNext, &PdfView::sizeChanged,
-	  [=] () { fitBox(mediaBox(-2), iNext); });
+	  [this] () { fitBox(mediaBox(-2), iNext); });
 
   connect(iCurrent, &PdfView::mouseButton,
-	  [=] (int button, const Vector &pos) {
+	  [this] (int button, const Vector &pos) {
 	    const PdfDict *action = findLink(pos);
 	    if (action) {
 	      interpretAction(action);
@@ -405,15 +405,13 @@ void MainWindow::closeEvent(QCloseEvent *event)
 
 static const char * const aboutText =
   "<qt><h1>IpePresenter %d.%d.%d</h1>"
-  "<p>Copyright (c) 2020 Otfried Cheong</p>"
+  "<p>Copyright (c) 2020-2023 Otfried Cheong</p>"
   "<p>A presentation tool for giving PDF presentations "
   "created in Ipe or using beamer.</p>"
   "<p>Originally invented by Dmitriy Morozov, "
   "IpePresenter is now developed together with Ipe and released under the GNU Public License.</p>"
   "<p>See the <a href=\"http://ipepresenter.otfried.org\">IpePresenter homepage</a>"
   " for further information.</p>"
-  "<p>You can \"like\" IpePresenter and follow IpePresenter announcements on "
-  "<a href=\"http://www.facebook.com/drawing.editor.Ipe7\">Facebook</a>.</p>"
   "<p>If you are an IpePresenter fan and want to show others, have a look at the "
   "<a href=\"https://www.shirtee.com/en/store/ipe\">Ipe T-shirts</a>.</p>"
   "<h3>Platinum and gold sponsors</h3>"
@@ -421,7 +419,9 @@ static const char * const aboutText =
   "<li>Günter Rote</li>"
   "<li>SCALGO</li>"
   "<li>Martin Ziegler</li></ul>"
-  "<p>If you enjoy IpePresenter, you can become a member of the exclusive community of "
+  "<p>If you enjoy IpePresenter, feel free to treat the author on a cup of coffee at "
+  "<a href=\"https://ko-fi.com/ipe7author\">Ko-fi</a>.</p>"
+  "<p>You can also become a member of the exclusive community of "
   "<a href=\"http://patreon.com/otfried\">Ipe patrons</a>. "
   "For the price of a cup of coffee per month you can make a meaningful contribution "
   "to the continuing development of IpePresenter and Ipe.</p>"
@@ -430,14 +430,14 @@ static const char * const aboutText =
 void MainWindow::aboutIpePresenter()
 {
   std::vector<char> buf(strlen(aboutText) + 100);
-  sprintf(&buf[0], aboutText,
+  sprintf(buf.data(), aboutText,
 	  IPELIB_VERSION / 10000,
 	  (IPELIB_VERSION / 100) % 100,
 	  IPELIB_VERSION % 100);
 
   QMessageBox msgBox(this);
   msgBox.setWindowTitle("About IpePresenter");
-  msgBox.setInformativeText(&buf[0]);
+  msgBox.setInformativeText(buf.data());
   msgBox.setStandardButtons(QMessageBox::Ok);
   msgBox.exec();
 }
