@@ -4,7 +4,7 @@
 /*
 
     This file is part of the extensible drawing editor Ipe.
-    Copyright (c) 1993-2020 Otfried Cheong
+    Copyright (c) 1993-2023 Otfried Cheong
 
     Ipe is free software; you can redistribute it and/or modify it
     under the terms of the GNU General Public License as published by
@@ -53,7 +53,7 @@ BOOL setWindowText(HWND h, const char *s)
 {
   int rw = MultiByteToWideChar(CP_UTF8, 0, s, -1, nullptr, 0);
   std::vector<wchar_t> ws(rw);
-  MultiByteToWideChar(CP_UTF8, 0, s, -1, &ws[0], rw);
+  MultiByteToWideChar(CP_UTF8, 0, s, -1, ws.data(), rw);
   std::vector<wchar_t> w;
   for (auto ch : ws) {
     if (ch != '\r') {
@@ -63,7 +63,7 @@ BOOL setWindowText(HWND h, const char *s)
     }
   }
   w.push_back(0);
-  return SetWindowTextW(h, &w[0]);
+  return SetWindowTextW(h, w.data());
 }
 
 // --------------------------------------------------------------------
@@ -169,7 +169,7 @@ void AppUi::initUi()
   */
 
   HMENU hSubMenu = CreatePopupMenu();
-  AppendMenuA(hSubMenu, MF_STRING, EShowPresentation, "Show &Presentation");
+  AppendMenuA(hSubMenu, MF_STRING, EShowPresentation, "Show &Presentation\tF5");
   AppendMenuA(hSubMenu, MF_STRING, EFullScreen, "Full &Screen\tF11");
   AppendMenuA(hSubMenu, MF_STRING, EBlackout, "Blackout\tB");
   AppendMenuA(hSubMenu, MF_STRING, EZoomIn, "Larger\tCtrl++");
@@ -177,7 +177,7 @@ void AppUi::initUi()
   AppendMenuA(hMenuBar, MF_STRING | MF_POPUP, UINT_PTR(hSubMenu), "&View");
 
   hSubMenu = CreatePopupMenu();
-  AppendMenuA(hSubMenu, MF_STRING, ESetTime, "&Set time");
+  AppendMenuA(hSubMenu, MF_STRING, ESetTime, "&Set time\tL");
   AppendMenuA(hSubMenu, MF_STRING, EResetTime, "&Reset time\tR");
   AppendMenuA(hSubMenu, MF_STRING, ETimeCountdown, "Count down\t/");
   AppendMenuA(hSubMenu, MF_STRING, EToggleTimeCounting, "Count time\tT");
@@ -288,10 +288,11 @@ void AppUi::handlePdfViewMessage(int param, const Vector &pos)
   // ignore all messages before PDF file loaded
   if (!iPdf)
     return;
-  // ipeDebug("handlePdfViewMessage %x", param);
-  // button press on current view
-  if ((param & 0x0f) != 0 && (param & 0xf0) == 0) {
-    const PdfDict *action = findLink(iCurrent->devToUser(pos));
+  int screen = param & 0xf0;
+  // left button press on current view or screen view
+  if ((param & 0x0f) == 1 && (screen == 0 || screen == 0x20)) {
+    PdfView *source = (screen == 0x20) ? iScreen : iCurrent;
+    const PdfDict *action = findLink(source->devToUser(pos));
     if (action) {
       interpretAction(action);
       setView();
@@ -521,14 +522,12 @@ void AppUi::browseLaunch(bool launch, String dest)
 
 static const char * const aboutText =
   "IpePresenter %d.%d.%d\n\n"
-  "Copyright (c) 2020 Otfried Cheong\n\n"
+  "Copyright (c) 2020-2023 Otfried Cheong\n\n"
   "A presentation tool for giving PDF presentations "
   "created in Ipe or using beamer.\n"
   "Originally invented by Dmitriy Morozov, "
   "IpePresenter is now developed together with Ipe and released under the GNU Public License.\n"
   "See http://ipepresenter.otfried.org for details.\n\n"
-  "You can \"like\" IpePresenter and follow IpePresenter announcements on Facebook "
-  "(http://www.facebook.com/drawing.editor.Ipe7).\n\n"
   "If you are an IpePresenter fan and want to show others, have a look at the "
   "Ipe T-shirts (www.shirtee.com/en/store/ipe).\n\n"
   "Platinum and gold sponsors\n\n"
@@ -536,7 +535,8 @@ static const char * const aboutText =
   " * Günter Rote\n"
   " * SCALGO\n"
   " * Martin Ziegler\n\n"
-  "If you enjoy IpePresenter, you can become a member of the exclusive community of "
+  "If you enjoy IpePresenter, feel free to treat the author on a cup of coffee at https://ko-fi.com/ipe7author.\n\n"
+  "You can also become a member of the exclusive community of "
   "Ipe patrons (http://patreon.com/otfried). "
   "For the price of a cup of coffee per month you can make a meaningful contribution "
   "to the continuing development of IpePresenter and Ipe.";
@@ -544,7 +544,7 @@ static const char * const aboutText =
 void AppUi::aboutIpePresenter()
 {
   std::vector<char> buf(strlen(aboutText) + 100);
-  sprintf(&buf[0], aboutText,
+  sprintf(buf.data(), aboutText,
 	  IPELIB_VERSION / 10000,
 	  (IPELIB_VERSION / 100) % 100,
 	  IPELIB_VERSION % 100);
@@ -799,6 +799,7 @@ int WINAPI WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance,
     { FVIRTKEY, 'T', AppUi::EToggleTimeCounting },
     { FVIRTKEY, 'J', AppUi::EJumpTo },
     { FVIRTKEY, 'S', AppUi::ESelectPage },
+    { FVIRTKEY, 'L', AppUi::ESetTime },
     { FVIRTKEY, 'R', AppUi::EResetTime },
     { FVIRTKEY, VK_OEM_2, AppUi::ETimeCountdown },
     { FVIRTKEY, VK_HOME, AppUi::EFirstView },
