@@ -4,7 +4,7 @@
 /*
 
     This file is part of the extensible drawing editor Ipe.
-    Copyright (c) 1993-2019 Otfried Cheong
+    Copyright (c) 1993-2023 Otfried Cheong
 
     Ipe is free software; you can redistribute it and/or modify it
     under the terms of the GNU General Public License as published by
@@ -72,6 +72,8 @@ static void usage()
 	  " -pages <n-m> : export only these pages (implies -export).\n"
 	  " -view <p-v>  : export only this view (implies -export).\n"
 	  " -markedview  : export only marked views on marked pages (implies -export).\n"
+	  "     If all views of a marked page are unmarked, the last view is exported.\n"
+	  "     This is convenient to make handouts for slides.\n"
 	  " -runlatex    : run Latex even for XML output.\n"
 	  " -nozip       : do not compress PDF streams.\n"
 	  " -keepnotes   : save page notes as PDF annotations even when exporting.\n"
