@@ -5,7 +5,7 @@
 /*
 
     This file is part of the extensible drawing editor Ipe.
-    Copyright (c) 1993-2020 Otfried Cheong
+    Copyright (c) 1993-2023 Otfried Cheong
 
     Ipe is free software; you can redistribute it and/or modify it
     under the terms of the GNU General Public License as published by
@@ -29,18 +29,12 @@
 
 */
 
-#ifdef IPESTRICT
-#include "../include/ipeosx.h"
-#endif
-
 #include "ipeui_common.h"
 
 #import <Cocoa/Cocoa.h>
 
 #include "ipeuilayout_cocoa.h"
 
-#include <thread>
-
 #define COLORICONSIZE 12
 
 inline const char *N2C(NSString *aStr) { return aStr.UTF8String; }
@@ -72,12 +66,9 @@ void addToLayout(NSView *view, NSView *subview)
 id layoutGuide(NSView *owner)
 {
 #if MAC_OS_X_VERSION_MAX_ALLOWED >= 101100
-  if ([owner respondsToSelector:@selector(addLayoutGuide:)]) {
-#pragma clang diagnostic push
-#pragma clang diagnostic ignored "-Wdeprecated-declarations"
+  if (@available(macOS 10.11, *)) {
     NSLayoutGuide *g = [[NSLayoutGuide alloc] init];
     [owner addLayoutGuide:g];
-#pragma clang diagnostic pop
     return g;
   }
 #endif
@@ -180,7 +171,7 @@ NSString *ipeui_set_mnemonic(NSString *title, NSButton *button)
 
 class PDialog : public Dialog {
 public:
-  PDialog(lua_State *L0, WINID parent, const char *caption);
+  PDialog(lua_State *L0, WINID parent, const char *caption, const char *language);
   virtual ~PDialog();
 
   void itemAction(int idx);
@@ -256,8 +247,8 @@ objectValueForTableColumn:(NSTableColumn *) col
 
 // --------------------------------------------------------------------
 
-PDialog::PDialog(lua_State *L0, WINID parent, const char *caption)
-  : Dialog(L0, parent, caption)
+PDialog::PDialog(lua_State *L0, WINID parent, const char *caption, const char * language)
+: Dialog(L0, parent, caption, language)
 {
   //
 }
@@ -674,27 +665,60 @@ static int dialog_constructor(lua_State *L)
 {
   WINID parent = check_winid(L, 1);
   const char *s = luaL_checkstring(L, 2);
+  const char *language = "";
+  if (lua_isstring(L, 3))
+    language = luaL_checkstring(L, 3);
 
   Dialog **dlg = (Dialog **) lua_newuserdata(L, sizeof(Dialog *));
   *dlg = nullptr;
   luaL_getmetatable(L, "Ipe.dialog");
   lua_setmetatable(L, -2);
-  *dlg = new PDialog(L, parent, s);
+  *dlg = new PDialog(L, parent, s, language);
   return 1;
 }
 
 // --------------------------------------------------------------------
 
-void editor_thread(const char *cmd)
+@interface IpePanelDelegate : NSObject <NSWindowDelegate>
+@property void (^threadFunction)();
+@property BOOL threadStarted;
+@end
+
+@implementation IpePanelDelegate
+- (instancetype) init
 {
-  int result = std::system(cmd);
-  (void) result;
-  [NSApp abortModal];
+  self = [super init];
+  if (self) {
+    _threadStarted = false;
+  }
+  return self;
 }
 
+- (void) windowDidBecomeKey:(NSNotification *) notification
+{
+  if (!self.threadStarted && self.threadFunction) {
+    [NSThread detachNewThreadSelector:@selector(startThread:)
+			     toTarget:self
+			   withObject:nil];
+    self.threadStarted = true;
+  }
+}
+
+- (void) startThread:(id) arg
+{
+  self.threadFunction();
+}
+@end
+
 static int ipeui_wait(lua_State *L)
 {
-  const char *cmd = luaL_checkstring(L, 2);
+  const char *cmd = nullptr;
+  if (!lua_isfunction(L, 2)) {
+    cmd = luaL_checkstring(L, 2);
+  }
+  const char *text = "Waiting for external editor";
+  if (lua_isstring(L, 3))
+    text = lua_tolstring(L, 3, nullptr);
 
   NSPanel *panel = [[NSPanel alloc]
 		     initWithContentRect:NSMakeRect(400.,800., 200, 100)
@@ -704,15 +728,27 @@ static int ipeui_wait(lua_State *L)
   panel.title = @"Ipe: waiting";
   NSTextField *l = [[NSTextField alloc]
 		     initWithFrame:NSMakeRect(0., 0., 200., 100.)];
-  l.stringValue = @"Waiting for external editor";
+  l.stringValue = C2N(text);
   l.editable = NO;
   l.bordered = NO;
   l.drawsBackground = NO;
-  panel.contentView = l;
-  std::thread t(editor_thread, cmd);
+
+  addToLayout(panel.contentView, l);
+  layout(panel.contentView, l, "x=x");
+  layout(panel.contentView, l, "y=y");
+
+  auto delegate = [[IpePanelDelegate alloc] init];
+  panel.delegate = delegate;
+
+  if (cmd) {
+    delegate.threadFunction = ^() { (void) std::system(cmd); [NSApp abortModal]; };
+  } else {
+    lua_pushvalue(L, 2);
+    delegate.threadFunction = ^() { lua_call(L, 0, 0); [NSApp abortModal];};
+  }
+
   [NSApp runModalForWindow:panel];
   [panel close];
-  t.join();
   return 0;
 }
 
