@@ -4,7 +4,7 @@
 /*
 
     This file is part of the extensible drawing editor Ipe.
-    Copyright (c) 1993-2019 Otfried Cheong
+    Copyright (c) 1993-2023 Otfried Cheong
 
     Ipe is free software; you can redistribute it and/or modify it
     under the terms of the GNU General Public License as published by
@@ -63,14 +63,14 @@ static const struct luaL_Reg winid_methods[] = {
 
 // --------------------------------------------------------------------
 
-Dialog::Dialog(lua_State *L0, WINID parent, const char *caption)
-  : iCaption(caption)
+Dialog::Dialog(lua_State *L0, WINID parent, const char *caption, const char *language)
+  : iCaption(caption), iLanguage(language)
 {
   L = L0;
   iParent = parent;
   iLuaDialog = LUA_NOREF;
   hDialog = nullptr;
-  iIgnoreEscape = false;
+  iIgnoreEscapeField = -1;
   iNoRows = 1;
   iNoCols = 1;
 }
@@ -350,7 +350,8 @@ int Dialog::set(lua_State *L)
 {
   auto s = checkstring(L, 2);
   if (s == "ignore-escape") {
-    iIgnoreEscape = lua_toboolean(L, 3);
+    iIgnoreEscapeField = findElement(L, 3);
+    iIgnoreEscapeText = checkstring(L, 4);
     return 0;
   }
 
