@@ -4,7 +4,7 @@
 /*
 
     This file is part of the extensible drawing editor Ipe.
-    Copyright (c) 1993-2019 Otfried Cheong
+    Copyright (c) 1993-2023 Otfried Cheong
 
     Ipe is free software; you can redistribute it and/or modify it
     under the terms of the GNU General Public License as published by
@@ -85,7 +85,7 @@ inline void luacall(lua_State *L, int nargs, int nresults) {
 
 class Dialog {
 public:
-  Dialog(lua_State *L0, WINID parent, const char *caption);
+  Dialog(lua_State *L0, WINID parent, const char *caption, const char *language);
   virtual ~Dialog();
 
   // Lua methods
@@ -151,7 +151,10 @@ protected:
   std::string iCaption;
   std::vector<SElement> iElements;
   int iLuaDialog;
-  bool iIgnoreEscape;
+  int iIgnoreEscapeField;
+  std::string iIgnoreEscapeText;
+  std::string iLanguage;
+
   int iBaseX, iBaseY;
   int iNoRows, iNoCols;
   std::vector<int> iRowStretch;
