@@ -4,7 +4,7 @@
 /*
 
     This file is part of the extensible drawing editor Ipe.
-    Copyright (c) 1993-2020 Otfried Cheong
+    Copyright (c) 1993-2023 Otfried Cheong
 
     Ipe is free software; you can redistribute it and/or modify it
     under the terms of the GNU General Public License as published by
@@ -46,12 +46,21 @@
 #include <QMenu>
 #include <QMessageBox>
 #include <QPushButton>
+#include <QRegularExpression>
 #include <QShortcut>
 #include <QSyntaxHighlighter>
 #include <QTextEdit>
+#include <QTextStream>
 #include <QThread>
 #include <QTimer>
 
+#ifdef IPE_SPELLCHECK
+#pragma GCC diagnostic push
+#pragma GCC diagnostic ignored "-Wzero-as-null-pointer-constant"
+#include <QtSpell.hpp>
+#pragma GCC diagnostic pop
+#endif
+
 // --------------------------------------------------------------------
 
 inline void push_string(lua_State *L, const QString &str)
@@ -71,38 +80,56 @@ inline QString checkqstring(lua_State *L, int i)
 
 // --------------------------------------------------------------------
 
+#ifdef IPE_SPELLCHECK
+class TextEdit : public QTextEdit {
+public:
+  TextEdit(QWidget *parent, const std::string &language) : QTextEdit(parent) {
+    checker.setTextEdit(this);
+    if (!language.empty())
+      checker.setLanguage(QString::fromUtf8(language.c_str()));
+  }
+
+private:
+  QtSpell::TextEditChecker checker;
+};
+#endif
+
+// --------------------------------------------------------------------
+
 class XmlHighlighter : public QSyntaxHighlighter
 {
 public:
   XmlHighlighter(QTextEdit *textEdit);
 protected:
-  void applyFormat(const QString &text, QRegExp &exp,
+  void applyFormat(const QString &text, QRegularExpression &exp,
 		   const QTextCharFormat &format);
   virtual void highlightBlock(const QString &text);
 };
 
-void XmlHighlighter::applyFormat(const QString &text, QRegExp &exp,
+void XmlHighlighter::applyFormat(const QString &text, QRegularExpression &exp,
 				 const QTextCharFormat &format)
 {
-  int index = text.indexOf(exp);
+  QRegularExpressionMatch match;
+  int index = text.indexOf(exp, 0, &match);
   while (index >= 0) {
-    int length = exp.matchedLength();
+    int length = match.capturedLength(0);
     setFormat(index, length, format);
-    index = text.indexOf(exp, index + length);
+    index = text.indexOf(exp, index + length, &match);
   }
 }
 
 void XmlHighlighter::highlightBlock(const QString &text)
 {
+  bool dark = QGuiApplication::palette().text().color().value() > 128;
   QTextCharFormat tagFormat, stringFormat, numberFormat;
   tagFormat.setFontWeight(QFont::Bold);
-  tagFormat.setForeground(Qt::blue);
-  stringFormat.setForeground(Qt::darkMagenta);
+  tagFormat.setForeground(dark ? Qt::yellow : Qt::blue);
+  stringFormat.setForeground(dark ? Qt::cyan : Qt::darkMagenta);
   numberFormat.setForeground(Qt::red);
 
-  QRegExp tagExp( "<.*>" );
-  QRegExp stringExp( "\"[a-zA-Z]*\"" );
-  QRegExp numberExp( "[+|-]*[0-9]*.[0-9][0-9]*" );
+  QRegularExpression tagExp( "<.*>" );
+  QRegularExpression stringExp( "\"[a-zA-Z]*\"" );
+  QRegularExpression numberExp( "[+|-]*[0-9]*.[0-9][0-9]*" );
   applyFormat(text, tagExp, tagFormat);
   applyFormat(text, stringExp, stringFormat);
   applyFormat(text, numberExp, numberFormat);
@@ -121,31 +148,33 @@ class LatexHighlighter : public QSyntaxHighlighter
 public:
   LatexHighlighter(QTextEdit *textEdit);
 protected:
-  void applyFormat(const QString &text, QRegExp &exp,
+  void applyFormat(const QString &text, QRegularExpression &exp,
 		   const QTextCharFormat &format);
   virtual void highlightBlock(const QString &text);
 };
 
-void LatexHighlighter::applyFormat(const QString &text, QRegExp &exp,
+void LatexHighlighter::applyFormat(const QString &text, QRegularExpression &exp,
 				   const QTextCharFormat &format)
 {
-  int index = text.indexOf(exp);
+  QRegularExpressionMatch match;
+  int index = text.indexOf(exp, 0, &match);
   while (index >= 0) {
-    int length = exp.matchedLength();
+    int length = match.capturedLength(0);
     setFormat(index, length, format);
-    index = text.indexOf(exp, index + length);
+    index = text.indexOf(exp, index + length, &match);
   }
 }
 
 void LatexHighlighter::highlightBlock(const QString &text)
 {
+  bool dark = QGuiApplication::palette().text().color().value() > 128;
   QTextCharFormat mathFormat, tagFormat;
-  mathFormat.setForeground(Qt::red);
+  mathFormat.setForeground(dark ? Qt::cyan : Qt::red);
   tagFormat.setFontWeight(QFont::Bold);
-  tagFormat.setForeground(Qt::blue);
+  tagFormat.setForeground(dark ? Qt::yellow : Qt::blue);
 
-  QRegExp mathExp( "\\$[^$]+\\$" );
-  QRegExp tagExp( "\\\\[a-zA-Z]+" );
+  QRegularExpression mathExp( "\\$[^$]+\\$" );
+  QRegularExpression tagExp( "\\\\[a-zA-Z]+" );
   applyFormat(text, mathExp, mathFormat);
   applyFormat(text, tagExp, tagFormat);
 }
@@ -158,8 +187,8 @@ LatexHighlighter::LatexHighlighter(QTextEdit *textEdit)
 
 // --------------------------------------------------------------------
 
-PDialog::PDialog(lua_State *L0, WINID parent, const char *caption)
-  : QDialog(parent), Dialog(L0, parent, caption)
+PDialog::PDialog(lua_State *L0, WINID parent, const char *caption, const char *language)
+  : QDialog(parent), Dialog(L0, parent, caption, language)
 {
   setWindowTitle(caption);
   QVBoxLayout *vlo = new QVBoxLayout;
@@ -211,10 +240,10 @@ static void markupLog(QTextEdit *t, const QString &text)
     cursor.setPosition(pos, QTextCursor::KeepAnchor);
     ++errNo;
     QString s;
-    s.sprintf("err%d", errNo);
+    QTextStream(&s) << "err" << errNo;
     QTextCharFormat format;
     format.setBackground(Qt::yellow);
-    format.setAnchorName(s);
+    format.setAnchorNames(QStringList(s));
     format.setAnchor(true);
     cursor.setCharFormat(format);
   }
@@ -320,7 +349,12 @@ bool PDialog::buildAndRun(int w, int h)
 	break;
       case ETextEdit:
 	{
+#ifdef IPE_SPELLCHECK
+	  QTextEdit *t = (m.flags & ELogFile) ?
+	    new QTextEdit(this) : new TextEdit(this, this->iLanguage);
+#else
 	  QTextEdit *t = new QTextEdit(this);
+#endif
 	  t->setAcceptRichText(false);
 	  if (m.flags & EReadOnly)
 	    t->setReadOnly(true);
@@ -346,7 +380,9 @@ bool PDialog::buildAndRun(int w, int h)
 	  b->setCurrentIndex(m.value);
 	  if (m.lua_method != LUA_NOREF) {
 	    connect<void (QComboBox::*)(int)>(b, &QComboBox::activated,
-					      [=](int index){ callLua(m.lua_method); });
+					      [this,method=m.lua_method](int index){
+						callLua(method);
+					      });
 	  }
 	  w = b;
 	}
@@ -358,7 +394,7 @@ bool PDialog::buildAndRun(int w, int h)
 	    l->addItem(QString::fromUtf8(m.items[k].c_str()));
 	  if (m.lua_method != LUA_NOREF) {
 	    connect(l, &QListWidget::itemActivated,
-		    [=](QListWidgetItem *){ callLua(m.lua_method); });
+		    [this,method=m.lua_method](QListWidgetItem *){ callLua(method); });
 	  }
 	  w = l;
 	}
@@ -428,12 +464,15 @@ static int dialog_constructor(lua_State *L)
 {
   QWidget *parent = check_winid(L, 1);
   const char *s = luaL_checkstring(L, 2);
+  const char *language = "";
+  if (lua_isstring(L, 3))
+    language = luaL_checkstring(L, 3);
 
   Dialog **dlg = (Dialog **) lua_newuserdata(L, sizeof(Dialog *));
   *dlg = nullptr;
   luaL_getmetatable(L, "Ipe.dialog");
   lua_setmetatable(L, -2);
-  *dlg = new PDialog(L, parent, s);
+  *dlg = new PDialog(L, parent, s, language);
   return 1;
 }
 
@@ -575,59 +614,98 @@ static int menu_constructor(lua_State *L)
 class EditorThread : public QThread
 {
 public:
-  EditorThread(const QString &cmd);
+  EditorThread(lua_State *L, const QString &cmd);
 
 protected:
   virtual void run();
 private:
+  lua_State *L;
   QString iCommand;
 };
 
-EditorThread::EditorThread(const QString &cmd) : QThread()
+EditorThread::EditorThread(lua_State *L, const QString &cmd)
+  : QThread(), L{L}, iCommand{cmd}
 {
-  iCommand = cmd;
+  // nothing
 }
 
 void EditorThread::run()
 {
-  int result = std::system(iCommand.toUtf8());
-  (void) result;
+  if (iCommand.isEmpty()) {
+    lua_call(L, 0, 0);
+  } else {
+    // give wait dialog a chance to show up before we pass control to the editor
+    // this helps to make sure the editor has keyboard focus when it starts
+    QThread::msleep(100);
+    int result = std::system(iCommand.toUtf8());
+    (void) result;
+  }
 }
 
 class EditorDialog : public QDialog
 {
 public:
-  EditorDialog(QWidget *parent = nullptr);
+  EditorDialog(QString label, QWidget *parent = nullptr);
 protected:
+  void keyPressEvent(QKeyEvent *e);
   void closeEvent(QCloseEvent *ev);
 };
 
-EditorDialog::EditorDialog(QWidget *parent)
+EditorDialog::EditorDialog(QString label, QWidget *parent)
   : QDialog(parent)
 {
   QGridLayout *lo = new QGridLayout;
   setLayout(lo);
   setWindowTitle("Ipe: waiting");
-  QLabel *l = new QLabel("Waiting for external editor", this);
+  QLabel *l = new QLabel(label, this);
   lo->addWidget(l, 0, 0);
 }
 
+void EditorDialog::keyPressEvent(QKeyEvent *e)
+{
+  // do not let Escape close the dialog
+  if (e->key() != Qt::Key_Escape)
+    QDialog::keyPressEvent(e);
+}
+
 void EditorDialog::closeEvent(QCloseEvent *ev)
 {
+  // do not let user close the dialog
   ev->ignore();
 }
 
 static int ipeui_wait(lua_State *L)
 {
-  QString cmd = checkqstring(L, 2);
+  PDialog *parent = nullptr;
+  Dialog **dlg = (Dialog **) luaL_testudata(L, 1, "Ipe.dialog");
+  if (dlg != nullptr)
+    parent = (PDialog *) *dlg;
+  QString cmd;
+  if (!lua_isfunction(L, 2))
+    cmd = checkqstring(L, 2);
+  QString label = "Waiting for external editor";
+  if (lua_isstring(L, 3))
+    label = checkqstring(L, 3);
 
-  EditorDialog *dialog = new EditorDialog();
+  EditorThread *thread = new EditorThread(L, cmd);
+  EditorDialog *dialog = new EditorDialog(label);
 
-  EditorThread *thread = new EditorThread(cmd);
-  dialog->connect(thread, &QThread::finished, dialog, &QDialog::accept);
+  if (cmd.isEmpty()) {
+    // call Lua function
+    lua_pushvalue(L, 2);
+    thread->start();
+    if (!thread->wait(300)) {
+      dialog->connect(thread, &QThread::finished, dialog, &QDialog::accept);
+      dialog->exec();
+    }
+  } else {
+    dialog->connect(thread, &QThread::finished, dialog, &QDialog::accept);
+    thread->start();
+    dialog->exec();
+    if (parent)
+      parent->activateWindow();
+  }
 
-  thread->start();
-  dialog->exec();
   delete dialog;
   delete thread;
   return 0;
@@ -710,11 +788,7 @@ static int ipeui_getColor(lua_State *L)
   QColor initial = QColor::fromRgbF(luaL_checknumber(L, 3),
 				    luaL_checknumber(L, 4),
 				    luaL_checknumber(L, 5));
-#if QT_VERSION >= 0x040500
   QColor changed = QColorDialog::getColor(initial, parent, title);
-#else
-  QColor changed = QColorDialog::getColor(initial, parent);
-#endif
   if (changed.isValid()) {
     lua_pushnumber(L, changed.redF());
     lua_pushnumber(L, changed.greenF());
@@ -757,7 +831,8 @@ static int ipeui_fileDialog(lua_State *L)
   QFileDialog dialog(parent);
   dialog.setWindowTitle(caption);
   dialog.setNameFilters(filters);
-  dialog.setConfirmOverwrite(false);
+  dialog.setOption(QFileDialog::DontConfirmOverwrite, true);
+  // dialog.setConfirmOverwrite(false);
 
   if (type == 0) {
     dialog.setFileMode(QFileDialog::ExistingFile);
@@ -880,10 +955,9 @@ static int ipeui_messageBox(lua_State *L)
 static int ipeui_currentDateTime(lua_State *L)
 {
   QDateTime dt = QDateTime::currentDateTime();
-  QString mod;
-  mod.sprintf("%04d%02d%02d%02d%02d%02d",
-	      dt.date().year(), dt.date().month(), dt.date().day(),
-	      dt.time().hour(), dt.time().minute(), dt.time().second());
+  QString mod = QString::asprintf("%04d%02d%02d%02d%02d%02d",
+				  dt.date().year(), dt.date().month(), dt.date().day(),
+				  dt.time().hour(), dt.time().minute(), dt.time().second());
   push_string(L, mod);
   return 1;
 }
