@@ -4,7 +4,7 @@
 /*
 
     This file is part of the extensible drawing editor Ipe.
-    Copyright (c) 1993-2020 Otfried Cheong
+    Copyright (c) 1993-2023 Otfried Cheong
 
     Ipe is free software; you can redistribute it and/or modify it
     under the terms of the GNU General Public License as published by
@@ -64,7 +64,7 @@ BOOL setWindowText(HWND h, const char *s)
     }
   }
   w.push_back(0);
-  return SetWindowTextW(h, &w[0]);
+  return SetWindowTextW(h, w.data());
 }
 
 void sendMessage(HWND h, UINT code, const char *t, WPARAM wParam)
@@ -77,8 +77,8 @@ static std::string wideToUtf8(const wchar_t *wbuf)
 {
   int rm = WideCharToMultiByte(CP_UTF8, 0, wbuf, -1, nullptr, 0, nullptr, nullptr);
   std::vector<char> multi(rm);
-  WideCharToMultiByte(CP_UTF8, 0, wbuf, -1, &multi[0], rm, nullptr, nullptr);
-  return std::string(&multi[0]);
+  WideCharToMultiByte(CP_UTF8, 0, wbuf, -1, multi.data(), rm, nullptr, nullptr);
+  return std::string(multi.data());
 }
 
 static void buildFlags(std::vector<short> &t, DWORD flags)
@@ -118,7 +118,7 @@ static void buildControl(std::vector<short> &t, short what, const char *s = null
 
 class PDialog : public Dialog {
 public:
-  PDialog(lua_State *L0, WINID parent, const char *caption);
+  PDialog(lua_State *L0, WINID parent, const char *caption, const char *language);
   virtual ~PDialog();
 
 protected:
@@ -146,8 +146,9 @@ private:
   std::vector<int> iColWidth;
 };
 
-PDialog::PDialog(lua_State *L0, WINID parent, const char *caption)
-  : Dialog(L0, parent, caption)
+PDialog::PDialog(lua_State *L0, WINID parent, const char *caption,
+		 const char * language)
+  : Dialog(L0, parent, caption, language)
 {
   LONG base = GetDialogBaseUnits();
   iBaseX = LOWORD(base);
@@ -275,7 +276,7 @@ static std::string getEditText(HWND h)
       w.push_back(ch);
   }
   w.push_back(0);
-  return wideToUtf8(&w[0]);
+  return wideToUtf8(w.data());
 }
 
 void PDialog::retrieveValues()
@@ -535,7 +536,7 @@ bool PDialog::buildAndRun(int w, int h)
   int res =
     DialogBoxIndirectParamW((HINSTANCE) GetWindowLongPtr(iParent,
 							 GWLP_HINSTANCE),
-			    (LPCDLGTEMPLATE) &t[0],
+			    (LPCDLGTEMPLATE) t.data(),
 			    iParent, (DLGPROC) dialogProc, (LPARAM) this);
   // retrieveValues() has been called before EndDialog!
   hDialog = nullptr; // already destroyed by Windows
@@ -673,12 +674,15 @@ static int dialog_constructor(lua_State *L)
 {
   HWND parent = check_winid(L, 1);
   const char *s = luaL_checklstring(L, 2, nullptr);
+  const char *language = "";
+  if (lua_isstring(L, 3))
+    language = luaL_checkstring(L, 3);
 
   Dialog **dlg = (Dialog **) lua_newuserdata(L, sizeof(Dialog *));
   *dlg = nullptr;
   luaL_getmetatable(L, "Ipe.dialog");
   lua_setmetatable(L, -2);
-  *dlg = new PDialog(L, parent, s);
+  *dlg = new PDialog(L, parent, s, language);
   return 1;
 }
 
@@ -1040,7 +1044,7 @@ static int ipeui_fileDialog(lua_State *L)
   ofn.lStructSize = sizeof(ofn);
   ofn.hwndOwner = hwnd;
 
-  ofn.lpstrFilter = &filters[0];
+  ofn.lpstrFilter = filters.data();
   ofn.nFilterIndex = selected;
 
   ofn.lpstrFile = szFileName;
@@ -1163,13 +1167,24 @@ static int ipeui_messageBox(lua_State *L)
 
 // --------------------------------------------------------------------
 
+struct SDialogHandle {
+  HWND hwnd;
+  HANDLE thread;
+};
+
 BOOL CALLBACK waitDialogProc(HWND hwnd, UINT message,
 			     WPARAM wParam, LPARAM lParam)
 {
   switch (message) {
   case WM_INITDIALOG: {
-    HWND *p = (HWND *) lParam;
-    *p = hwnd;
+    SDialogHandle *d = (SDialogHandle *) lParam;
+    d->hwnd = hwnd;
+    if (d->thread) {
+      ResumeThread(d->thread);
+      // delay showing the dialog by 300ms
+      // if Latex is fast, the dialog never gets shown
+      Sleep(300);
+    }
     return TRUE; }
   default:
     return FALSE;
@@ -1178,15 +1193,29 @@ BOOL CALLBACK waitDialogProc(HWND hwnd, UINT message,
 
 VOID CALLBACK waitCallback(PVOID lpParameter, BOOLEAN timerOrWaitFired)
 {
-  HWND *hDialog = (HWND *) lpParameter;
-  if (*hDialog)
-    EndDialog(*hDialog, 1);
+  SDialogHandle *d = (SDialogHandle *) lpParameter;
+  EndDialog(d->hwnd, 1);
+}
+
+DWORD CALLBACK waitLuaThreadProc(LPVOID lpParameter)
+{
+  lua_State *L = (lua_State *) lpParameter;
+  lua_callk(L, 0, 0, 0, nullptr);
+  return 0;
 }
 
 static int ipeui_wait(lua_State *L)
 {
-  Dialog **dlg = check_dialog(L, 1);
-  HWND parent = (*dlg)->winId();
+  Dialog **dlg = (Dialog **) luaL_testudata(L, 1, "Ipe.dialog");
+  HWND parent = dlg ? (*dlg)->winId() : check_winid(L, 1);
+
+  const char *cmd = nullptr;
+  if (!lua_isfunction(L, 2))
+    cmd = luaL_checklstring(L, 2, nullptr);
+
+  const char *label = "Waiting for external editor";
+  if (lua_isstring(L, 3))
+    label = luaL_checklstring(L, 3, nullptr);
 
   std::vector<short> t;
 
@@ -1215,44 +1244,61 @@ static int ipeui_wait(lua_State *L)
   t.push_back(120);
   t.push_back(20);
   t.push_back(IDBASE);
-  buildControl(t, 0x0082, "Waiting for external editor");
+  buildControl(t, 0x0082, label);
 
-  // Declare and initialize process blocks
-  PROCESS_INFORMATION processInformation;
-  STARTUPINFOW startupInfo;
+  SDialogHandle dialogHandle = { nullptr, nullptr };
 
-  memset(&processInformation, 0, sizeof(processInformation));
-  memset(&startupInfo, 0, sizeof(startupInfo));
-  startupInfo.cb = sizeof(startupInfo);
-
-  // Call the executable program
-  const char *cmd = luaL_checklstring(L, 2, nullptr);
-  WString wcmd(cmd);
-
-  int result = CreateProcessW(nullptr, wcmd.data(), nullptr, nullptr, FALSE,
-			      NORMAL_PRIORITY_CLASS|CREATE_NO_WINDOW,
-			      nullptr, nullptr, &startupInfo, &processInformation);
-  if (result == 0)
-    return 0;
-
-  HWND dialogHandle = nullptr;
   HANDLE waitHandle;
 
-  RegisterWaitForSingleObject(&waitHandle,
-			      processInformation.hProcess,
-			      waitCallback,
-			      &dialogHandle,
-			      INFINITE,
-			      WT_EXECUTEINWAITTHREAD|WT_EXECUTEONLYONCE);
+  if (cmd) {
+    // call external process
 
-  DialogBoxIndirectParamW((HINSTANCE) GetWindowLongPtr(parent, GWLP_HINSTANCE),
-			  (LPCDLGTEMPLATE) &t[0],
-			  parent, (DLGPROC) waitDialogProc,
-			  (LPARAM) &dialogHandle);
+    // Declare and initialize process blocks
+    PROCESS_INFORMATION processInformation;
+    STARTUPINFOW startupInfo;
+    memset(&processInformation, 0, sizeof(processInformation));
+    memset(&startupInfo, 0, sizeof(startupInfo));
+    startupInfo.cb = sizeof(startupInfo);
 
-  UnregisterWait(waitHandle);
-  CloseHandle(processInformation.hProcess);
-  CloseHandle(processInformation.hThread);
+    WString wcmd(cmd);
+
+    int result = CreateProcessW(nullptr, wcmd.data(), nullptr, nullptr, FALSE,
+				NORMAL_PRIORITY_CLASS|CREATE_NO_WINDOW,
+				nullptr, nullptr, &startupInfo, &processInformation);
+    if (result == 0)
+      return 0;
+
+    RegisterWaitForSingleObject(&waitHandle, processInformation.hProcess, waitCallback, &dialogHandle,
+				INFINITE, WT_EXECUTEINWAITTHREAD|WT_EXECUTEONLYONCE);
+
+    DialogBoxIndirectParamW((HINSTANCE) GetWindowLongPtr(parent, GWLP_HINSTANCE),
+			    (LPCDLGTEMPLATE) t.data(),
+			    parent, (DLGPROC) waitDialogProc,
+			    (LPARAM) &dialogHandle);
+
+    UnregisterWait(waitHandle);
+    CloseHandle(processInformation.hProcess);
+    CloseHandle(processInformation.hThread);
+
+  } else {
+    // call Lua function
+
+    dialogHandle.thread = CreateThread(nullptr, 0, &waitLuaThreadProc, L, CREATE_SUSPENDED, nullptr);
+    if (dialogHandle.thread == nullptr)
+      return 0;
+
+    RegisterWaitForSingleObject(&waitHandle, dialogHandle.thread, waitCallback, &dialogHandle,
+				INFINITE, WT_EXECUTEINWAITTHREAD|WT_EXECUTEONLYONCE);
+
+    lua_pushvalue(L, 2);
+
+    DialogBoxIndirectParamW((HINSTANCE) GetWindowLongPtr(parent, GWLP_HINSTANCE),
+			    (LPCDLGTEMPLATE) t.data(),
+			    parent, (DLGPROC) waitDialogProc,
+			    (LPARAM) &dialogHandle);
+    UnregisterWait(waitHandle);
+    CloseHandle(dialogHandle.thread);
+  }
   return 0;
 }
 
